@@ -19,7 +19,7 @@
 --
 -- >>> prove $ \x -> x `shiftL` 2 .== 2 * (x :: SWord8)
 -- Falsifiable. Counter-example:
---   s0 = 32 :: Word8
+--   s0 = 64 :: Word8
 --
 -- The function 'prove' has the following type:
 --
@@ -46,13 +46,18 @@
 --
 --   * 'SInteger': Unbounded signed integers.
 --
---   * 'SReal': Algebraic-real numbers
+--   * 'SReal': Algebraic-real numbers.
 --
---   * 'SFloat': IEEE-754 single-precision floating point values
+--   * 'SFloat': IEEE-754 single-precision floating point values.
 --
---   * 'SDouble': IEEE-754 double-precision floating point values
+--   * 'SDouble': IEEE-754 double-precision floating point values.
 --
---   * 'SChar', 'SString', 'RegExp': Characters, strings and regular expressions
+--   * 'SRational': Rationals. (Ratio of two symbolic integers.)
+--
+--   * 'SFloatingPoint': Generalized IEEE-754 floating point values, with user specified exponent and
+--   mantissa widths.
+--
+--   * 'SChar', 'SString', 'RegExp': Characters, strings and regular expressions.
 --
 --   * 'SList': Symbolic lists (which can be nested)
 --
@@ -60,11 +65,11 @@
 --
 --   * 'SEither': Symbolic sums
 --
---   * 'SMaybe': Symbolic optional values
+--   * 'SMaybe': Symbolic optional values.
 --
---   * 'SSet': Symbolic sets
+--   * 'SSet': Symbolic sets.
 --
---   * 'SArray', 'SFunArray': Flat arrays of symbolic values.
+--   * 'SArray': Arrays of symbolic values.
 --
 --   * Symbolic polynomials over GF(2^n), polynomial arithmetic, and CRCs.
 --
@@ -108,14 +113,17 @@
 --
 --   * ABC from University of Berkeley: <http://www.eecs.berkeley.edu/~alanmi/abc/>
 --
---   * CVC4 from Stanford: <https://cvc4.github.io/>
+--   * CVC4, and CVC5 from Stanford University and the University of Iowa. <https://cvc4.github.io/> and <https://cvc5.github.io>
 --
---   * Boolector from Johannes Kepler University: <http://fmv.jku.at/boolector/>
+--   * Boolector from Johannes Kepler University: <http://fmv.jku.at/boolector/> and its successor Bitwuzla from Stanford
+--     university: <https://github.com/bitwuzla/bitwuzla>
 --
 --   * MathSAT from Fondazione Bruno Kessler and DISI-University of Trento: <http://mathsat.fbk.eu/>
 --
 --   * Yices from SRI: <http://yices.csl.sri.com/>
 --
+--   * DReal from CMU: <http://dreal.github.io/>
+--
 --   * Z3 from Microsoft: <http://github.com/Z3Prover/z3/wiki>
 --
 -- SBV requires recent versions of these solvers; please see the file
@@ -147,16 +155,24 @@ module Data.SBV (
   -- *** Signed bit-vectors
   , SInt8, SInt16, SInt32, SInt64, SInt, IntN
   -- *** Converting between fixed-size and arbitrary bitvectors
-  , IsNonZero, FromSized, ToSized, fromSized, toSized
+  , BVIsNonZero, FromSized, ToSized, fromSized, toSized
   -- ** Unbounded integers
   -- $unboundedLimitations
   , SInteger
   -- ** Floating point numbers
   -- $floatingPoints
-  , SFloat, SDouble
+  , ValidFloat, SFloat, SDouble
+  , SFloatingPoint, FloatingPoint
+  , SFPHalf, FPHalf
+  , SFPBFloat, FPBFloat
+  , SFPSingle, FPSingle
+  , SFPDouble, FPDouble
+  , SFPQuad, FPQuad
+  -- ** Rationals
+  , SRational
   -- ** Algebraic reals
   -- $algReals
-  , SReal, AlgReal, sRealToSInteger, algRealToRational
+  , SReal, AlgReal(..), sRealToSInteger, algRealToRational, RealPoint(..), realPoint, RationalCV(..)
   -- ** Characters, Strings and Regular Expressions
   -- $strings
   , SChar, SString
@@ -171,7 +187,7 @@ module Data.SBV (
   -- ** Sets
   , RCSet(..), SSet
   -- * Arrays of symbolic values
-  , SymArray(readArray, writeArray, mergeArrays), newArray_, newArray, SArray, SFunArray
+  , SymArray(readArray, writeArray, mergeArrays, sListArray), newArray_, newArray, SArray
 
   -- * Creating symbolic values
   -- ** Single value
@@ -181,8 +197,15 @@ module Data.SBV (
   , sInt8,  sInt8_,  sInt16,  sInt16_,  sInt32,  sInt32_,  sInt64,  sInt64_, sInt, sInt_
   , sInteger, sInteger_
   , sReal, sReal_
+  , sRational, sRational_
   , sFloat, sFloat_
   , sDouble, sDouble_
+  , sFloatingPoint, sFloatingPoint_
+  , sFPHalf, sFPHalf_
+  , sFPBFloat, sFPBFloat_
+  , sFPSingle, sFPSingle_
+  , sFPDouble, sFPDouble_
+  , sFPQuad, sFPQuad_
   , sChar, sChar_
   , sString, sString_
   , sList, sList_
@@ -198,8 +221,15 @@ module Data.SBV (
   , sInt8s,  sInt16s,  sInt32s,  sInt64s, sInts
   , sIntegers
   , sReals
+  , sRationals
   , sFloats
   , sDoubles
+  , sFloatingPoints
+  , sFPHalfs
+  , sFPBFloats
+  , sFPSingles
+  , sFPDoubles
+  , sFPQuads
   , sChars
   , sStrings
   , sLists
@@ -235,8 +265,19 @@ module Data.SBV (
   , sRoundNearestTiesToEven, sRoundNearestTiesToAway, sRoundTowardPositive, sRoundTowardNegative, sRoundTowardZero, sRNE, sRNA, sRTP, sRTN, sRTZ
   -- ** Conversion to/from floats
   , IEEEFloatConvertible(..)
+
   -- ** Bit-pattern conversions
-  , sFloatAsSWord32, sWord32AsSFloat, sDoubleAsSWord64, sWord64AsSDouble, blastSFloat, blastSDouble
+  , sFloatAsSWord32,       sWord32AsSFloat
+  , sDoubleAsSWord64,      sWord64AsSDouble
+  , sFloatingPointAsSWord, sWordAsSFloatingPoint
+
+  -- ** Extracting bit patterns from floats
+  , blastSFloat
+  , blastSDouble
+  , blastSFloatingPoint
+
+  -- ** Showing values in detail
+  , crack
 
   -- * Enumerations
   -- $enumerations
@@ -244,15 +285,20 @@ module Data.SBV (
 
   -- * Uninterpreted sorts, axioms, constants, and functions
   -- $uninterpreted
-  , Uninterpreted(..), addAxiom
+  , mkUninterpretedSort, Uninterpreted(..), addAxiom
 
   -- * Properties, proofs, and satisfiability
   -- $proveIntro
   -- $noteOnNestedQuantifiers
   -- $multiIntro
   , Predicate, Goal
-  , Provable, forAll_, forAll, forSome_, forSome, prove, proveWith, sat
-  , satWith, allSat, allSatWith, optimize, optimizeWith, isVacuous
+  , Provable, forAll_, forAll, forSome_, forSome
+  , prove, proveWith
+  , dprove, dproveWith
+  , sat, satWith
+  , dsat, dsatWith
+  , allSat, allSatWith
+  , optimize, optimizeWith, isVacuous
   , isVacuousWith, isTheorem, isTheoremWith, isSatisfiable, isSatisfiableWith
   , proveWithAll, proveWithAny, satWithAll
   , proveConcurrentWithAny, proveConcurrentWithAll, satConcurrentWithAny, satConcurrentWithAll
@@ -323,9 +369,9 @@ module Data.SBV (
   -- $verbosity
 
   -- ** Solvers
-  , boolector, cvc4, yices, z3, mathSAT, abc
+  , boolector, bitwuzla, cvc4, cvc5, yices, dReal, z3, mathSAT, abc
   -- ** Configurations
-  , defaultSolverConfig, defaultSMTCfg, sbvCheckSolverInstallation, sbvAvailableSolvers
+  , defaultSolverConfig, defaultSMTCfg, defaultDeltaSMTCfg, sbvCheckSolverInstallation, getAvailableSolvers
   , setLogic, Logic(..), setOption, setInfo, setTimeOut
   -- ** SBV exceptions
   , SBVException(..)
@@ -356,20 +402,28 @@ import Data.SBV.Core.Model      hiding (assertWithPenalty, minimize, maximize,
                                         forall, forall_, exists, exists_,
                                         solve, sBool, sBool_, sBools, sChar, sChar_, sChars,
                                         sDouble, sDouble_, sDoubles, sFloat, sFloat_, sFloats,
+                                        sFloatingPoint, sFloatingPoint_, sFloatingPoints,
+                                        sFPHalf, sFPHalf_, sFPHalfs, sFPBFloat, sFPBFloat_, sFPBFloats, sFPSingle, sFPSingle_, sFPSingles,
+                                        sFPDouble, sFPDouble_, sFPDoubles, sFPQuad, sFPQuad_, sFPQuads,
                                         sInt8, sInt8_, sInt8s, sInt16, sInt16_, sInt16s, sInt32, sInt32_, sInt32s,
                                         sInt64, sInt64_, sInt64s, sInteger, sInteger_, sIntegers,
                                         sList, sList_, sLists, sTuple, sTuple_, sTuples,
                                         sReal, sReal_, sReals, sString, sString_, sStrings,
+                                        sRational, sRational_, sRationals,
                                         sWord8, sWord8_, sWord8s, sWord16, sWord16_, sWord16s,
                                         sWord32, sWord32_, sWord32s, sWord64, sWord64_, sWord64s,
                                         sMaybe, sMaybe_, sMaybes, sEither, sEither_, sEithers, sSet, sSet_, sSets)
 import Data.SBV.Core.Sized      hiding (sWord, sWord_, sWords, sInt, sInt_, sInts)
+import Data.SBV.Core.Kind
+
+import Data.SBV.Core.SizedFloats
 
 import Data.SBV.Core.Floating
 import Data.SBV.Core.Symbolic   (MonadSymbolic(..), SymbolicT)
 
 import Data.SBV.Provers.Prover hiding (forAll_, forAll, forSome_, forSome,
                                        prove, proveWith, sat, satWith, allSat,
+                                       dsat, dsatWith, dprove, dproveWith,
                                        allSatWith, optimize, optimizeWith,
                                        isVacuous, isVacuousWith, isTheorem,
                                        isTheoremWith, isSatisfiable,
@@ -389,6 +443,14 @@ import Data.Word
 import Data.SBV.SMT.Utils (SBVException(..))
 import Data.SBV.Control.Types (SMTReasonUnknown(..), Logic(..))
 
+import qualified Data.SBV.Utils.CrackNum as CN
+
+-- | Show a value in detailed (cracked) form, if possible.
+-- This makes most sense with numbers, and especially floating-point types.
+crack :: SBV a -> String
+crack (SBV (SVal _ (Left cv))) | Just s <- CN.crackNum cv = s
+crack (SBV sv)                                            = show sv
+
 -- Haddock section documentation
 {- $progIntro
 The SBV library is really two things:
@@ -450,7 +512,7 @@ in parallel, and return the results of the first one finishing. The other thread
 are useful when you do not care if the solvers produce the same result, but rather want to get the
 solution as quickly as possible, taking advantage of modern many-core machines.
 
-Note that the function 'sbvAvailableSolvers' will return all the installed solvers, which can be
+Note that the function 'getAvailableSolvers' will return all the installed solvers, which can be
 used as the first argument to all these functions, if you simply want to try all available solvers on a machine.
 -}
 
@@ -471,8 +533,8 @@ We can use 'safe' to statically see if such a violation is possible before we us
 
 >>> safe (sub :: SInt8 -> SInt8 -> SInt8)
 [sub: x >= y must hold!: Violated. Model:
-  s0 = 30 :: Int8
-  s1 = 32 :: Int8]
+  s0 = 0 :: Int8
+  s1 = 1 :: Int8]
 
 What happens if we make sure to arrange for this invariant? Consider this version:
 
@@ -683,7 +745,7 @@ by Rummer and Wahl: <http://www.philipp.ruemmer.org/publications/smt-fpa.pdf>.
 -}
 
 {- $strings
-Support for characters, strings, and regular expressions (intial version contributed by Joel Burget)
+Support for characters, strings, and regular expressions (initial version contributed by Joel Burget)
 adds support for QF_S logic, described here: <http://smtlib.cs.uiowa.edu/theories-UnicodeStrings.shtml>
 and here: <http://rise4fun.com/z3/tutorialcontent/sequences>. Note
 that this logic is still not part of official SMTLib (as of March 2018), so it should be considered
@@ -693,7 +755,7 @@ See "Data.SBV.Char", "Data.SBV.String", "Data.SBV.RegExp" for related functions.
 -}
 
 {- $lists
-Support for symbolic lists (intial version contributed by Joel Burget)
+Support for symbolic lists (initial version contributed by Joel Burget)
 adds support for sequence support, described here: <http://rise4fun.com/z3/tutorialcontent/sequences>. Note
 that this logic is still not part of official SMTLib (as of March 2018), so it should be considered
 experimental.
@@ -835,18 +897,19 @@ See 'Data.SBV.Control.getUnsatCore' for details and "Documentation.SBV.Examples.
 -}
 
 {- $uninterpreted
-Users can introduce new uninterpreted sorts simply by defining a data-type in Haskell and registering it as such. The
+Users can introduce new uninterpreted sorts simply by defining an empty data-type in Haskell and registering it as such. The
 following example demonstrates:
 
   @
-     data B = B () deriving (Eq, Ord, Show, Read, Data, SymVal, HasKind, SatModel)
+     data B
+     mkUninterpretedSort ''B
   @
 
-(Note that you'll also need to use the language pragmas @DeriveDataTypeable@, @DeriveAnyClass@, and import @Data.Generics@ for the above to work.)
+(Note that you'll also need to use pragmas @TemplateHaskell@, @StandAloneDeriving@, @DeriveDataTypeable@, and @DeriveAnyClass@ for this to work, follow GHC's error messages!)
 
 This is all it takes to introduce @B@ as an uninterpreted sort in SBV, which makes the type @SBV B@ automagically become available as the type
-of symbolic values that ranges over @B@ values. Note that the @()@ argument is important to distinguish it from enumerations, which will be
-translated to proper SMT data-types.
+of symbolic values that ranges over @B@ values. Note that this will also introduce the type @SB@ into your environment, which is a synonym
+for @SBV B@.
 
 
 Uninterpreted functions over both uninterpreted and regular sorts can be declared using the facilities introduced by
@@ -854,8 +917,9 @@ the 'Data.SBV.Core.Model.Uninterpreted' class.
 -}
 
 {- $enumerations
-If the uninterpreted sort definition takes the form of an enumeration (i.e., a simple data type with all nullary constructors), then SBV will actually
-translate that as just such a data-type to SMT-Lib, and will use the constructors as the inhabitants of the said sort. A simple example is:
+If the uninterpreted sort definition takes the form of an enumeration (i.e., a simple data type with all nullary constructors), then 
+tou can use the 'mkSymbolicEnumeration' functio to turn it into an enumeration in SMTLib.
+A simple example is:
 
 @
     data X = A | B | C
@@ -870,17 +934,17 @@ options turned on:
 >   LANGUAGE DeriveDataTypeable
 >   LANGUAGE DeriveAnyClass
 
-Now, the user can define
+The declaration will automatically introduce the type:
 
 @
     type SX = SBV X
 @
 
-and treat @SX@ as a regular symbolic type ranging over the values @A@, @B@, and @C@. Such values can be compared for equality, and with the usual
-other comparison operators, such as @.==@, @./=@, @.>@, @.>=@, @<@, and @<=@.
+along with symbolic values of each of the enumerated values @sA@, @sB@, and @sC@. This way,
+you can refer to the symbolic version as @SX@, treating it as a regular symbolic type ranging over the values @A@, @B@, and @C@. Such values can be compared for equality, and with the usual
+other comparison operators, such as @.==@, @./=@, @.>@, @.>=@, @<@, and @<=@. For each enumerated value @X@, the symbolic versions @sX@ is defined to be equal to @literal X@.
 
-Note that in this latter case the type is no longer uninterpreted, but is properly represented as a simple enumeration of the said elements. A simple
-query would look like:
+A simple query would look like:
 
 @
      allSat $ \x -> x .== (x :: SX)
@@ -898,14 +962,13 @@ which would list all three elements of this domain as satisfying solutions.
      Found 3 different solutions.
 @
 
-Note that the result is properly typed as @X@ elements; these are not mere strings. So, in a 'getModelAssignment' scenario, the user can recover actual
-elements of the domain and program further with those values as usual.
+Note that the result is properly typed as @X@ elements; these are not mere strings.
 
 See "Documentation.SBV.Examples.Misc.Enumerate" for an extended example on how to use symbolic enumerations.
 -}
 
 {- $noteOnNestedQuantifiers
-=== A note on reasoning in the presence of quantifers
+=== A note on reasoning in the presence of quantifiers
 
 Note that SBV allows reasoning with quantifiers: Inputs can be existentially or universally quantified. Predicates can be built
 with arbitrary nesting of such quantifiers as well. However, SBV always /assumes/ that the input is in
