@@ -25,12 +25,14 @@ module Data.SBV.Client.BaseIO where
 
 import Data.SBV.Core.Data      (HasKind, Kind, Outputtable, Penalty, SymArray,
                                 SymVal, SBool, SBV, SChar, SDouble, SFloat,
+                                SFPHalf, SFPBFloat, SFPSingle, SFPDouble, SFPQuad, SFloatingPoint,
                                 SInt8, SInt16, SInt32, SInt64, SInteger, SList,
                                 SReal, SString, SV, SWord8, SWord16, SWord32,
-                                SWord64, SEither, SMaybe, SSet)
-import Data.SBV.Core.Sized     (SInt, SWord, IntN, WordN, IsNonZero)
+                                SWord64, SEither, SRational, SMaybe, SSet)
+import Data.SBV.Core.Sized     (SInt, SWord, IntN, WordN)
+import Data.SBV.Core.Kind      (BVIsNonZero, ValidFloat)
 import Data.SBV.Core.Model     (Metric(..), SymTuple)
-import Data.SBV.Core.Symbolic  (Objective, OptimizeStyle, Quantifier, Result,
+import Data.SBV.Core.Symbolic  (Objective, OptimizeStyle, Result, VarContext,
                                 Symbolic, SBVRunMode, SMTConfig, SVal)
 import Data.SBV.Control.Types  (SMTOption)
 import Data.SBV.Provers.Prover (Provable, SExecutable, ThmResult)
@@ -103,6 +105,18 @@ prove = Trans.prove
 proveWith :: Provable a => SMTConfig -> a -> IO ThmResult
 proveWith = Trans.proveWith
 
+-- | Prove a predicate with delta-satisfiability, using the default solver.
+--
+-- NB. For a version which generalizes over the underlying monad, see 'Data.SBV.Trans.prove'
+dprove :: Provable a => a -> IO ThmResult
+dprove = Trans.dprove
+
+-- | Prove the predicate with delta-satisfiability using the given SMT-solver.
+--
+-- NB. For a version which generalizes over the underlying monad, see 'Data.SBV.Trans.proveWith'
+dproveWith :: Provable a => SMTConfig -> a -> IO ThmResult
+dproveWith = Trans.dproveWith
+
 -- | Find a satisfying assignment for a predicate, using the default solver.
 --
 -- NB. For a version which generalizes over the underlying monad, see 'Data.SBV.Trans.sat'
@@ -115,6 +129,18 @@ sat = Trans.sat
 satWith :: Provable a => SMTConfig -> a -> IO SatResult
 satWith = Trans.satWith
 
+-- | Find a delta-satisfying assignment for a predicate, using the default solver for delta-satisfiability.
+--
+-- NB. For a version which generalizes over the underlying monad, see 'Data.SBV.Trans.dsat'
+dsat :: Provable a => a -> IO SatResult
+dsat = Trans.dsat
+
+-- | Find a satisfying assignment using the given SMT-solver.
+--
+-- NB. For a version which generalizes over the underlying monad, see 'Data.SBV.Trans.satWith'
+dsatWith :: Provable a => SMTConfig -> a -> IO SatResult
+dsatWith = Trans.dsatWith
+
 -- | Find all satisfying assignments, using the default solver.
 -- Equivalent to @'allSatWith' 'Data.SBV.defaultSMTCfg'@. See 'allSatWith' for details.
 --
@@ -223,7 +249,7 @@ safeWith = Trans.safeWith
 -- | Create a symbolic variable.
 --
 -- NB. For a version which generalizes over the underlying monad, see 'Data.SBV.Trans.mkSymSBV'
-mkSymSBV :: Maybe Quantifier -> Kind -> Maybe String -> Symbolic (SBV a)
+mkSymSBV :: VarContext -> Kind -> Maybe String -> Symbolic (SBV a)
 mkSymSBV = Trans.mkSymSBV
 
 -- | Convert a symbolic value to an SV, inside the Symbolic monad
@@ -308,7 +334,7 @@ symbolics = Trans.symbolics
 -- | One stop allocator
 --
 -- NB. For a version which generalizes over the underlying monad, see 'Data.SBV.Trans.mkSymVal'
-mkSymVal :: SymVal a => Maybe Quantifier -> Maybe String -> Symbolic (SBV a)
+mkSymVal :: SymVal a => VarContext -> Maybe String -> Symbolic (SBV a)
 mkSymVal = Trans.mkSymVal
 
 -- | Create a new anonymous array, possibly with a default initial value.
@@ -328,7 +354,7 @@ newArray = Trans.newArray
 -- | Generically make a symbolic var
 --
 -- NB. For a version which generalizes over the underlying monad, see 'Data.SBV.Trans.genMkSymVar'
-genMkSymVar :: Kind -> Maybe Quantifier -> Maybe String -> Symbolic (SBV a)
+genMkSymVar :: Kind -> VarContext -> Maybe String -> Symbolic (SBV a)
 genMkSymVar = Trans.genMkSymVar
 
 -- | Declare a named 'SBool'
@@ -424,19 +450,19 @@ sWord64s = Trans.sWord64s
 -- | Declare a named 'SWord'
 --
 -- NB. For a version which generalizes over the underlying monad, see 'Data.SBV.Trans.sWord'
-sWord :: (KnownNat n, IsNonZero n) => String -> Symbolic (SWord n)
+sWord :: (KnownNat n, BVIsNonZero n) => String -> Symbolic (SWord n)
 sWord = Trans.sWord
 
 -- | Declare an unnamed 'SWord'
 --
 -- NB. For a version which generalizes over the underlying monad, see 'Data.SBV.Trans.sWord_'
-sWord_ :: (KnownNat n, IsNonZero n) => Symbolic (SWord n)
+sWord_ :: (KnownNat n, BVIsNonZero n) => Symbolic (SWord n)
 sWord_ = Trans.sWord_
 
 -- | Declare a list of 'SWord8's
 --
 -- NB. For a version which generalizes over the underlying monad, see 'Data.SBV.Trans.sWords'
-sWords :: (KnownNat n, IsNonZero n) => [String] -> Symbolic [SWord n]
+sWords :: (KnownNat n, BVIsNonZero n) => [String] -> Symbolic [SWord n]
 sWords = Trans.sWords
 
 -- | Declare a named 'SInt8'
@@ -514,19 +540,19 @@ sInt64s = Trans.sInt64s
 -- | Declare a named 'SInt'
 --
 -- NB. For a version which generalizes over the underlying monad, see 'Data.SBV.Trans.sInt'
-sInt :: (KnownNat n, IsNonZero n) => String -> Symbolic (SInt n)
+sInt :: (KnownNat n, BVIsNonZero n) => String -> Symbolic (SInt n)
 sInt = Trans.sInt
 
 -- | Declare an unnamed 'SInt'
 --
 -- NB. For a version which generalizes over the underlying monad, see 'Data.SBV.Trans.sInt_'
-sInt_ :: (KnownNat n, IsNonZero n) => Symbolic (SInt n)
+sInt_ :: (KnownNat n, BVIsNonZero n) => Symbolic (SInt n)
 sInt_ = Trans.sInt_
 
 -- | Declare a list of 'SInt's
 --
 -- NB. For a version which generalizes over the underlying monad, see 'Data.SBV.Trans.sInts'
-sInts :: (KnownNat n, IsNonZero n) => [String] -> Symbolic [SInt n]
+sInts :: (KnownNat n, BVIsNonZero n) => [String] -> Symbolic [SInt n]
 sInts = Trans.sInts
 
 -- | Declare a named 'SInteger'
@@ -601,6 +627,114 @@ sDouble_ = Trans.sDouble_
 sDoubles :: [String] -> Symbolic [SDouble]
 sDoubles = Trans.sDoubles
 
+-- | Declare a named 'SFloatingPoint eb sb'
+--
+-- NB. For a version which generalizes over the underlying monad, see 'Data.SBV.Trans.sFloatingPoint'
+sFloatingPoint :: ValidFloat eb sb => String -> Symbolic (SFloatingPoint eb sb)
+sFloatingPoint = Trans.sFloatingPoint
+
+-- | Declare an unnamed 'SFloatingPoint' @eb@ @sb@
+--
+-- NB. For a version which generalizes over the underlying monad, see 'Data.SBV.Trans.sFloatingPoint_'
+sFloatingPoint_ :: ValidFloat eb sb => Symbolic (SFloatingPoint eb sb)
+sFloatingPoint_ = Trans.sFloatingPoint_
+
+-- | Declare a list of 'SFloatingPoint' @eb@ @sb@'s
+--
+-- NB. For a version which generalizes over the underlying monad, see 'Data.SBV.Trans.sFloatingPoints'
+sFloatingPoints :: ValidFloat eb sb => [String] -> Symbolic [SFloatingPoint eb sb]
+sFloatingPoints = Trans.sFloatingPoints
+
+-- | Declare a named 'SFPHalf'
+--
+-- NB. For a version which generalizes over the underlying monad, see 'Data.SBV.Trans.sFPHalf'
+sFPHalf :: String -> Symbolic SFPHalf
+sFPHalf = Trans.sFPHalf
+
+-- | Declare an unnamed 'SFPHalf'
+--
+-- NB. For a version which generalizes over the underlying monad, see 'Data.SBV.Trans.sFPHalf_'
+sFPHalf_ :: Symbolic SFPHalf
+sFPHalf_ = Trans.sFPHalf_
+
+-- | Declare a list of 'SFPHalf's
+--
+-- NB. For a version which generalizes over the underlying monad, see 'Data.SBV.Trans.sFPHalfs'
+sFPHalfs :: [String] -> Symbolic [SFPHalf]
+sFPHalfs = Trans.sFPHalfs
+
+-- | Declare a named 'SFPBFloat'
+--
+-- NB. For a version which generalizes over the underlying monad, see 'Data.SBV.Trans.SFPBFloat'
+sFPBFloat :: String -> Symbolic SFPBFloat
+sFPBFloat = Trans.sFPBFloat
+
+-- | Declare an unnamed 'SFPBFloat'
+--
+-- NB. For a version which generalizes over the underlying monad, see 'Data.SBV.Trans.SFPBFloat'
+sFPBFloat_ :: Symbolic SFPBFloat
+sFPBFloat_ = Trans.sFPBFloat_
+
+-- | Declare a list of 'SFPQuad's
+--
+-- NB. For a version which generalizes over the underlying monad, see 'Data.SBV.Trans.sFPBFloats'
+sFPBFloats :: [String] -> Symbolic [SFPBFloat]
+sFPBFloats = Trans.sFPBFloats
+
+-- | Declare a named 'SFPSingle'
+--
+-- NB. For a version which generalizes over the underlying monad, see 'Data.SBV.Trans.sFPSingle'
+sFPSingle :: String -> Symbolic SFPSingle
+sFPSingle = Trans.sFPSingle
+
+-- | Declare an unnamed 'SFPSingle'
+--
+-- NB. For a version which generalizes over the underlying monad, see 'Data.SBV.Trans.sFPSingle_'
+sFPSingle_ :: Symbolic SFPSingle
+sFPSingle_ = Trans.sFPSingle_
+
+-- | Declare a list of 'SFPSingle's
+--
+-- NB. For a version which generalizes over the underlying monad, see 'Data.SBV.Trans.sFPSingles'
+sFPSingles :: [String] -> Symbolic [SFPSingle]
+sFPSingles = Trans.sFPSingles
+
+-- | Declare a named 'SFPDouble'
+--
+-- NB. For a version which generalizes over the underlying monad, see 'Data.SBV.Trans.sFPDouble'
+sFPDouble :: String -> Symbolic SFPDouble
+sFPDouble = Trans.sFPDouble
+
+-- | Declare an unnamed 'SFPDouble'
+--
+-- NB. For a version which generalizes over the underlying monad, see 'Data.SBV.Trans.sFPDouble_'
+sFPDouble_ :: Symbolic SFPDouble
+sFPDouble_ = Trans.sFPDouble_
+
+-- | Declare a list of 'SFPDouble's
+--
+-- NB. For a version which generalizes over the underlying monad, see 'Data.SBV.Trans.sFPDoubles'
+sFPDoubles :: [String] -> Symbolic [SFPDouble]
+sFPDoubles = Trans.sFPDoubles
+
+-- | Declare a named 'SFPQuad'
+--
+-- NB. For a version which generalizes over the underlying monad, see 'Data.SBV.Trans.sFPQuad'
+sFPQuad :: String -> Symbolic SFPQuad
+sFPQuad = Trans.sFPQuad
+
+-- | Declare an unnamed 'SFPQuad'
+--
+-- NB. For a version which generalizes over the underlying monad, see 'Data.SBV.Trans.sFPQuad_'
+sFPQuad_ :: Symbolic SFPQuad
+sFPQuad_ = Trans.sFPQuad_
+
+-- | Declare a list of 'SFPQuad's
+--
+-- NB. For a version which generalizes over the underlying monad, see 'Data.SBV.Trans.sFPQuads'
+sFPQuads :: [String] -> Symbolic [SFPQuad]
+sFPQuads = Trans.sFPQuads
+
 -- | Declare a named 'SChar'
 --
 -- NB. For a version which generalizes over the underlying monad, see 'Data.SBV.Trans.sChar'
@@ -691,6 +825,24 @@ sEither_ = Trans.sEither_
 sEithers :: (SymVal a, SymVal b) => [String] -> Symbolic [SEither a b]
 sEithers = Trans.sEithers
 
+-- | Declare a named 'Data.SBV.SRational'.
+--
+-- NB. For a version which generalizes over the underlying monad, see 'Data.SBV.Trans.sRational'
+sRational :: String -> Symbolic SRational
+sRational = Trans.sRational
+
+-- | Declare an unnamed 'Data.SBV.SRational'.
+--
+-- NB. For a version which generalizes over the underlying monad, see 'Data.SBV.Trans.sRational_'
+sRational_ :: Symbolic SRational
+sRational_ = Trans.sRational_
+
+-- | Declare a list of 'Data.SBV.SRational' values.
+--
+-- NB. For a version which generalizes over the underlying monad, see 'Data.SBV.Trans.sRationals'
+sRationals :: [String] -> Symbolic [SRational]
+sRationals = Trans.sRationals
+
 -- | Declare a named 'Data.SBV.SMaybe'.
 --
 -- NB. For a version which generalizes over the underlying monad, see 'Data.SBV.Trans.sMaybe'
