@@ -193,20 +193,22 @@ showCType i = case kindOf i of
 -- | The printf specifier for the type
 specifier :: CgConfig -> SV -> Doc
 specifier cfg sv = case kindOf sv of
-                     KBool              -> spec (False, 1)
-                     KBounded b i       -> spec (b, i)
-                     KUnbounded         -> spec (True, fromJust (cgInteger cfg))
-                     KReal              -> specF (fromJust (cgReal cfg))
-                     KFloat             -> specF CgFloat
-                     KDouble            -> specF CgDouble
-                     KString            -> text "%s"
-                     KChar              -> text "%c"
-                     KList k            -> die $ "list sort: " ++ show k
-                     KSet  k            -> die $ "set sort: " ++ show k
-                     KUninterpreted s _ -> die $ "uninterpreted sort: " ++ s
-                     KTuple k           -> die $ "tuple sort: " ++ show k
-                     KMaybe  k          -> die $ "maybe sort: "  ++ show k
-                     KEither k1 k2      -> die $ "either sort: " ++ show (k1, k2)
+                     KBool         -> spec (False, 1)
+                     KBounded b i  -> spec (b, i)
+                     KUnbounded    -> spec (True, fromJust (cgInteger cfg))
+                     KReal         -> specF (fromJust (cgReal cfg))
+                     KFloat        -> specF CgFloat
+                     KDouble       -> specF CgDouble
+                     KString       -> text "%s"
+                     KChar         -> text "%c"
+                     KRational     -> die   "rational sort"
+                     KFP{}         -> die   "arbitrary float sort"
+                     KList k       -> die $ "list sort: " ++ show k
+                     KSet  k       -> die $ "set sort: " ++ show k
+                     KUserSort s _ -> die $ "user sort: " ++ s
+                     KTuple k      -> die $ "tuple sort: " ++ show k
+                     KMaybe  k     -> die $ "maybe sort: "  ++ show k
+                     KEither k1 k2 -> die $ "either sort: " ++ show (k1, k2)
   where u8InHex = cgShowU8InHex cfg
 
         spec :: (Bool, Int) -> Doc
@@ -442,7 +444,7 @@ genDriver cfg randVals fn inps outs mbRet = [pre, header, body, post]
 
 -- | Generate the C program
 genCProg :: CgConfig -> String -> Doc -> Result -> [(String, CgVal)] -> [(String, CgVal)] -> Maybe SV -> Doc -> ([Doc], [String])
-genCProg cfg fn proto (Result kindInfo _tvals _ovals cgs ins preConsts tbls arrs _uis _axioms (SBVPgm asgns) cstrs origAsserts _) inVars outVars mbRet extDecls
+genCProg cfg fn proto (Result kindInfo _tvals _ovals cgs ins (_, preConsts) tbls arrs _uis _axioms (SBVPgm asgns) cstrs origAsserts _) inVars outVars mbRet extDecls
   | isNothing (cgInteger cfg) && KUnbounded `Set.member` kindInfo
   = error $ "SBV->C: Unbounded integers are not supported by the C compiler."
           ++ "\nUse 'cgIntegerSize' to specify a fixed size for SInteger representation."
@@ -476,7 +478,7 @@ genCProg cfg fn proto (Result kindInfo _tvals _ovals cgs ins preConsts tbls arrs
  where asserts | cgIgnoreAsserts cfg = []
                | True                = origAsserts
 
-       usorts = [s | KUninterpreted s _ <- Set.toList kindInfo, s /= "RoundingMode"] -- No support for any sorts other than RoundingMode!
+       usorts = [s | KUserSort s _ <- Set.toList kindInfo, s /= "RoundingMode"] -- No support for any sorts other than RoundingMode!
 
        pre    =  text "/* File:" <+> doubleQuotes (nm P.<> text ".c") P.<> text ". Automatically generated by SBV. Do not edit! */"
               $$ text ""
@@ -510,22 +512,24 @@ genCProg cfg fn proto (Result kindInfo _tvals _ovals cgs ins preConsts tbls arrs
                          $$ vcat (map text ls)
                          $$ text ""
 
-       typeWidth = getMax 0 $ [len (kindOf s) | (s, _) <- assignments] ++ [len (kindOf s) | (_, (s, _)) <- fst ins]
-                where len KReal{}              = 5
-                      len KFloat{}             = 6 -- SFloat
-                      len KDouble{}            = 7 -- SDouble
-                      len KString{}            = 7 -- SString
-                      len KChar{}              = 5 -- SChar
-                      len KUnbounded{}         = 8
-                      len KBool                = 5 -- SBool
-                      len (KBounded False n)   = 5 + length (show n) -- SWordN
-                      len (KBounded True  n)   = 4 + length (show n) -- SIntN
-                      len (KList s)            = die $ "List sort: " ++ show s
-                      len (KSet  s)            = die $ "Set sort: " ++ show s
-                      len (KTuple s)           = die $ "Tuple sort: " ++ show s
-                      len (KMaybe k)           = die $ "Maybe sort: " ++ show k
-                      len (KEither k1 k2)      = die $ "Either sort: " ++ show (k1, k2)
-                      len (KUninterpreted s _) = die $ "Uninterpreted sort: " ++ s
+       typeWidth = getMax 0 $ [len (kindOf s) | (s, _) <- assignments] ++ [len (kindOf s) | (_, NamedSymVar s _) <- fst ins]
+                where len KReal{}            = 5
+                      len KFloat{}           = 6 -- SFloat
+                      len KDouble{}          = 7 -- SDouble
+                      len KString{}          = 7 -- SString
+                      len KChar{}            = 5 -- SChar
+                      len KUnbounded{}       = 8
+                      len KBool              = 5 -- SBool
+                      len (KBounded False n) = 5 + length (show n) -- SWordN
+                      len (KBounded True  n) = 4 + length (show n) -- SIntN
+                      len KRational{}        = die   "Rational."
+                      len KFP{}              = die   "Arbitrary float."
+                      len (KList s)          = die $ "List sort: " ++ show s
+                      len (KSet  s)          = die $ "Set sort: " ++ show s
+                      len (KTuple s)         = die $ "Tuple sort: " ++ show s
+                      len (KMaybe k)         = die $ "Maybe sort: " ++ show k
+                      len (KEither k1 k2)    = die $ "Either sort: " ++ show (k1, k2)
+                      len (KUserSort s _)    = die $ "Uninterpreted sort: " ++ s
 
                       getMax 8 _      = 8  -- 8 is the max we can get with SInteger, so don't bother looking any further
                       getMax m []     = m
@@ -671,16 +675,16 @@ handleIEEE w consts as var = cvt w
         fpArgs = case as of
                    []            -> []
                    ((m, _):args) -> case kindOf m of
-                                      KUninterpreted "RoundingMode" _ -> case checkRM (m `lookup` consts) of
-                                                                           Nothing          -> args
-                                                                           Just (Left  msg) -> die msg
-                                                                           Just (Right msg) -> tbd msg
-                                      _                               -> as
+                                      KUserSort "RoundingMode" _ -> case checkRM (m `lookup` consts) of
+                                                                      Nothing          -> args
+                                                                      Just (Left  msg) -> die msg
+                                                                      Just (Right msg) -> tbd msg
+                                      _                          -> as
 
         -- Check that the RM is RoundNearestTiesToEven.
         -- If we start supporting other rounding-modes, this would be the point where we'd insert the rounding-mode set/reset code
         -- instead of merely returning OK or not
-        checkRM (Just cv@(CV (KUninterpreted "RoundingMode" _) v)) =
+        checkRM (Just cv@(CV (KUserSort "RoundingMode" _) v)) =
               case v of
                 CUserSort (_, "RoundNearestTiesToEven") -> Nothing
                 CUserSort (_, s)                        -> Just (Right $ "handleIEEE: Unsupported rounding-mode: " ++ show s ++ " for: " ++ show w)
@@ -771,22 +775,24 @@ ppExpr cfg consts (SBVApp op opArgs) lhs (typ, var)
                 canOverflow False sz = (2::Integer)^sz    -1 >= fromIntegral len
 
                 (needsCheckL, needsCheckR) = case k of
-                                               KBool              -> (False, canOverflow False (1::Int))
-                                               KBounded sg sz     -> (sg, canOverflow sg sz)
-                                               KReal              -> die "array index with real value"
-                                               KFloat             -> die "array index with float value"
-                                               KDouble            -> die "array index with double value"
-                                               KString            -> die "array index with string value"
-                                               KChar              -> die "array index with character value"
-                                               KUnbounded         -> case cgInteger cfg of
-                                                                       Nothing -> (True, True) -- won't matter, it'll be rejected later
-                                                                       Just i  -> (True, canOverflow True i)
-                                               KList     s        -> die $ "List sort " ++ show s
-                                               KSet      s        -> die $ "Set sort " ++ show s
-                                               KTuple    s        -> die $ "Tuple sort " ++ show s
-                                               KMaybe    ek       -> die $ "Maybe sort " ++ show ek
-                                               KEither   k1 k2    -> die $ "Either sort " ++ show (k1, k2)
-                                               KUninterpreted s _ -> die $ "Uninterpreted sort: " ++ s
+                                               KBool           -> (False, canOverflow False (1::Int))
+                                               KBounded sg sz  -> (sg, canOverflow sg sz)
+                                               KReal           -> die "array index with real value"
+                                               KFloat          -> die "array index with float value"
+                                               KDouble         -> die "array index with double value"
+                                               KFP{}           -> die "array index with arbitrary float value"
+                                               KRational       -> die "array index with rational value"
+                                               KString         -> die "array index with string value"
+                                               KChar           -> die "array index with character value"
+                                               KUnbounded      -> case cgInteger cfg of
+                                                                    Nothing -> (True, True) -- won't matter, it'll be rejected later
+                                                                    Just i  -> (True, canOverflow True i)
+                                               KList     s     -> die $ "List sort " ++ show s
+                                               KSet      s     -> die $ "Set sort " ++ show s
+                                               KTuple    s     -> die $ "Tuple sort " ++ show s
+                                               KMaybe    ek    -> die $ "Maybe sort " ++ show ek
+                                               KEither   k1 k2 -> die $ "Either sort " ++ show (k1, k2)
+                                               KUserSort s _   -> die $ "Uninterpreted sort: " ++ s
 
         -- Div/Rem should be careful on 0, in the SBV world x `div` 0 is 0, x `rem` 0 is x
         -- NB: Quot is supposed to truncate toward 0; Not clear to me if C guarantees this behavior.
