@@ -19,7 +19,7 @@ import Data.SBV.Control.Query (Assignment)
 import Data.SBV.Control.Types (CheckSatResult, SMTInfoFlag, SMTInfoResponse, SMTOption, SMTReasonUnknown)
 import Data.SBV.Core.Concrete (CV)
 import Data.SBV.Core.Data     (HasKind, Symbolic, SymArray, SymVal, SBool, SBV, SBVType)
-import Data.SBV.Core.Symbolic (Query, QueryContext, QueryState, State, SMTModel, SMTResult, SV)
+import Data.SBV.Core.Symbolic (Query, QueryContext, QueryState, State, SMTModel, SMTResult, SV, Name)
 
 import qualified Data.SBV.Control.Query as Trans
 import qualified Data.SBV.Control.Utils as Trans
@@ -51,10 +51,10 @@ getUnknownReason = Trans.getUnknownReason
 -- | Get the observables recorded during a query run.
 --
 -- NB. For a version which generalizes over the underlying monad, see 'Data.SBV.Trans.Control.getObservables'
-getObservables :: Query [(String, CV)]
+getObservables :: Query [(Name, CV)]
 getObservables = Trans.getObservables
 
--- | Get the uinterpreted constants/functions recorded during a run.
+-- | Get the uninterpreted constants/functions recorded during a run.
 --
 -- NB. For a version which generalizes over the underlying monad, see 'Data.SBV.Trans.Control.getUIs'
 getUIs :: Query [(String, SBVType)]
@@ -448,6 +448,10 @@ getFunction = Trans.getFunction
 getValueCV :: Maybe Int -> SV -> Query CV
 getValueCV = Trans.getValueCV
 
+-- | Get the value of an uninterpreted value
+getUICVal :: Maybe Int -> (String, SBVType) -> Query CV
+getUICVal = Trans.getUICVal
+
 -- | Get the value of an uninterpreted function as an association list
 --
 -- NB. For a version which generalizes over the underlying monad, see 'Data.SBV.Trans.Control.getUIFunCVAssoc'
@@ -488,7 +492,7 @@ getUnsatAssumptions = Trans.getUnsatAssumptions
 -- Semantics: The call @timeout n q@ causes the timeout value to be applied to all interactive calls that take place
 -- as we execute the query @q@. That is, each call that happens during the execution of @q@ gets a separate
 -- time-out value, as opposed to one timeout value that limits the whole query. This is typically the intended behavior.
--- It is advisible to apply this combinator to calls that involve a single call to the solver for
+-- It is advisable to apply this combinator to calls that involve a single call to the solver for
 -- finer control, as opposed to an entire set of interactions. However, different use cases might call for different scenarios.
 --
 -- If the solver responds within the time-out specified, then we continue as usual. However, if the backend solver times-out
