@@ -9,10 +9,13 @@
 -- Querying a solver interactively.
 -----------------------------------------------------------------------------
 
+{-# LANGUAGE BangPatterns        #-}
 {-# LANGUAGE LambdaCase          #-}
 {-# LANGUAGE NamedFieldPuns      #-}
 {-# LANGUAGE Rank2Types          #-}
 {-# LANGUAGE ScopedTypeVariables #-}
+{-# LANGUAGE TupleSections       #-}
+{-# LANGUAGE ViewPatterns        #-}
 
 {-# OPTIONS_GHC -Wall -Werror -fno-warn-orphans #-}
 
@@ -37,17 +40,23 @@ import Data.IORef (readIORef)
 
 import qualified Data.Map.Strict    as M
 import qualified Data.IntMap.Strict as IM
+import qualified Data.Sequence      as S
+import qualified Data.Text          as T
 
 
-import Data.Char     (toLower)
-import Data.List     (intercalate, nubBy, sortBy, sortOn)
-import Data.Maybe    (listToMaybe, catMaybes)
-import Data.Function (on)
+import Data.Char      (toLower)
+import Data.List      (intercalate, nubBy, sortOn)
+import Data.Maybe     (listToMaybe, catMaybes)
+import Data.Function  (on)
+import Data.Bifunctor (first)
+import Data.Foldable  (toList)
 
 import Data.SBV.Core.Data
 
 import Data.SBV.Core.Symbolic   ( MonadQuery(..), State(..)
                                 , incrementInternalCounter, validationRequested
+                                , prefixExistentials, prefixUniversals
+                                , namedSymVar, getSV, lookupInput, userInputsToList
                                 )
 
 import Data.SBV.Utils.SExpr
@@ -55,6 +64,8 @@ import Data.SBV.Utils.SExpr
 import Data.SBV.Control.Types
 import Data.SBV.Control.Utils
 
+import Data.SBV.Utils.PrettyNum (showNegativeNumber)
+
 -- | An Assignment of a model binding
 data Assignment = Assign SVal CV
 
@@ -78,29 +89,25 @@ fromECon _        = Nothing
 
 -- Collect strings appearing, used in 'getOption' only
 stringsOf :: SExpr -> [String]
-stringsOf (ECon s)      = [s]
-stringsOf (ENum (i, _)) = [show i]
-stringsOf (EReal   r)   = [show r]
-stringsOf (EFloat  f)   = [show f]
-stringsOf (EDouble d)   = [show d]
-stringsOf (EApp ss)     = concatMap stringsOf ss
+stringsOf (ECon s)           = [s]
+stringsOf (ENum (i, _))      = [show i]
+stringsOf (EReal   r)        = [show r]
+stringsOf (EFloat  f)        = [show f]
+stringsOf (EFloatingPoint f) = [show f]
+stringsOf (EDouble d)        = [show d]
+stringsOf (EApp ss)          = concatMap stringsOf ss
 
 -- Sort of a light-hearted show for SExprs, for better consumption at the user level.
 serialize :: Bool -> SExpr -> String
 serialize removeQuotes = go
-  where go (ECon s)      = if removeQuotes then unQuote s else s
-        go (ENum (i, _)) = shNN i
-        go (EReal   r)   = shNN r
-        go (EFloat  f)   = shNN f
-        go (EDouble d)   = shNN d
-        go (EApp [x])    = go x
-        go (EApp ss)     = "(" ++ unwords (map go ss) ++ ")"
-
-        -- be careful with negative number printing in SMT-Lib..
-        shNN :: (Show a, Num a, Ord a) => a -> String
-        shNN i
-          | i < 0 = "(- " ++ show (-i) ++ ")"
-          | True  = show i
+  where go (ECon s)           = if removeQuotes then unQuote s else s
+        go (ENum (i, _))      = showNegativeNumber i
+        go (EReal   r)        = showNegativeNumber r
+        go (EFloat  f)        = showNegativeNumber f
+        go (EDouble d)        = showNegativeNumber d
+        go (EFloatingPoint f) = show f
+        go (EApp [x])         = go x
+        go (EApp ss)          = "(" ++ unwords (map go ss) ++ ")"
 
 -- | Generalization of 'Data.SBV.Control.getInfo'
 getInfo :: (MonadIO m, MonadQuery m) => SMTInfoFlag -> m SMTInfoResponse
@@ -208,22 +215,24 @@ ensureSat :: (MonadIO m, MonadQuery m) => m ()
 ensureSat = do cfg <- getConfig
                cs <- checkSatUsing $ satCmd cfg
                case cs of
-                 Sat -> return ()
-                 Unk -> do s <- getUnknownReason
-                           error $ unlines [ ""
-                                           , "*** Data.SBV.ensureSat: Solver reported Unknown!"
-                                           , "*** Reason: " ++ show s
-                                           ]
-                 Unsat -> error "Data.SBV.ensureSat: Solver reported Unsat!"
+                 Sat    -> return ()
+                 DSat{} -> return ()
+                 Unk    -> do s <- getUnknownReason
+                              error $ unlines [ ""
+                                              , "*** Data.SBV.ensureSat: Solver reported Unknown!"
+                                              , "*** Reason: " ++ show s
+                                              ]
+                 Unsat  -> error "Data.SBV.ensureSat: Solver reported Unsat!"
 
 -- | Generalization of 'Data.SBV.Control.getSMTResult'
 getSMTResult :: (MonadIO m, MonadQuery m) => m SMTResult
 getSMTResult = do cfg <- getConfig
                   cs  <- checkSat
                   case cs of
-                    Unsat -> Unsatisfiable cfg <$> getUnsatCoreIfRequested
-                    Sat   -> Satisfiable   cfg <$> getModel
-                    Unk   -> Unknown       cfg <$> getUnknownReason
+                    Unsat  -> Unsatisfiable cfg   <$> getUnsatCoreIfRequested
+                    Sat    -> Satisfiable   cfg   <$> getModel
+                    DSat p -> DeltaSat      cfg p <$> getModel
+                    Unk    -> Unknown       cfg   <$> getUnknownReason
 
 -- | Classify a model based on whether it has unbound objectives or not.
 classifyModel :: SMTConfig -> SMTModel -> SMTResult
@@ -237,9 +246,10 @@ getLexicographicOptResults :: (MonadIO m, MonadQuery m) => m SMTResult
 getLexicographicOptResults = do cfg <- getConfig
                                 cs  <- checkSat
                                 case cs of
-                                  Unsat -> Unsatisfiable cfg <$> getUnsatCoreIfRequested
-                                  Sat   -> classifyModel cfg <$> getModelWithObjectives
-                                  Unk   -> Unknown       cfg <$> getUnknownReason
+                                  Unsat  -> Unsatisfiable cfg <$> getUnsatCoreIfRequested
+                                  Sat    -> classifyModel cfg <$> getModelWithObjectives
+                                  DSat{} -> classifyModel cfg <$> getModelWithObjectives
+                                  Unk    -> Unknown       cfg <$> getUnknownReason
    where getModelWithObjectives = do objectiveValues <- getObjectiveValues
                                      m               <- getModel
                                      return m {modelObjectives = objectiveValues}
@@ -250,10 +260,11 @@ getIndependentOptResults objNames = do cfg <- getConfig
                                        cs  <- checkSat
 
                                        case cs of
-                                         Unsat -> getUnsatCoreIfRequested >>= \mbUC -> return [(nm, Unsatisfiable cfg mbUC) | nm <- objNames]
-                                         Sat   -> continue (classifyModel cfg)
-                                         Unk   -> do ur <- Unknown cfg <$> getUnknownReason
-                                                     return [(nm, ur) | nm <- objNames]
+                                         Unsat  -> getUnsatCoreIfRequested >>= \mbUC -> return [(nm, Unsatisfiable cfg mbUC) | nm <- objNames]
+                                         Sat    -> continue (classifyModel cfg)
+                                         DSat{} -> continue (classifyModel cfg)
+                                         Unk    -> do ur <- Unknown cfg <$> getUnknownReason
+                                                      return [(nm, ur) | nm <- objNames]
 
   where continue classify = do objectiveValues <- getObjectiveValues
                                nms <- zipWithM getIndependentResult [0..] objNames
@@ -271,10 +282,11 @@ getParetoOptResults mbN      = do cfg <- getConfig
                                   cs  <- checkSat
 
                                   case cs of
-                                    Unsat -> return (False, [])
-                                    Sat   -> continue (classifyModel cfg)
-                                    Unk   -> do ur <- getUnknownReason
-                                                return (False, [ProofError cfg [show ur] Nothing])
+                                    Unsat  -> return (False, [])
+                                    Sat    -> continue (classifyModel cfg)
+                                    DSat{} -> continue (classifyModel cfg)
+                                    Unk    -> do ur <- getUnknownReason
+                                                 return (False, [ProofError cfg [show ur] Nothing])
 
   where continue classify = do m <- getModel
                                (limReached, fronts) <- getParetoFronts (subtract 1 <$> mbN) [m]
@@ -285,9 +297,10 @@ getParetoOptResults mbN      = do cfg <- getConfig
         getParetoFronts mbi      sofar          = do cs <- checkSat
                                                      let more = getModel >>= \m -> getParetoFronts (subtract 1 <$> mbi) (m : sofar)
                                                      case cs of
-                                                       Unsat -> return (False, sofar)
-                                                       Sat   -> more
-                                                       Unk   -> more
+                                                       Unsat  -> return (False, sofar)
+                                                       Sat    -> more
+                                                       DSat{} -> more
+                                                       Unk    -> more
 
 -- | Generalization of 'Data.SBV.Control.getModel'
 getModel :: (MonadIO m, MonadQuery m) => m SMTModel
@@ -297,7 +310,7 @@ getModel = getModelAtIndex Nothing
 getModelAtIndex :: (MonadIO m, MonadQuery m) => Maybe Int -> m SMTModel
 getModelAtIndex mbi = do
     State{runMode} <- queryState
-    rm     <- io $ readIORef runMode
+    rm <- io $ readIORef runMode
     case rm of
       m@CodeGen           -> error $ "SBV.getModel: Model is not available in mode: " ++ show m
       m@Concrete{}        -> error $ "SBV.getModel: Model is not available in mode: " ++ show m
@@ -307,30 +320,31 @@ getModelAtIndex mbi = do
           uis   <- getUIs
 
            -- for "sat", display the prefix existentials. for "proof", display the prefix universals
-          let allModelInputs = if isSAT then takeWhile ((/= ALL) . fst) qinps
-                                        else takeWhile ((== ALL) . fst) qinps
+          let allModelInputs = if isSAT then prefixExistentials qinps
+                                        else prefixUniversals   qinps
 
               -- Add on observables only if we're not in a quantified context
-              grabObservables = length allModelInputs == length qinps -- i.e., we didn't drop anything
+              grabObservables = S.length allModelInputs == S.length qinps -- i.e., we didn't drop anything
 
           obsvs <- if grabObservables
                       then getObservables
                       else do queryDebug ["*** In a quantified context, obvservables will not be printed."]
-                              return []
+                              return mempty
 
-          let sortByNodeId :: [(SV, (String, CV))] -> [(String, CV)]
-              sortByNodeId = map snd . sortBy (compare `on` (\(SV _ nid, _) -> nid))
+          let grab (NamedSymVar sv nm) = wrap <$> getValueCV mbi sv
+                where
+                  wrap !c = (sv, (nm, c))
 
-              grab (sv, nm) = wrap <$> getValueCV mbi sv
-                 where wrap c = (sv, (nm, c))
+          inputAssocs <- mapM (grab . namedSymVar) allModelInputs
 
-          inputAssocs <- mapM (grab . snd) allModelInputs
+          let name     = fst . snd
+              removeSV = snd
+              prepare  = S.unstableSort . S.filter (not . isNonModelVar cfg . T.unpack . name)
+              assocs   = S.fromList (sortOn fst obsvs) <> fmap removeSV (prepare inputAssocs)
 
-          let assocs =  sortOn fst obsvs
-                     ++ sortByNodeId [p | p@(_, (nm, _)) <- inputAssocs, not (isNonModelVar cfg nm)]
-
-          -- collect UIs if requested
-          let uiFuns = [ui | ui@(_, SBVType as) <- uis, length as > 1, satTrackUFs cfg] -- functions have at least two things in their type!
+          -- collect UIs, and UI functions if requested
+          let uiFuns = [ui | ui@(nm, SBVType as) <- uis, length as >  1, satTrackUFs cfg, not (isNonModelVar cfg nm)] -- functions have at least two things in their type!
+              uiRegs = [ui | ui@(nm, SBVType as) <- uis, length as == 1,                  not (isNonModelVar cfg nm)]
 
           -- If there are uninterpreted functions, arrange so that z3's pretty-printer flattens things out
           -- as cex's tend to get larger
@@ -341,10 +355,10 @@ getModelAtIndex mbi = do
                   Just cmds -> mapM_ (send True) cmds
 
           bindings <- let get i@(ALL, _)      = return (i, Nothing)
-                          get i@(EX, (sv, _)) = case sv `lookup` inputAssocs of
-                                                  Just (_, cv) -> return (i, Just cv)
-                                                  Nothing      -> do cv <- getValueCV mbi sv
-                                                                     return (i, Just cv)
+                          get i@(EX, getSV -> sv) = case lookupInput fst sv inputAssocs of
+                                                      Just (_, (_, cv)) -> return (i, Just cv)
+                                                      Nothing           -> do cv <- getValueCV mbi sv
+                                                                              return (i, Just cv)
 
                           flipQ i@(q, sv) = case (isSAT, q) of
                                              (True,  _ )  -> i
@@ -355,12 +369,14 @@ getModelAtIndex mbi = do
                          then Just <$> mapM (get . flipQ) qinps
                          else return Nothing
 
-          uivs <- mapM (\ui@(nm, t) -> (\a -> (nm, (t, a))) <$> getUIFunCVAssoc mbi ui) uiFuns
+          uiFunVals <- mapM (\ui@(nm, t) -> (\a -> (nm, (t, a))) <$> getUIFunCVAssoc mbi ui) uiFuns
+
+          uiVals    <- mapM (\ui@(nm, _) -> (nm,) <$> getUICVal mbi ui) uiRegs
 
           return SMTModel { modelObjectives = []
-                          , modelBindings   = bindings
-                          , modelAssocs     = assocs
-                          , modelUIFuns     = uivs
+                          , modelBindings   = toList <$> bindings
+                          , modelAssocs     = uiVals ++ toList (first T.unpack <$> assocs)
+                          , modelUIFuns     = uiFunVals
                           }
 
 -- | Just after a check-sat is issued, collect objective values. Used
@@ -372,7 +388,7 @@ getObjectiveValues = do let cmd = "(get-objectives)"
 
                         r <- ask cmd
 
-                        inputs <- map snd <$> getQuantifiedInputs
+                        inputs <- toList . fmap namedSymVar <$> getQuantifiedInputs
 
                         parse r bad $ \case EApp (ECon "objectives" : es) -> catMaybes <$> mapM (getObjValue (bad r) inputs) es
                                             _                             -> bad r Nothing
@@ -385,9 +401,9 @@ getObjectiveValues = do let cmd = "(get-objectives)"
                   EApp [ECon nm, v] -> locate nm v               -- Regular case
                   _                 -> dontUnderstand (show expr)
 
-          where locate nm v = case listToMaybe [p | p@(sv, _) <- inputs, show sv == nm] of
-                                Nothing               -> return Nothing -- Happens when the soft assertion has a group-id that's not one of the input names
-                                Just (sv, actualName) -> grab sv v >>= \val -> return $ Just (actualName, val)
+          where locate nm v = case listToMaybe [p | p@(NamedSymVar sv _) <- inputs, show sv == nm] of
+                                Nothing                          -> return Nothing -- Happens when the soft assertion has a group-id that's not one of the input names
+                                Just (NamedSymVar sv actualName) -> grab sv v >>= \val -> return $ Just (T.unpack actualName, val)
 
                 dontUnderstand s = bailOut $ Just [ "Unable to understand solver output."
                                                   , "While trying to process: " ++ s
@@ -545,16 +561,20 @@ caseSplit printCases cases = do cfg <- getConfig
                                 r <- checkSatAssuming [c]
 
                                 case r of
-                                  Unsat -> do notify "Unsatisfiable"
-                                              go cfg ncs
+                                  Unsat    -> do notify "Unsatisfiable"
+                                                 go cfg ncs
 
-                                  Sat   -> do notify "Satisfiable"
-                                              res <- Satisfiable cfg <$> getModel
-                                              return $ Just (n, res)
+                                  Sat      -> do notify "Satisfiable"
+                                                 res <- Satisfiable cfg <$> getModel
+                                                 return $ Just (n, res)
 
-                                  Unk   -> do notify "Unknown"
-                                              res <- Unknown cfg <$> getUnknownReason
-                                              return $ Just (n, res)
+                                  DSat mbP -> do notify $ "Delta satisfiable" ++ maybe "" (" (precision: " ++) mbP
+                                                 res <- DeltaSat cfg mbP <$> getModel
+                                                 return $ Just (n, res)
+
+                                  Unk      -> do notify "Unknown"
+                                                 res <- Unknown cfg <$> getUnknownReason
+                                                 return $ Just (n, res)
 
 -- | Generalization of 'Data.SBV.Control.resetAssertions'
 resetAssertions :: (MonadIO m, MonadQuery m) => m ()
@@ -749,7 +769,7 @@ SBV a |-> v = case literal v of
 mkSMTResult :: (MonadIO m, MonadQuery m) => [Assignment] -> m SMTResult
 mkSMTResult asgns = do
              QueryState{queryConfig} <- getQueryState
-             inps <- getQuantifiedInputs
+             inps <- userInputsToList <$> getQuantifiedInputs
 
              let grabValues st = do let extract (Assign s n) = sbvToSV st (SBV s) >>= \sv -> return (sv, n)
 
@@ -762,8 +782,8 @@ mkSMTResult asgns = do
                                     let userSS = map fst modelAssignment
 
                                         missing, extra, dup :: [String]
-                                        missing = [n | (EX, (s, n)) <- inps, s `notElem` userSS]
-                                        extra   = [show s | s <- userSS, s `notElem` map (fst . snd) inps]
+                                        missing = [T.unpack n | (EX, NamedSymVar s n) <- inps, s `notElem` userSS]
+                                        extra   = [show s | s <- userSS, s `notElem` map (getSV . namedSymVar) inps]
                                         dup     = let walk []     = []
                                                       walk (n:ns)
                                                         | n `elem` ns = show n : walk (filter (/= n) ns)
@@ -794,7 +814,7 @@ mkSMTResult asgns = do
                                                             , "*** Data.SBV: Check your query result construction!"
                                                             ]
 
-                                    let findName s = case [nm | (_, (i, nm)) <- inps, s == i] of
+                                    let findName s = case [T.unpack nm | (_, NamedSymVar i nm) <- inps, s == i] of
                                                         [nm] -> nm
                                                         []   -> error "*** Data.SBV: Impossible happened: Cannot find " ++ show s ++ " in the input list"
                                                         nms  -> error $ unlines [ ""
@@ -814,4 +834,4 @@ mkSMTResult asgns = do
 
              return $ Satisfiable queryConfig m
 
-{-# ANN getModelAtIndex ("HLint: ignore Use forM_"          :: String) #-}
+{-# ANN getModelAtIndex ("HLint: ignore Use forM_" :: String) #-}
