@@ -6,7 +6,7 @@
 -- Maintainer: erkokl@gmail.com
 -- Stability : experimental
 --
--- Algrebraic reals in Haskell.
+-- Algebraic reals in Haskell.
 -----------------------------------------------------------------------------
 
 {-# LANGUAGE FlexibleInstances #-}
@@ -16,6 +16,8 @@
 module Data.SBV.Core.AlgReals (
              AlgReal(..)
            , AlgRealPoly(..)
+           , RationalCV(..)
+           , RealPoint(..), realPoint
            , mkPolyReal
            , algRealToSMTLib2
            , algRealToHaskell
@@ -27,6 +29,8 @@ module Data.SBV.Core.AlgReals (
            )
    where
 
+import Data.Char       (isDigit)
+
 import Data.List       (sortBy, isPrefixOf, partition)
 import Data.Ratio      ((%), numerator, denominator)
 import Data.Function   (on)
@@ -35,13 +39,24 @@ import Test.QuickCheck (Arbitrary(..))
 
 import Numeric (readSigned, readFloat)
 
+-- | Is the endpoint included in the interval?
+data RealPoint a = OpenPoint   a -- ^ open: i.e., doesn't include the point
+                 | ClosedPoint a -- ^ closed: i.e., includes the point
+                 deriving (Show, Eq, Ord)
+
+-- | Extract the point associated with the open-closed point
+realPoint :: RealPoint a -> a
+realPoint (OpenPoint   a) = a
+realPoint (ClosedPoint a) = a
+
 -- | Algebraic reals. Note that the representation is left abstract. We represent
 -- rational results explicitly, while the roots-of-polynomials are represented
 -- implicitly by their defining equation
-data AlgReal = AlgRational Bool Rational                          -- ^ bool says it's exact (i.e., SMT-solver did not return it with ? at the end.)
-             | AlgPolyRoot (Integer,  AlgRealPoly) (Maybe String) -- ^ which root of this polynomial and an approximate decimal representation with given precision, if available
+data AlgReal = AlgRational Bool Rational                             -- ^ bool says it's exact (i.e., SMT-solver did not return it with ? at the end.)
+             | AlgPolyRoot (Integer,  AlgRealPoly) (Maybe String)    -- ^ which root of this polynomial and an approximate decimal representation with given precision, if available
+             | AlgInterval (RealPoint Rational) (RealPoint Rational) -- ^ interval, with low and high bounds
 
--- | Check wheter a given argument is an exact rational
+-- | Check whether a given argument is an exact rational
 isExactRational :: AlgReal -> Bool
 isExactRational (AlgRational True _) = True
 isExactRational _                    = False
@@ -56,9 +71,15 @@ newtype AlgRealPoly = AlgRealPoly [(Integer, Integer)]
 mkPolyReal :: Either (Bool, String) (Integer, [(Integer, Integer)]) -> AlgReal
 mkPolyReal (Left (exact, str))
  = case (str, break (== '.') str) of
-      ("", (_, _))    -> AlgRational exact 0
-      (_, (x, '.':y)) -> AlgRational exact (read (x++y) % (10 ^ length y))
-      (_, (x, _))     -> AlgRational exact (read x % 1)
+      ("", _)                                -> AlgRational exact 0
+      (_, (x, '.':y)) | all isDigit (x ++ y) -> AlgRational exact (read (x++y) % (10 ^ length y))
+      (_, (x, ""))    | all isDigit x        -> AlgRational exact (read x % 1)
+      _                                      -> error $ unlines [ "*** Data.SBV.mkPolyReal: Unable to read a number from:"
+                                                                , "***"
+                                                                , "*** " ++ str
+                                                                , "***"
+                                                                , "*** Please report this as a bug."
+                                                                ]
 mkPolyReal (Right (k, coeffs))
  = AlgPolyRoot (k, AlgRealPoly (normalize coeffs)) Nothing
  where normalize :: [(Integer, Integer)] -> [(Integer, Integer)]
@@ -93,6 +114,11 @@ instance Show AlgReal where
   show (AlgPolyRoot (i, p) mbApprox) = "root(" ++ show i ++ ", " ++ show p ++ ")" ++ maybe "" app mbApprox
      where app v | last v == '?' = " = " ++ init v ++ "..."
                  | True          = " = " ++ v
+  show (AlgInterval a b)         = case (a, b) of
+                                     (OpenPoint   l, OpenPoint   h) -> "(" ++ show l ++ ", " ++ show h ++ ")"
+                                     (OpenPoint   l, ClosedPoint h) -> "(" ++ show l ++ ", " ++ show h ++ "]"
+                                     (ClosedPoint l, OpenPoint   h) -> "[" ++ show l ++ ", " ++ show h ++ ")"
+                                     (ClosedPoint l, ClosedPoint h) -> "[" ++ show l ++ ", " ++ show h ++ "]"
 
 -- lift unary op through an exact rational, otherwise bail
 lift1 :: String -> (Rational -> Rational) -> AlgReal -> AlgReal
@@ -126,8 +152,13 @@ _               `algRealStructuralEqual` _               = False
 algRealStructuralCompare :: AlgReal -> AlgReal -> Ordering
 AlgRational a b `algRealStructuralCompare` AlgRational c d = (a, b) `compare` (c, d)
 AlgRational _ _ `algRealStructuralCompare` AlgPolyRoot _ _ = LT
+AlgRational _ _ `algRealStructuralCompare` AlgInterval _ _ = LT
 AlgPolyRoot _ _ `algRealStructuralCompare` AlgRational _ _ = GT
 AlgPolyRoot a b `algRealStructuralCompare` AlgPolyRoot c d = (a, b) `compare` (c, d)
+AlgPolyRoot _ _ `algRealStructuralCompare` AlgInterval _ _ = LT
+AlgInterval _ _ `algRealStructuralCompare` AlgRational _ _ = GT
+AlgInterval _ _ `algRealStructuralCompare` AlgPolyRoot _ _ = GT
+AlgInterval a b `algRealStructuralCompare` AlgInterval c d = (a, b) `compare` (c, d)
 
 instance Num AlgReal where
   (+)         = lift2 "+"      (+)
@@ -185,6 +216,8 @@ algRealToSMTLib2 (AlgPolyRoot (i, AlgRealPoly xs) _) = "(root-obj (+ " ++ unword
         term (k, p) = ["(* " ++ coeff k ++ " (^ x " ++ show p ++ "))"]
         coeff n | n < 0 = "(- " ++ show (abs n) ++ ")"
                 | True  = show n
+algRealToSMTLib2 r@AlgInterval{}
+   = error $ "SBV: Unexpected inexact rational to be converted to SMTLib2: " ++ show r
 
 -- | Render an 'AlgReal' as a Haskell value. Only supports rationals, since there is no corresponding
 -- standard Haskell type that can represent root-of-polynomial variety.
@@ -198,26 +231,34 @@ algRealToHaskell r                    = error $ unlines [ ""
                                                         , "represents an irrational number, and cannot be converted to a Haskell value."
                                                         ]
 
+-- | Conversion from internal rationals to Haskell values
+data RationalCV = RatIrreducible AlgReal                                   -- ^ Root of a polynomial, cannot be reduced
+                | RatExact       Rational                                  -- ^ An exact rational
+                | RatApprox      Rational                                  -- ^ An approximated value
+                | RatInterval    (RealPoint Rational) (RealPoint Rational) -- ^ Interval. Can be open/closed on both ends.
+                deriving Show
+
 -- | Convert an 'AlgReal' to a 'Rational'. If the 'AlgReal' is exact, then you get a 'Left' value. Otherwise,
 -- you get a 'Right' value which is simply an approximation.
-algRealToRational :: AlgReal -> Either Rational Rational
+algRealToRational :: AlgReal -> RationalCV
 algRealToRational a = case a of
-                        AlgRational True  r        -> Left r
-                        AlgRational False r        -> Left r
-                        AlgPolyRoot _     Nothing  -> bad
+                        AlgRational True  r        -> RatExact r
+                        AlgRational False r        -> RatExact r
+                        AlgPolyRoot _     Nothing  -> RatIrreducible a
                         AlgPolyRoot _     (Just s) -> let trimmed = case reverse s of
                                                                      '.':'.':'.':rest -> reverse rest
                                                                      _                -> s
                                                       in case readSigned readFloat trimmed of
-                                                           [(v, "")] -> Right v
-                                                           _         -> bad
-   where bad = error $ unlines [ ""
-                               , "SBV.algRealToRational: Unsupported argument:"
-                               , ""
-                               , "   " ++ show a
-                               , ""
-                               , "represents an irrational number that cannot be approximated."
-                               ]
+                                                           [(v, "")] -> RatApprox v
+                                                           _         -> bad "represents a value that cannot be converted to a rational"
+                        AlgInterval lo hi          -> RatInterval lo hi
+   where bad w = error $ unlines [ ""
+                                 , "SBV.algRealToRational: Unsupported argument:"
+                                 , ""
+                                 , "   " ++ show a
+                                 , ""
+                                 , w
+                                 ]
 
 -- Try to show a rational precisely if we can, with finite number of
 -- digits. Otherwise, show it as a rational value.
