@@ -11,6 +11,7 @@
 
 {-# LANGUAGE ScopedTypeVariables #-}
 {-# LANGUAGE TypeApplications    #-}
+{-# LANGUAGE Rank2Types          #-}
 
 {-# OPTIONS_GHC -Wall -Werror #-}
 
@@ -28,6 +29,7 @@ import Data.List (isPrefixOf, intercalate)
 
 import Data.SBV.Core.Kind
 import Data.SBV.Core.AlgReals
+import Data.SBV.Core.SizedFloats
 
 import Data.Proxy
 
@@ -69,33 +71,37 @@ instance HasKind a => HasKind (RCSet a) where
   kindOf _ = KSet (kindOf (Proxy @a))
 
 -- | A constant value
-data CVal = CAlgReal  !AlgReal              -- ^ Algebraic real
-          | CInteger  !Integer              -- ^ Bit-vector/unbounded integer
-          | CFloat    !Float                -- ^ Float
-          | CDouble   !Double               -- ^ Double
-          | CChar     !Char                 -- ^ Character
-          | CString   !String               -- ^ String
-          | CList     ![CVal]               -- ^ List
-          | CSet      !(RCSet CVal)         -- ^ Set. Can be regular or complemented.
-          | CUserSort !(Maybe Int, String)  -- ^ Value of an uninterpreted/user kind. The Maybe Int shows index position for enumerations
-          | CTuple    ![CVal]               -- ^ Tuple
-          | CMaybe    !(Maybe CVal)         -- ^ Maybe
-          | CEither   !(Either CVal CVal)   -- ^ Disjoint union
+data CVal = CAlgReal  !AlgReal             -- ^ Algebraic real
+          | CInteger  !Integer             -- ^ Bit-vector/unbounded integer
+          | CFloat    !Float               -- ^ Float
+          | CDouble   !Double              -- ^ Double
+          | CFP       !FP                  -- ^ Arbitrary float
+          | CRational Rational             -- ^ Rational
+          | CChar     !Char                -- ^ Character
+          | CString   !String              -- ^ String
+          | CList     ![CVal]              -- ^ List
+          | CSet      !(RCSet CVal)        -- ^ Set. Can be regular or complemented.
+          | CUserSort !(Maybe Int, String) -- ^ Value of an uninterpreted/user kind. The Maybe Int shows index position for enumerations
+          | CTuple    ![CVal]              -- ^ Tuple
+          | CMaybe    !(Maybe CVal)        -- ^ Maybe
+          | CEither   !(Either CVal CVal)  -- ^ Disjoint union
 
--- | Assing a rank to constant values, this is structural and helps with ordering
+-- | Assign a rank to constant values, this is structural and helps with ordering
 cvRank :: CVal -> Int
 cvRank CAlgReal  {} =  0
 cvRank CInteger  {} =  1
 cvRank CFloat    {} =  2
 cvRank CDouble   {} =  3
-cvRank CChar     {} =  4
-cvRank CString   {} =  5
-cvRank CList     {} =  6
-cvRank CSet      {} =  7
-cvRank CUserSort {} =  8
-cvRank CTuple    {} =  9
-cvRank CMaybe    {} = 10
-cvRank CEither   {} = 11
+cvRank CFP       {} =  4
+cvRank CRational {} =  5
+cvRank CChar     {} =  6
+cvRank CString   {} =  7
+cvRank CList     {} =  8
+cvRank CSet      {} =  9
+cvRank CUserSort {} = 10
+cvRank CTuple    {} = 11
+cvRank CMaybe    {} = 12
+cvRank CEither   {} = 13
 
 -- | Eq instance for CVVal. Note that we cannot simply derive Eq/Ord, since CVAlgReal doesn't have proper
 -- instances for these when values are infinitely precise reals. However, we do
@@ -105,6 +111,7 @@ instance Eq CVal where
   CInteger  a == CInteger  b = a == b
   CFloat    a == CFloat    b = a `fpIsEqualObjectH` b   -- We don't want +0/-0 to be confused; and also we want NaN = NaN here!
   CDouble   a == CDouble   b = a `fpIsEqualObjectH` b   -- ditto
+  CRational a == CRational b = a == b
   CChar     a == CChar     b = a == b
   CString   a == CString   b = a == b
   CList     a == CList     b = a == b
@@ -125,18 +132,20 @@ instance Eq CVal where
 
 -- | Ord instance for VWVal. Same comments as the 'Eq' instance why this cannot be derived.
 instance Ord CVal where
-  CAlgReal  a `compare` CAlgReal b  = a        `algRealStructuralCompare` b
-  CInteger  a `compare` CInteger b  = a        `compare`                  b
-  CFloat    a `compare` CFloat b    = a        `fpCompareObjectH`         b
-  CDouble   a `compare` CDouble b   = a        `fpCompareObjectH`         b
-  CChar     a `compare` CChar b     = a        `compare`                  b
-  CString   a `compare` CString b   = a        `compare`                  b
-  CList     a `compare` CList   b   = a        `compare`                  b
-  CSet      a `compare` CSet    b   = a        `compareRCSet`             b
-  CUserSort a `compare` CUserSort b = a        `compare`                  b
-  CTuple    a `compare` CTuple    b = a        `compare`                  b
-  CMaybe    a `compare` CMaybe    b = a        `compare`                  b
-  CEither   a `compare` CEither   b = a        `compare`                  b
+  CAlgReal  a `compare` CAlgReal  b = a `algRealStructuralCompare` b
+  CInteger  a `compare` CInteger  b = a `compare`                  b
+  CFloat    a `compare` CFloat    b = a `fpCompareObjectH`         b
+  CDouble   a `compare` CDouble   b = a `fpCompareObjectH`         b
+  CRational a `compare` CRational b = a `compare`                  b
+  CFP       a `compare` CFP       b = a `fprCompareObject`         b
+  CChar     a `compare` CChar     b = a `compare`                  b
+  CString   a `compare` CString   b = a `compare`                  b
+  CList     a `compare` CList     b = a `compare`                  b
+  CSet      a `compare` CSet      b = a `compareRCSet`             b
+  CUserSort a `compare` CUserSort b = a `compare`                  b
+  CTuple    a `compare` CTuple    b = a `compare`                  b
+  CMaybe    a `compare` CMaybe    b = a `compare`                  b
+  CEither   a `compare` CEither   b = a `compare`                  b
   a           `compare` b           = let ra = cvRank a
                                           rb = cvRank b
                                       in if ra == rb
@@ -160,7 +169,7 @@ data CV = CV { _cvKind  :: !Kind
 data GeneralizedCV = ExtendedCV ExtCV
                    | RegularCV  CV
 
--- | A simple expression type over extendent values, covering infinity, epsilon and intervals.
+-- | A simple expression type over extended values, covering infinity, epsilon and intervals.
 data ExtCV = Infinite  Kind         -- infinity
            | Epsilon   Kind         -- epsilon
            | Interval  ExtCV ExtCV  -- closed interval
@@ -241,13 +250,22 @@ cvToBool x = cvVal x /= CInteger 0
 normCV :: CV -> CV
 normCV c@(CV (KBounded signed sz) (CInteger v)) = c { cvVal = CInteger norm }
  where norm | sz == 0 = 0
+
             | signed  = let rg = 2 ^ (sz - 1)
                         in case divMod v rg of
                                   (a, b) | even a -> b
                                   (_, b)          -> b - rg
-            | True    = v `mod` (2 ^ sz)
+
+            | True    = {- We really want to do:
+
+                                v `mod` (2 ^ sz)
+
+                           Below is equivalent, and hopefully faster!
+                        -}
+                        v .&. (((1 :: Integer) `shiftL` sz) - 1)
 normCV c@(CV KBool (CInteger v)) = c { cvVal = CInteger (v .&. 1) }
 normCV c                         = c
+{-# INLINE normCV #-}
 
 -- | Constant False as a 'CV'. We represent it using the integer value 0.
 falseCV :: CV
@@ -262,6 +280,8 @@ liftCV :: (AlgReal             -> b)
        -> (Integer             -> b)
        -> (Float               -> b)
        -> (Double              -> b)
+       -> (FP                  -> b)
+       -> (Rational            -> b)
        -> (Char                -> b)
        -> (String              -> b)
        -> ((Maybe Int, String) -> b)
@@ -272,24 +292,28 @@ liftCV :: (AlgReal             -> b)
        -> (Either CVal CVal    -> b)
        -> CV
        -> b
-liftCV f _ _ _ _ _ _ _ _ _ _ _ (CV _ (CAlgReal  v)) = f v
-liftCV _ f _ _ _ _ _ _ _ _ _ _ (CV _ (CInteger  v)) = f v
-liftCV _ _ f _ _ _ _ _ _ _ _ _ (CV _ (CFloat    v)) = f v
-liftCV _ _ _ f _ _ _ _ _ _ _ _ (CV _ (CDouble   v)) = f v
-liftCV _ _ _ _ f _ _ _ _ _ _ _ (CV _ (CChar     v)) = f v
-liftCV _ _ _ _ _ f _ _ _ _ _ _ (CV _ (CString   v)) = f v
-liftCV _ _ _ _ _ _ f _ _ _ _ _ (CV _ (CUserSort v)) = f v
-liftCV _ _ _ _ _ _ _ f _ _ _ _ (CV _ (CList     v)) = f v
-liftCV _ _ _ _ _ _ _ _ f _ _ _ (CV _ (CSet      v)) = f v
-liftCV _ _ _ _ _ _ _ _ _ f _ _ (CV _ (CTuple    v)) = f v
-liftCV _ _ _ _ _ _ _ _ _ _ f _ (CV _ (CMaybe    v)) = f v
-liftCV _ _ _ _ _ _ _ _ _ _ _ f (CV _ (CEither   v)) = f v
+liftCV f _ _ _ _ _ _ _ _ _ _ _ _ _ (CV _ (CAlgReal  v)) = f v
+liftCV _ f _ _ _ _ _ _ _ _ _ _ _ _ (CV _ (CInteger  v)) = f v
+liftCV _ _ f _ _ _ _ _ _ _ _ _ _ _ (CV _ (CFloat    v)) = f v
+liftCV _ _ _ f _ _ _ _ _ _ _ _ _ _ (CV _ (CDouble   v)) = f v
+liftCV _ _ _ _ f _ _ _ _ _ _ _ _ _ (CV _ (CFP       v)) = f v
+liftCV _ _ _ _ _ f _ _ _ _ _ _ _ _ (CV _ (CRational v)) = f v
+liftCV _ _ _ _ _ _ f _ _ _ _ _ _ _ (CV _ (CChar     v)) = f v
+liftCV _ _ _ _ _ _ _ f _ _ _ _ _ _ (CV _ (CString   v)) = f v
+liftCV _ _ _ _ _ _ _ _ f _ _ _ _ _ (CV _ (CUserSort v)) = f v
+liftCV _ _ _ _ _ _ _ _ _ f _ _ _ _ (CV _ (CList     v)) = f v
+liftCV _ _ _ _ _ _ _ _ _ _ f _ _ _ (CV _ (CSet      v)) = f v
+liftCV _ _ _ _ _ _ _ _ _ _ _ f _ _ (CV _ (CTuple    v)) = f v
+liftCV _ _ _ _ _ _ _ _ _ _ _ _ f _ (CV _ (CMaybe    v)) = f v
+liftCV _ _ _ _ _ _ _ _ _ _ _ _ _ f (CV _ (CEither   v)) = f v
 
 -- | Lift a binary function through a 'CV'.
 liftCV2 :: (AlgReal             -> AlgReal             -> b)
         -> (Integer             -> Integer             -> b)
         -> (Float               -> Float               -> b)
         -> (Double              -> Double              -> b)
+        -> (FP                  -> FP                  -> b)
+        -> (Rational            -> Rational            -> b)
         -> (Char                -> Char                -> b)
         -> (String              -> String              -> b)
         -> ([CVal]              -> [CVal]              -> b)
@@ -298,68 +322,78 @@ liftCV2 :: (AlgReal             -> AlgReal             -> b)
         -> (Either CVal CVal    -> Either CVal CVal    -> b)
         -> ((Maybe Int, String) -> (Maybe Int, String) -> b)
         -> CV                   -> CV                  -> b
-liftCV2 r i f d c s u v m e w x y = case (cvVal x, cvVal y) of
-                                      (CAlgReal   a, CAlgReal   b) -> r a b
-                                      (CInteger   a, CInteger   b) -> i a b
-                                      (CFloat     a, CFloat     b) -> f a b
-                                      (CDouble    a, CDouble    b) -> d a b
-                                      (CChar      a, CChar      b) -> c a b
-                                      (CString    a, CString    b) -> s a b
-                                      (CList      a, CList      b) -> u a b
-                                      (CTuple     a, CTuple     b) -> v a b
-                                      (CMaybe     a, CMaybe     b) -> m a b
-                                      (CEither    a, CEither    b) -> e a b
-                                      (CUserSort  a, CUserSort  b) -> w a b
-                                      _                            -> error $ "SBV.liftCV2: impossible, incompatible args received: " ++ show (x, y)
+liftCV2 r i f d af ra c s u v m e w x y = case (cvVal x, cvVal y) of
+                                           (CAlgReal   a, CAlgReal   b) -> r  a b
+                                           (CInteger   a, CInteger   b) -> i  a b
+                                           (CFloat     a, CFloat     b) -> f  a b
+                                           (CDouble    a, CDouble    b) -> d  a b
+                                           (CFP        a, CFP        b) -> af a b
+                                           (CRational  a, CRational  b) -> ra a b
+                                           (CChar      a, CChar      b) -> c  a b
+                                           (CString    a, CString    b) -> s  a b
+                                           (CList      a, CList      b) -> u  a b
+                                           (CTuple     a, CTuple     b) -> v  a b
+                                           (CMaybe     a, CMaybe     b) -> m  a b
+                                           (CEither    a, CEither    b) -> e  a b
+                                           (CUserSort  a, CUserSort  b) -> w  a b
+                                           _                            -> error $ "SBV.liftCV2: impossible, incompatible args received: " ++ show (x, y)
 
 -- | Map a unary function through a 'CV'.
 mapCV :: (AlgReal             -> AlgReal)
       -> (Integer             -> Integer)
       -> (Float               -> Float)
       -> (Double              -> Double)
+      -> (FP                  -> FP)
+      -> (Rational            -> Rational)
       -> (Char                -> Char)
       -> (String              -> String)
       -> ((Maybe Int, String) -> (Maybe Int, String))
       -> CV
       -> CV
-mapCV r i f d c s u x  = normCV $ CV (kindOf x) $ case cvVal x of
-                                                    CAlgReal  a -> CAlgReal  (r a)
-                                                    CInteger  a -> CInteger  (i a)
-                                                    CFloat    a -> CFloat    (f a)
-                                                    CDouble   a -> CDouble   (d a)
-                                                    CChar     a -> CChar     (c a)
-                                                    CString   a -> CString   (s a)
-                                                    CUserSort a -> CUserSort (u a)
-                                                    CList{}     -> error "Data.SBV.mapCV: Unexpected call through mapCV with lists!"
-                                                    CSet{}      -> error "Data.SBV.mapCV: Unexpected call through mapCV with sets!"
-                                                    CTuple{}    -> error "Data.SBV.mapCV: Unexpected call through mapCV with tuples!"
-                                                    CMaybe{}    -> error "Data.SBV.mapCV: Unexpected call through mapCV with maybe!"
-                                                    CEither{}   -> error "Data.SBV.mapCV: Unexpected call through mapCV with either!"
+mapCV r i f d af ra c s u x  = normCV $ CV (kindOf x) $ case cvVal x of
+                                                          CAlgReal  a -> CAlgReal  (r  a)
+                                                          CInteger  a -> CInteger  (i  a)
+                                                          CFloat    a -> CFloat    (f  a)
+                                                          CDouble   a -> CDouble   (d  a)
+                                                          CFP       a -> CFP       (af a)
+                                                          CRational a -> CRational (ra a)
+                                                          CChar     a -> CChar     (c  a)
+                                                          CString   a -> CString   (s  a)
+                                                          CUserSort a -> CUserSort (u  a)
+                                                          CList{}     -> error "Data.SBV.mapCV: Unexpected call through mapCV with lists!"
+                                                          CSet{}      -> error "Data.SBV.mapCV: Unexpected call through mapCV with sets!"
+                                                          CTuple{}    -> error "Data.SBV.mapCV: Unexpected call through mapCV with tuples!"
+                                                          CMaybe{}    -> error "Data.SBV.mapCV: Unexpected call through mapCV with maybe!"
+                                                          CEither{}   -> error "Data.SBV.mapCV: Unexpected call through mapCV with either!"
 
 -- | Map a binary function through a 'CV'.
 mapCV2 :: (AlgReal             -> AlgReal             -> AlgReal)
        -> (Integer             -> Integer             -> Integer)
        -> (Float               -> Float               -> Float)
        -> (Double              -> Double              -> Double)
+       -> (FP                  -> FP                  -> FP)
+       -> (Rational            -> Rational            -> Rational)
        -> (Char                -> Char                -> Char)
        -> (String              -> String              -> String)
        -> ((Maybe Int, String) -> (Maybe Int, String) -> (Maybe Int, String))
        -> CV
        -> CV
        -> CV
-mapCV2 r i f d c s u x y = case (cvSameType x y, cvVal x, cvVal y) of
-                            (True, CAlgReal  a, CAlgReal  b) -> normCV $ CV (kindOf x) (CAlgReal  (r a b))
-                            (True, CInteger  a, CInteger  b) -> normCV $ CV (kindOf x) (CInteger  (i a b))
-                            (True, CFloat    a, CFloat    b) -> normCV $ CV (kindOf x) (CFloat    (f a b))
-                            (True, CDouble   a, CDouble   b) -> normCV $ CV (kindOf x) (CDouble   (d a b))
-                            (True, CChar     a, CChar     b) -> normCV $ CV (kindOf x) (CChar     (c a b))
-                            (True, CString   a, CString   b) -> normCV $ CV (kindOf x) (CString   (s a b))
-                            (True, CUserSort a, CUserSort b) -> normCV $ CV (kindOf x) (CUserSort (u a b))
-                            (True, CList{},     CList{})     -> error "Data.SBV.mapCV2: Unexpected call through mapCV2 with lists!"
-                            (True, CTuple{},    CTuple{})    -> error "Data.SBV.mapCV2: Unexpected call through mapCV2 with tuples!"
-                            (True, CMaybe{},    CMaybe{})    -> error "Data.SBV.mapCV2: Unexpected call through mapCV2 with maybes!"
-                            (True, CEither{},   CEither{})   -> error "Data.SBV.mapCV2: Unexpected call through mapCV2 with eithers!"
-                            _                                -> error $ "SBV.mapCV2: impossible, incompatible args received: " ++ show (x, y)
+mapCV2 r i f d af ra c s u x y = case (cvSameType x y, cvVal x, cvVal y) of
+                                  (True, CAlgReal  a, CAlgReal  b) -> normCV $ CV (kindOf x) (CAlgReal  (r  a b))
+                                  (True, CInteger  a, CInteger  b) -> normCV $ CV (kindOf x) (CInteger  (i  a b))
+                                  (True, CFloat    a, CFloat    b) -> normCV $ CV (kindOf x) (CFloat    (f  a b))
+                                  (True, CDouble   a, CDouble   b) -> normCV $ CV (kindOf x) (CDouble   (d  a b))
+                                  (True, CFP       a, CFP       b) -> normCV $ CV (kindOf x) (CFP       (af a b))
+                                  (True, CRational a, CRational b) -> normCV $ CV (kindOf x) (CRational (ra a b))
+                                  (True, CChar     a, CChar     b) -> normCV $ CV (kindOf x) (CChar     (c  a b))
+                                  (True, CString   a, CString   b) -> normCV $ CV (kindOf x) (CString   (s  a b))
+                                  (True, CUserSort a, CUserSort b) -> normCV $ CV (kindOf x) (CUserSort (u  a b))
+                                  (True, CList{},     CList{})     -> error "Data.SBV.mapCV2: Unexpected call through mapCV2 with lists!"
+                                  (True, CTuple{},    CTuple{})    -> error "Data.SBV.mapCV2: Unexpected call through mapCV2 with tuples!"
+                                  (True, CMaybe{},    CMaybe{})    -> error "Data.SBV.mapCV2: Unexpected call through mapCV2 with maybes!"
+                                  (True, CEither{},   CEither{})   -> error "Data.SBV.mapCV2: Unexpected call through mapCV2 with eithers!"
+                                  _                                -> error $ "Data.SBV.mapCV2: impossible, incompatible args received: " ++ show (x, y)
 
 -- | Show instance for 'CV'.
 instance Show CV where
@@ -373,7 +407,7 @@ instance Show GeneralizedCV where
 -- | Show a CV, with kind info if bool is True
 showCV :: Bool -> CV -> String
 showCV shk w | isBoolean w = show (cvToBool w) ++ (if shk then " :: Bool" else "")
-showCV shk w               = liftCV show show show show show show snd shL shS shT shMaybe shEither w ++ kInfo
+showCV shk w               = liftCV show show show show show show show show snd shL shS shT shMaybe shEither w ++ kInfo
       where kw = kindOf w
 
             kInfo | shk  = " :: " ++ showBaseKind kw
@@ -424,36 +458,47 @@ showCV shk w               = liftCV show show show show show show snd shL shS sh
 
 -- | Create a constant word from an integral.
 mkConstCV :: Integral a => Kind -> a -> CV
-mkConstCV KBool                a = normCV $ CV KBool      (CInteger (toInteger a))
-mkConstCV k@KBounded{}         a = normCV $ CV k          (CInteger (toInteger a))
-mkConstCV KUnbounded           a = normCV $ CV KUnbounded (CInteger (toInteger a))
-mkConstCV KReal                a = normCV $ CV KReal      (CAlgReal (fromInteger (toInteger a)))
-mkConstCV KFloat               a = normCV $ CV KFloat     (CFloat   (fromInteger (toInteger a)))
-mkConstCV KDouble              a = normCV $ CV KDouble    (CDouble  (fromInteger (toInteger a)))
-mkConstCV KChar                a = error $ "Unexpected call to mkConstCV (Char) with value: " ++ show (toInteger a)
-mkConstCV KString              a = error $ "Unexpected call to mkConstCV (String) with value: " ++ show (toInteger a)
-mkConstCV (KUninterpreted s _) a = error $ "Unexpected call to mkConstCV with uninterpreted kind: " ++ s ++ " with value: " ++ show (toInteger a)
-mkConstCV k@KList{}            a = error $ "Unexpected call to mkConstCV (" ++ show k ++ ") with value: " ++ show (toInteger a)
-mkConstCV k@KSet{}             a = error $ "Unexpected call to mkConstCV (" ++ show k ++ ") with value: " ++ show (toInteger a)
-mkConstCV k@KTuple{}           a = error $ "Unexpected call to mkConstCV (" ++ show k ++ ") with value: " ++ show (toInteger a)
-mkConstCV k@KMaybe{}           a = error $ "Unexpected call to mkConstCV (" ++ show k ++ ") with value: " ++ show (toInteger a)
-mkConstCV k@KEither{}          a = error $ "Unexpected call to mkConstCV (" ++ show k ++ ") with value: " ++ show (toInteger a)
+mkConstCV KBool           a = normCV $ CV KBool      (CInteger  (toInteger a))
+mkConstCV k@KBounded{}    a = normCV $ CV k          (CInteger  (toInteger a))
+mkConstCV KUnbounded      a = normCV $ CV KUnbounded (CInteger  (toInteger a))
+mkConstCV KReal           a = normCV $ CV KReal      (CAlgReal  (fromInteger (toInteger a)))
+mkConstCV KFloat          a = normCV $ CV KFloat     (CFloat    (fromInteger (toInteger a)))
+mkConstCV KDouble         a = normCV $ CV KDouble    (CDouble   (fromInteger (toInteger a)))
+mkConstCV k@KFP{}         a = normCV $ CV k          (CFP       (fromInteger (toInteger a)))
+mkConstCV KRational       a = normCV $ CV KRational  (CRational (fromInteger (toInteger a)))
+mkConstCV KChar           a = error $ "Unexpected call to mkConstCV (Char) with value: "   ++ show (toInteger a)
+mkConstCV KString         a = error $ "Unexpected call to mkConstCV (String) with value: " ++ show (toInteger a)
+mkConstCV (KUserSort s _) a = error $ "Unexpected call to mkConstCV with user kind: " ++ s ++ " with value: " ++ show (toInteger a)
+mkConstCV k@KList{}       a = error $ "Unexpected call to mkConstCV (" ++ show k ++ ") with value: " ++ show (toInteger a)
+mkConstCV k@KSet{}        a = error $ "Unexpected call to mkConstCV (" ++ show k ++ ") with value: " ++ show (toInteger a)
+mkConstCV k@KTuple{}      a = error $ "Unexpected call to mkConstCV (" ++ show k ++ ") with value: " ++ show (toInteger a)
+mkConstCV k@KMaybe{}      a = error $ "Unexpected call to mkConstCV (" ++ show k ++ ") with value: " ++ show (toInteger a)
+mkConstCV k@KEither{}     a = error $ "Unexpected call to mkConstCV (" ++ show k ++ ") with value: " ++ show (toInteger a)
 
 -- | Generate a random constant value ('CVal') of the correct kind.
 randomCVal :: Kind -> IO CVal
 randomCVal k =
   case k of
-    KBool              -> CInteger <$> randomRIO (0, 1)
-    KBounded s w       -> CInteger <$> randomRIO (bounds s w)
-    KUnbounded         -> CInteger <$> randomIO
-    KReal              -> CAlgReal <$> randomIO
-    KFloat             -> CFloat   <$> randomIO
-    KDouble            -> CDouble  <$> randomIO
+    KBool              -> CInteger  <$> randomRIO (0, 1)
+    KBounded s w       -> CInteger  <$> randomRIO (bounds s w)
+    KUnbounded         -> CInteger  <$> randomIO
+    KReal              -> CAlgReal  <$> randomIO
+    KFloat             -> CFloat    <$> randomIO
+    KDouble            -> CDouble   <$> randomIO
+    KRational          -> CRational <$> randomIO
+
+    -- Rather bad, but OK
+    KFP eb sb          -> do sgn <- randomRIO (0 :: Integer, 1)
+                             let sign = sgn == 1
+                             e   <- randomRIO (0 :: Integer, 2^eb-1)
+                             s   <- randomRIO (0 :: Integer, 2^sb-1)
+                             pure $ CFP $ fpFromRawRep sign (e, eb) (s, sb)
+
     -- TODO: KString/KChar currently only go for 0..255; include unicode?
     KString            -> do l <- randomRIO (0, 100)
                              CString <$> replicateM l (chr <$> randomRIO (0, 255))
     KChar              -> CChar . chr <$> randomRIO (0, 255)
-    KUninterpreted s _ -> error $ "Unexpected call to randomCVal with uninterpreted kind: " ++ s
+    KUserSort s _      -> error $ "Unexpected call to randomCVal with user kind: " ++ s
     KList ek           -> do l <- randomRIO (0, 100)
                              CList <$> replicateM l (randomCVal ek)
     KSet  ek           -> do i <- randomIO                           -- regular or complement
