@@ -10,36 +10,51 @@
 -----------------------------------------------------------------------------
 
 {-# LANGUAGE DefaultSignatures    #-}
+{-# LANGUAGE FlexibleContexts     #-}
 {-# LANGUAGE FlexibleInstances    #-}
 {-# LANGUAGE Rank2Types           #-}
 {-# LANGUAGE ScopedTypeVariables  #-}
 {-# LANGUAGE TypeApplications     #-}
 {-# LANGUAGE TypeFamilies         #-}
+{-# LANGUAGE TypeOperators        #-}
+{-# LANGUAGE UndecidableInstances #-}
 
 {-# OPTIONS_GHC -Wall -Werror -fno-warn-orphans #-}
 
 module Data.SBV.Core.Floating (
          IEEEFloating(..), IEEEFloatConvertible(..)
-       , sFloatAsSWord32, sDoubleAsSWord64, sWord32AsSFloat, sWord64AsSDouble
-       , blastSFloat, blastSDouble
-       , sFloatAsComparableSWord32, sDoubleAsComparableSWord64
+       , sFloatAsSWord32, sDoubleAsSWord64, sFloatingPointAsSWord
+       , sWord32AsSFloat, sWord64AsSDouble, sWordAsSFloatingPoint
+       , blastSFloat, blastSDouble,  blastSFloatingPoint
+       , sFloatAsComparableSWord32,  sDoubleAsComparableSWord64,  sFloatingPointAsComparableSWord
+       , sComparableSWord32AsSFloat, sComparableSWord64AsSDouble, sComparableSWordAsSFloatingPoint
        ) where
 
-import qualified Data.Numbers.CrackNum as CN (wordToFloat, wordToDouble, floatToWord, doubleToWord)
-
-import Data.Int            (Int8,  Int16,  Int32,  Int64)
-import Data.Word           (Word8, Word16, Word32, Word64)
+import Data.Bits (testBit)
+import Data.Int  (Int8,  Int16,  Int32,  Int64)
+import Data.Word (Word8, Word16, Word32, Word64)
 
 import Data.Proxy
 
 import Data.SBV.Core.AlgReals (isExactRational)
+import Data.SBV.Core.Sized
+import Data.SBV.Core.SizedFloats
 
 import Data.SBV.Core.Data
+import Data.SBV.Core.Kind
 import Data.SBV.Core.Model
 import Data.SBV.Core.Symbolic (addSValOptGoal)
 
 import Data.SBV.Utils.Numeric
 
+import Data.Ratio
+
+import GHC.TypeLits
+
+import LibBF
+
+import Data.SBV.Core.Operations
+
 -- For doctest use only
 --
 -- $setup
@@ -162,9 +177,10 @@ instance IEEEFloating Double
 --
 -- Conversions to float: 'toSFloat' and 'toSDouble' simply return the
 -- nearest representable float from the given type based on the rounding
--- mode provided.
+-- mode provided. Similarly, 'toSFloatingPoint' converts to a generalized
+-- floating-point number with specified exponent and significand bith widths.
 --
--- Conversions from float: 'fromSFloat' and 'fromSDouble' functions do
+-- Conversions from float: 'fromSFloat', 'fromSDouble', 'fromSFloatingPoint' functions do
 -- the reverse conversion. However some care is needed when given values
 -- that are not representable in the integral target domain. For instance,
 -- converting an 'SFloat' to an 'SInt8' is problematic. The rules are as follows:
@@ -222,21 +238,21 @@ class SymVal a => IEEEFloatConvertible a where
   -- Q.E.D.
   -- >>> prove $ roundTrip @Int32
   -- Falsifiable. Counter-example:
-  --   s0 = RoundNearestTiesToEven :: RoundingMode
-  --   s1 =             -264306721 :: Int32
+  --   s0 = RoundTowardNegative :: RoundingMode
+  --   s1 =           826336134 :: Int32
   --
   -- Note how we get a failure on `Int32`. The counter-example value is not representable exactly as a single precision float:
   --
-  -- >>> toRational (-264306721 :: Float)
-  -- (-264306720) % 1
+  -- >>> toRational (826336134 :: Float)
+  -- 826336128 % 1
   --
-  -- Note how the numerator is different, it is off by 1. This is hardly surprising, since floats become sparser as
+  -- Note how the numerator is different, it is off by 6. This is hardly surprising, since floats become sparser as
   -- the magnitude increases to be able to cover all the integer values representable.
   toSFloat :: SRoundingMode -> SBV a -> SFloat
 
   -- default definition if we have an integral like
   default toSFloat :: Integral a => SRoundingMode -> SBV a -> SFloat
-  toSFloat = genericToFloat (Just . fromRational . fromIntegral)
+  toSFloat = genericToFloat (onlyWhenRNE (Just . fromRational . fromIntegral))
 
   -- | Convert from an IEEE74 double precision float.
   fromSDouble :: SRoundingMode -> SDouble -> SBV a
@@ -263,21 +279,37 @@ class SymVal a => IEEEFloatConvertible a where
   -- Q.E.D.
   -- >>> prove $ roundTrip @Int64
   -- Falsifiable. Counter-example:
-  --   s0 =  RoundTowardNegative :: RoundingMode
-  --   s1 = -8069753317450726624 :: Int64
+  --   s0 = RoundNearestTiesToEven :: RoundingMode
+  --   s1 =   -2305879001039372796 :: Int64
   --
   -- Just like in the `SFloat` case, once we reach 64-bits, we no longer can exactly represent the
   -- integer value for all possible values:
   --
-  -- >>>  toRational ( -8069753317450726624 :: Double)
-  -- (-8069753317450726400) % 1
+  -- >>>  toRational (-2305879001039372796 :: Double)
+  -- (-2305879001039372800) % 1
   --
-  -- In this case the numerator is off by 224!
+  -- In this case the numerator is off by 4.
   toSDouble :: SRoundingMode -> SBV a -> SDouble
 
   -- default definition if we have an integral like
   default toSDouble :: Integral a => SRoundingMode -> SBV a -> SDouble
-  toSDouble = genericToFloat (Just . fromRational . fromIntegral)
+  toSDouble = genericToFloat (onlyWhenRNE (Just . fromRational . fromIntegral))
+
+  -- | Convert from an arbitrary floating point.
+  fromSFloatingPoint :: ValidFloat eb sb => SRoundingMode -> SFloatingPoint eb sb -> SBV a
+  fromSFloatingPoint = genericFromFloat
+
+  -- | Convert to an arbitrary floating point.
+  toSFloatingPoint :: ValidFloat eb sb => SRoundingMode -> SBV a -> SFloatingPoint eb sb
+
+  -- -- default definition if we have an integral like
+  default toSFloatingPoint :: (Integral a, ValidFloat eb sb) => SRoundingMode -> SBV a -> SFloatingPoint eb sb
+  toSFloatingPoint = genericToFloat (const (Just . fromRational . fromIntegral))
+
+-- Run the function if the conversion is in RNE. Otherwise return Nothing.
+onlyWhenRNE :: (a -> Maybe b) -> RoundingMode -> a -> Maybe b
+onlyWhenRNE f RoundNearestTiesToEven v = f v
+onlyWhenRNE _ _                      _ = Nothing
 
 -- | A generic from-float converter. Note that this function does no constant folding since
 -- it's behavior is undefined when the input float is out-of-bounds or not a point.
@@ -292,14 +324,14 @@ genericFromFloat rm f = SBV (SVal kTo (Right (cache r)))
                    xsv <- sbvToSV st f
                    newExpr st kTo (SBVApp (IEEEFP (FP_Cast kFrom kTo msv)) [xsv])
 
--- | A generic to-float converter, which will constant-fold as necessary, but only in the sRNE mode.
+-- | A generic to-float converter, which will constant-fold as necessary, but only in the sRNE mode for regular floats.
 genericToFloat :: forall a r. (IEEEFloatConvertible a, IEEEFloating r)
-               => (a -> Maybe r)     -- How to convert concretely, if possible
-               -> SRoundingMode      -- Rounding mode
-               -> SBV a              -- Input convertible
+               => (RoundingMode -> a -> Maybe r)     -- How to convert concretely, if possible
+               -> SRoundingMode                      -- Rounding mode
+               -> SBV a                              -- Input convertible
                -> SBV r
 genericToFloat converter rm i
-  | Just w <- unliteral i, Just RoundNearestTiesToEven <- unliteral rm, Just result <- converter w
+  | Just w <- unliteral i, Just crm <- unliteral rm, Just result <- converter crm w
   = literal result
   | True
   = SBV (SVal kTo (Right (cache r)))
@@ -322,7 +354,9 @@ instance IEEEFloatConvertible Integer
 -- For float and double, skip the conversion if the same and do the constant folding, unlike all others.
 instance IEEEFloatConvertible Float where
   toSFloat  _ f = f
-  toSDouble     = genericToFloat (Just . fp2fp)
+  toSDouble     = genericToFloat (onlyWhenRNE (Just . fp2fp))
+
+  toSFloatingPoint rm f = toSFloatingPoint rm $ toSDouble rm f
 
   fromSFloat  _  f = f
   fromSDouble rm f
@@ -333,9 +367,22 @@ instance IEEEFloatConvertible Float where
     = genericFromFloat rm f
 
 instance IEEEFloatConvertible Double where
-  toSFloat      = genericToFloat (Just . fp2fp)
+  toSFloat      = genericToFloat (onlyWhenRNE (Just . fp2fp))
   toSDouble _ d = d
 
+  toSFloatingPoint rm sd
+    | Just d <- unliteral sd, Just brm <- rmToRM rm
+    = literal $ FloatingPoint $ FP ei si $ fst (bfRoundFloat (mkBFOpts ei si brm) (bfFromDouble d))
+    | True
+    = res
+    where (k, ei, si) = case kindOf res of
+                         kr@(KFP eb sb) -> (kr, eb, sb)
+                         kr             -> error $ "Unexpected kind in toSFloatingPoint: " ++ show (kr, rm, sd)
+          res = SBV $ SVal k $ Right $ cache r
+          r st = do msv <- sbvToSV st rm
+                    xsv <- sbvToSV st sd
+                    newExpr st k (SBVApp (IEEEFP (FP_Cast KDouble k msv)) [xsv])
+
   fromSDouble _  d = d
   fromSFloat  rm d
     | Just RoundNearestTiesToEven <- unliteral rm
@@ -344,39 +391,90 @@ instance IEEEFloatConvertible Double where
     | True
     = genericFromFloat rm d
 
+convertWhenExactRational :: Fractional a => AlgReal -> Maybe a
+convertWhenExactRational r
+  | isExactRational r = Just (fromRational (toRational r))
+  | True              = Nothing
+
 -- For AlgReal; be careful to only process exact rationals concretely
 instance IEEEFloatConvertible AlgReal where
-  toSFloat  = genericToFloat (\r -> if isExactRational r then Just (fromRational (toRational r)) else Nothing)
-  toSDouble = genericToFloat (\r -> if isExactRational r then Just (fromRational (toRational r)) else Nothing)
+  toSFloat         = genericToFloat (onlyWhenRNE convertWhenExactRational)
+  toSDouble        = genericToFloat (onlyWhenRNE convertWhenExactRational)
+  toSFloatingPoint = genericToFloat (const       convertWhenExactRational)
+
+-- Arbitrary floats can handle all rounding modes in concrete mode
+instance ValidFloat eb sb => IEEEFloatConvertible (FloatingPoint eb sb) where
+  toSFloat rm i
+    | Just (FloatingPoint (FP _ _ v)) <- unliteral i, Just brm <- rmToRM rm
+    = literal $ fp2fp $ fst (bfToDouble brm (fst (bfRoundFloat (mkBFOpts ei si brm) v)))
+    | True
+    = genericToFloat (\_ _ -> Nothing) rm i
+    where ei = intOfProxy (Proxy @eb)
+          si = intOfProxy (Proxy @sb)
+
+  fromSFloat rm i
+    | Just f <- unliteral i, Just brm <- rmToRM rm
+    = literal $ FloatingPoint $ FP ei si $ fst (bfRoundFloat (mkBFOpts ei si brm) (bfFromDouble (fp2fp f :: Double)))
+    | True
+    = genericFromFloat rm i
+    where ei = intOfProxy (Proxy @eb)
+          si = intOfProxy (Proxy @sb)
+
+  toSDouble rm i
+    | Just (FloatingPoint (FP _ _ v)) <- unliteral i, Just brm <- rmToRM rm
+    = literal $ fst (bfToDouble brm (fst (bfRoundFloat (mkBFOpts ei si brm) v)))
+    | True
+    = genericToFloat (\_ _ -> Nothing) rm i
+    where ei = intOfProxy (Proxy @eb)
+          si = intOfProxy (Proxy @sb)
+
+  fromSDouble rm i
+    | Just f <- unliteral i, Just brm <- rmToRM rm
+    = literal $ FloatingPoint $ FP ei si $ fst (bfRoundFloat (mkBFOpts ei si brm) (bfFromDouble f))
+    | True
+    = genericFromFloat rm i
+    where ei = intOfProxy (Proxy @eb)
+          si = intOfProxy (Proxy @sb)
+
+  toSFloatingPoint rm i
+    | Just (FloatingPoint (FP _ _ v)) <- unliteral i, Just brm <- rmToRM rm
+    = literal $ FloatingPoint $ FP ei si $ fst (bfRoundFloat (mkBFOpts ei si brm) v)
+    | True
+    = genericToFloat (\_ _ -> Nothing) rm i
+    where ei = intOfProxy (Proxy @eb)
+          si = intOfProxy (Proxy @sb)
+
+  -- From and To are the same when the source is an arbitrary float!
+  fromSFloatingPoint = toSFloatingPoint
 
 -- | Concretely evaluate one arg function, if rounding mode is RoundNearestTiesToEven and we have enough concrete data
 concEval1 :: SymVal a => Maybe (a -> a) -> Maybe SRoundingMode -> SBV a -> Maybe (SBV a)
 concEval1 mbOp mbRm a = do op <- mbOp
                            v  <- unliteral a
                            case unliteral =<< mbRm of
-                             Nothing                     -> (Just . literal) (op v)
-                             Just RoundNearestTiesToEven -> (Just . literal) (op v)
-                             _                           -> Nothing
+                                   Nothing                     -> (Just . literal) (op v)
+                                   Just RoundNearestTiesToEven -> (Just . literal) (op v)
+                                   _                           -> Nothing
 
 -- | Concretely evaluate two arg function, if rounding mode is RoundNearestTiesToEven and we have enough concrete data
 concEval2 :: SymVal a => Maybe (a -> a -> a) -> Maybe SRoundingMode -> SBV a -> SBV a -> Maybe (SBV a)
-concEval2 mbOp mbRm a b  = do op <- mbOp
-                              v1 <- unliteral a
-                              v2 <- unliteral b
-                              case unliteral =<< mbRm of
-                                Nothing                     -> (Just . literal) (v1 `op` v2)
-                                Just RoundNearestTiesToEven -> (Just . literal) (v1 `op` v2)
-                                _                           -> Nothing
+concEval2 mbOp mbRm a b = do op <- mbOp
+                             v1 <- unliteral a
+                             v2 <- unliteral b
+                             case unliteral =<< mbRm of
+                                     Nothing                     -> (Just . literal) (v1 `op` v2)
+                                     Just RoundNearestTiesToEven -> (Just . literal) (v1 `op` v2)
+                                     _                           -> Nothing
 
 -- | Concretely evaluate a bool producing two arg function, if rounding mode is RoundNearestTiesToEven and we have enough concrete data
 concEval2B :: SymVal a => Maybe (a -> a -> Bool) -> Maybe SRoundingMode -> SBV a -> SBV a -> Maybe SBool
-concEval2B mbOp mbRm a b  = do op <- mbOp
-                               v1 <- unliteral a
-                               v2 <- unliteral b
-                               case unliteral =<< mbRm of
-                                 Nothing                     -> (Just . literal) (v1 `op` v2)
-                                 Just RoundNearestTiesToEven -> (Just . literal) (v1 `op` v2)
-                                 _                           -> Nothing
+concEval2B mbOp mbRm a b = do op <- mbOp
+                              v1 <- unliteral a
+                              v2 <- unliteral b
+                              case unliteral =<< mbRm of
+                                      Nothing                     -> (Just . literal) (v1 `op` v2)
+                                      Just RoundNearestTiesToEven -> (Just . literal) (v1 `op` v2)
+                                      _                           -> Nothing
 
 -- | Concretely evaluate two arg function, if rounding mode is RoundNearestTiesToEven and we have enough concrete data
 concEval3 :: SymVal a => Maybe (a -> a -> a -> a) -> Maybe SRoundingMode -> SBV a -> SBV a -> SBV a -> Maybe (SBV a)
@@ -385,9 +483,9 @@ concEval3 mbOp mbRm a b c = do op <- mbOp
                                v2 <- unliteral b
                                v3 <- unliteral c
                                case unliteral =<< mbRm of
-                                 Nothing                     -> (Just . literal) (op v1 v2 v3)
-                                 Just RoundNearestTiesToEven -> (Just . literal) (op v1 v2 v3)
-                                 _                           -> Nothing
+                                       Nothing                     -> (Just . literal) (op v1 v2 v3)
+                                       Just RoundNearestTiesToEven -> (Just . literal) (op v1 v2 v3)
+                                       _                           -> Nothing
 
 -- | Add the converted rounding mode if given as an argument
 addRM :: State -> Maybe SRoundingMode -> [SV] -> IO [SV]
@@ -483,20 +581,7 @@ lift3 w mbOp mbRm a b c
 -- converting it to float, and requiring it to be equivalent to the input. In code-generation mode, we simply map
 -- it to a simple conversion.
 sFloatAsSWord32 :: SFloat -> SWord32
-sFloatAsSWord32 fVal
-  | Just f <- unliteral fVal, not (isNaN f)
-  = literal (CN.floatToWord f)
-  | True
-  = SBV (SVal w32 (Right (cache y)))
-  where w32  = KBounded False 32
-        y st = do cg <- isCodeGenMode st
-                  if cg
-                     then do f <- sbvToSV st fVal
-                             newExpr st w32 (SBVApp (IEEEFP (FP_Reinterpret KFloat w32)) [f])
-                     else do n   <- internalVariable st w32
-                             ysw <- newExpr st KFloat (SBVApp (IEEEFP (FP_Reinterpret w32 KFloat)) [n])
-                             internalConstraint st False [] $ unSBV $ fVal `fpIsEqualObject` SBV (SVal KFloat (Right (cache (\_ -> return ysw))))
-                             return n
+sFloatAsSWord32 (SBV v) = SBV $ svFloatAsSWord32 v
 
 -- | Convert an 'SDouble' to an 'SWord64', preserving the bit-correspondence. Note that since the
 -- representation for @NaN@s are not unique, this function will return a symbolic value when given a
@@ -504,20 +589,7 @@ sFloatAsSWord32 fVal
 --
 -- See the implementation note for 'sFloatAsSWord32', as it applies here as well.
 sDoubleAsSWord64 :: SDouble -> SWord64
-sDoubleAsSWord64 fVal
-  | Just f <- unliteral fVal, not (isNaN f)
-  = literal (CN.doubleToWord f)
-  | True
-  = SBV (SVal w64 (Right (cache y)))
-  where w64  = KBounded False 64
-        y st = do cg <- isCodeGenMode st
-                  if cg
-                     then do f <- sbvToSV st fVal
-                             newExpr st w64 (SBVApp (IEEEFP (FP_Reinterpret KDouble w64)) [f])
-                     else do n   <- internalVariable st w64
-                             ysw <- newExpr st KDouble (SBVApp (IEEEFP (FP_Reinterpret w64 KDouble)) [n])
-                             internalConstraint st False [] $ unSBV $ fVal `fpIsEqualObject` SBV (SVal KDouble (Right (cache (\_ -> return ysw))))
-                             return n
+sDoubleAsSWord64 (SBV v) = SBV $ svDoubleAsSWord64 v
 
 -- | Extract the sign\/exponent\/mantissa of a single-precision float. The output will have
 -- 8 bits in the second argument for exponent, and 23 in the third for the mantissa.
@@ -531,10 +603,19 @@ blastSDouble :: SDouble -> (SBool, [SBool], [SBool])
 blastSDouble = extract . sDoubleAsSWord64
  where extract x = (sTestBit x 63, sExtractBits x [62, 61 .. 52], sExtractBits x [51, 50 .. 0])
 
+-- | Extract the sign\/exponent\/mantissa of an arbitrary precision float. The output will have
+-- @eb@ bits in the second argument for exponent, and @sb-1@ bits in the third for mantissa.
+blastSFloatingPoint :: forall eb sb. (ValidFloat eb sb, KnownNat (eb + sb), BVIsNonZero (eb + sb))
+                    => SFloatingPoint eb sb -> (SBool, [SBool], [SBool])
+blastSFloatingPoint = extract . sFloatingPointAsSWord
+  where ei = intOfProxy (Proxy @eb)
+        si = intOfProxy (Proxy @sb)
+        extract x = (sTestBit x (ei + si - 1), sExtractBits x [ei + si - 2, ei + si - 3 .. si - 1], sExtractBits x [si - 2, si - 3 .. 0])
+
 -- | Reinterpret the bits in a 32-bit word as a single-precision floating point number
 sWord32AsSFloat :: SWord32 -> SFloat
 sWord32AsSFloat fVal
-  | Just f <- unliteral fVal = literal $ CN.wordToFloat f
+  | Just f <- unliteral fVal = literal $ wordToFloat f
   | True                     = SBV (SVal KFloat (Right (cache y)))
   where y st = do xsv <- sbvToSV st fVal
                   newExpr st KFloat (SBVApp (IEEEFP (FP_Reinterpret (kindOf fVal) KFloat)) [xsv])
@@ -542,7 +623,7 @@ sWord32AsSFloat fVal
 -- | Reinterpret the bits in a 32-bit word as a single-precision floating point number
 sWord64AsSDouble :: SWord64 -> SDouble
 sWord64AsSDouble dVal
-  | Just d <- unliteral dVal = literal $ CN.wordToDouble d
+  | Just d <- unliteral dVal = literal $ wordToDouble d
   | True                     = SBV (SVal KDouble (Right (cache y)))
   where y st = do xsv <- sbvToSV st dVal
                   newExpr st KDouble (SBVApp (IEEEFP (FP_Reinterpret (kindOf dVal) KDouble)) [xsv])
@@ -555,6 +636,17 @@ sFloatAsComparableSWord32 :: SFloat -> SWord32
 sFloatAsComparableSWord32 f = ite (fpIsNegativeZero f) (sFloatAsComparableSWord32 0) (fromBitsBE $ sNot sb : ite sb (map sNot rest) rest)
   where (sb : rest) = blastBE $ sFloatAsSWord32 f
 
+-- | Inverse transformation to 'sFloatAsComparableSWord32'. Note that this isn't a perfect inverse, since @-0@ maps to @0@ and back to @0@.
+-- Otherwise, it's faithful:
+--
+-- >>> prove  $ \x -> let f = sComparableSWord32AsSFloat x in fpIsNaN f .|| fpIsNegativeZero f .|| sFloatAsComparableSWord32 f .== x
+-- Q.E.D.
+-- >>> prove $ \x -> fpIsNegativeZero x .|| sComparableSWord32AsSFloat (sFloatAsComparableSWord32 x) `fpIsEqualObject` x
+-- Q.E.D.
+sComparableSWord32AsSFloat :: SWord32 -> SFloat
+sComparableSWord32AsSFloat w = sWord32AsSFloat $ ite sb (fromBitsBE $ sFalse : rest) (fromBitsBE $ map sNot allBits)
+  where allBits@(sb : rest) = blastBE w
+
 -- | Convert a double to a comparable 'SWord64'. The trick is to ignore the
 -- sign of -0, and if it's a negative value flip all the bits, and otherwise
 -- only flip the sign bit. This is known as the lexicographic ordering on doubles
@@ -563,13 +655,24 @@ sDoubleAsComparableSWord64 :: SDouble -> SWord64
 sDoubleAsComparableSWord64 d = ite (fpIsNegativeZero d) (sDoubleAsComparableSWord64 0) (fromBitsBE $ sNot sb : ite sb (map sNot rest) rest)
   where (sb : rest) = blastBE $ sDoubleAsSWord64 d
 
+-- | Inverse transformation to 'sDoubleAsComparableSWord64'. Note that this isn't a perfect inverse, since @-0@ maps to @0@ and back to @0@.
+-- Otherwise, it's faithful:
+--
+-- >>> prove  $ \x -> let d = sComparableSWord64AsSDouble x in fpIsNaN d .|| fpIsNegativeZero d .|| sDoubleAsComparableSWord64 d .== x
+-- Q.E.D.
+-- >>> prove $ \x -> fpIsNegativeZero x .|| sComparableSWord64AsSDouble (sDoubleAsComparableSWord64 x) `fpIsEqualObject` x
+-- Q.E.D.
+sComparableSWord64AsSDouble :: SWord64 -> SDouble
+sComparableSWord64AsSDouble w = sWord64AsSDouble $ ite sb (fromBitsBE $ sFalse : rest) (fromBitsBE $ map sNot allBits)
+  where allBits@(sb : rest) = blastBE w
+
 -- | 'Float' instance for 'Metric' goes through the lexicographic ordering on 'Word32'.
 -- It implicitly makes sure that the value is not @NaN@.
 instance Metric Float where
 
    type MetricSpace Float = Word32
    toMetricSpace          = sFloatAsComparableSWord32
-   fromMetricSpace        = sWord32AsSFloat
+   fromMetricSpace        = sComparableSWord32AsSFloat
 
    msMinimize nm o = do constrain $ sNot $ fpIsNaN o
                         addSValOptGoal $ unSBV `fmap` Minimize nm (toMetricSpace o)
@@ -583,7 +686,7 @@ instance Metric Double where
 
    type MetricSpace Double = Word64
    toMetricSpace           = sDoubleAsComparableSWord64
-   fromMetricSpace         = sWord64AsSDouble
+   fromMetricSpace         = sComparableSWord64AsSDouble
 
    msMinimize nm o = do constrain $ sNot $ fpIsNaN o
                         addSValOptGoal $ unSBV `fmap` Minimize nm (toMetricSpace o)
@@ -591,4 +694,195 @@ instance Metric Double where
    msMaximize nm o = do constrain $ sNot $ fpIsNaN o
                         addSValOptGoal $ unSBV `fmap` Maximize nm (toMetricSpace o)
 
+-- | Real instance for FloatingPoint. NB. The methods haven't been subjected to much testing, so beware of any floating-point snafus here.
+instance ValidFloat eb sb => Real (FloatingPoint eb sb) where
+  toRational (FloatingPoint (FP _ _ r)) = case bfToRep r of
+                                            BFNaN     -> toRational (0/0 :: Double)
+                                            BFRep s n -> case n of
+                                                           Zero    -> 0 % 1
+                                                           Inf     -> (if s == Neg then -1 else 1) % 0
+                                                           Num x y -> -- The value here is x * 2^y
+                                                                      let v :: Integer
+                                                                          v   = 2 ^ abs (fromIntegral y :: Integer)
+                                                                          sgn = if s == Neg then ((-1) *) else id
+                                                                      in if y > 0
+                                                                            then sgn $ x * v % 1
+                                                                            else sgn $ x % v
+
+-- | RealFrac instance for FloatingPoint. NB. The methods haven't been subjected to much testing, so beware of any floating-point snafus here.
+instance ValidFloat eb sb => RealFrac (FloatingPoint eb sb) where
+  properFraction (FloatingPoint f) = (a, FloatingPoint b)
+     where (a, b) = properFraction f
+
+-- | RealFloat instance for FloatingPoint. NB. The methods haven't been subjected to much testing, so beware of any floating-point snafus here.
+instance ValidFloat eb sb => RealFloat (FloatingPoint eb sb) where
+  floatRadix     (FloatingPoint f) = floatRadix     f
+  floatDigits    (FloatingPoint f) = floatDigits    f
+  floatRange     (FloatingPoint f) = floatRange     f
+  isNaN          (FloatingPoint f) = isNaN          f
+  isInfinite     (FloatingPoint f) = isInfinite     f
+  isDenormalized (FloatingPoint f) = isDenormalized f
+  isNegativeZero (FloatingPoint f) = isNegativeZero f
+  isIEEE         (FloatingPoint f) = isIEEE         f
+  decodeFloat    (FloatingPoint f) = decodeFloat    f
+
+  encodeFloat m n = res
+     where res = FloatingPoint $ fpEncodeFloat ei si m n
+           ei = intOfProxy (Proxy @eb)
+           si = intOfProxy (Proxy @sb)
+
+-- | Convert a float to the word containing the corresponding bit pattern
+sFloatingPointAsSWord :: forall eb sb. (ValidFloat eb sb, KnownNat (eb + sb), BVIsNonZero (eb + sb)) => SFloatingPoint eb sb -> SWord (eb + sb)
+sFloatingPointAsSWord (SBV v) = SBV (svFloatingPointAsSWord v)
+
+-- | Convert a float to the correct size word, that can be used in lexicographic ordering. Used in optimization.
+sFloatingPointAsComparableSWord :: forall eb sb. (ValidFloat eb sb, KnownNat (eb + sb), BVIsNonZero (eb + sb)) => SFloatingPoint eb sb -> SWord (eb + sb)
+sFloatingPointAsComparableSWord f = ite (fpIsNegativeZero f) posZero (fromBitsBE $ sNot sb : ite sb (map sNot rest) rest)
+  where posZero     = sFloatingPointAsComparableSWord (0 :: SFloatingPoint eb sb)
+        (sb : rest) = blastBE (sFloatingPointAsSWord f :: SWord (eb + sb))
+
+-- | Inverse transformation to 'sFloatingPointAsComparableSWord'. Note that this isn't a perfect inverse, since @-0@ maps to @0@ and back to @0@.
+-- Otherwise, it's faithful:
+--
+-- >>> prove  $ \x -> let d = sComparableSWordAsSFloatingPoint x in fpIsNaN d .|| fpIsNegativeZero d .|| sFloatingPointAsComparableSWord (d :: SFPHalf) .== x
+-- Q.E.D.
+-- >>> prove $ \x -> fpIsNegativeZero x .|| sComparableSWordAsSFloatingPoint (sFloatingPointAsComparableSWord x) `fpIsEqualObject` (x :: SFPHalf)
+-- Q.E.D.
+sComparableSWordAsSFloatingPoint :: forall eb sb. (KnownNat (eb + sb), BVIsNonZero (eb + sb), ValidFloat eb sb) => SWord (eb + sb) -> SFloatingPoint eb sb
+sComparableSWordAsSFloatingPoint w = sWordAsSFloatingPoint $ ite signBit (fromBitsBE $ sFalse : rest) (fromBitsBE $ map sNot allBits)
+  where allBits@(signBit : rest) = blastBE w
+
+-- | Convert a word to an arbitrary float, by reinterpreting the bits of the word as the corresponding bits of the float.
+sWordAsSFloatingPoint :: forall eb sb. (KnownNat (eb + sb), BVIsNonZero (eb + sb), ValidFloat eb sb) => SWord (eb + sb) -> SFloatingPoint eb sb
+sWordAsSFloatingPoint sw
+   | Just (f :: WordN (eb + sb)) <- unliteral sw
+   = let ext i = f `testBit` i
+         exts  = map ext
+         (s, ebits, sigbits) = (ext (ei + si - 1), exts [ei + si - 2, ei + si - 3 .. si - 1], exts [si - 2, si - 3 .. 0])
+
+         cvt :: [Bool] -> Integer
+         cvt = foldr (\b sofar -> 2 * sofar + if b then 1 else 0) 0 . reverse
+
+         eIntV = cvt ebits
+         sIntV = cvt sigbits
+         fp    = fpFromRawRep s (eIntV, ei) (sIntV, si)
+     in literal $ FloatingPoint fp
+   | True
+   = SBV (SVal kTo (Right (cache y)))
+   where ei   = intOfProxy (Proxy @eb)
+         si   = intOfProxy (Proxy @sb)
+         kTo  = KFP ei si
+         y st = do xsv <- sbvToSV st sw
+                   newExpr st kTo (SBVApp (IEEEFP (FP_Reinterpret (kindOf sw) kTo)) [xsv])
+
+instance (BVIsNonZero (eb + sb), KnownNat (eb + sb), ValidFloat eb sb) => Metric (FloatingPoint eb sb) where
+
+   type MetricSpace (FloatingPoint eb sb) = WordN (eb + sb)
+   toMetricSpace                          = sFloatingPointAsComparableSWord
+   fromMetricSpace                        = sComparableSWordAsSFloatingPoint
+
+   msMinimize nm o = do constrain $ sNot $ fpIsNaN o
+                        addSValOptGoal $ unSBV `fmap` Minimize nm (toMetricSpace o)
+
+   msMaximize nm o = do constrain $ sNot $ fpIsNaN o
+                        addSValOptGoal $ unSBV `fmap` Maximize nm (toMetricSpace o)
+
+-- Map SBV's rounding modes to LibBF's
+rmToRM :: SRoundingMode -> Maybe RoundMode
+rmToRM srm = cvt <$> unliteral srm
+  where cvt RoundNearestTiesToEven = NearEven
+        cvt RoundNearestTiesToAway = NearAway
+        cvt RoundTowardPositive    = ToPosInf
+        cvt RoundTowardNegative    = ToNegInf
+        cvt RoundTowardZero        = ToZero
+
+-- | Lift a 1 arg Big-float op
+lift1FP :: forall eb sb. ValidFloat eb sb =>
+           (BFOpts -> BigFloat -> (BigFloat, Status))
+        -> (Maybe SRoundingMode -> SFloatingPoint eb sb -> SFloatingPoint eb sb)
+        -> SRoundingMode
+        -> SFloatingPoint eb sb
+        -> SFloatingPoint eb sb
+lift1FP bfOp mkDef rm a
+  | Just (FloatingPoint (FP _ _ v)) <- unliteral a
+  , Just brm <- rmToRM rm
+  = literal $ FloatingPoint (FP ei si (fst (bfOp (mkBFOpts ei si brm) v)))
+  | True
+  = mkDef (Just rm) a
+  where ei = intOfProxy (Proxy @eb)
+        si = intOfProxy (Proxy @sb)
+
+-- | Lift a 2 arg Big-float op
+lift2FP :: forall eb sb. ValidFloat eb sb =>
+           (BFOpts -> BigFloat -> BigFloat -> (BigFloat, Status))
+        -> (Maybe SRoundingMode -> SFloatingPoint eb sb -> SFloatingPoint eb sb -> SFloatingPoint eb sb)
+        -> SRoundingMode
+        -> SFloatingPoint eb sb
+        -> SFloatingPoint eb sb
+        -> SFloatingPoint eb sb
+lift2FP bfOp mkDef rm a b
+  | Just (FloatingPoint (FP _ _ v1)) <- unliteral a
+  , Just (FloatingPoint (FP _ _ v2)) <- unliteral b
+  , Just brm <- rmToRM rm
+  = literal $ FloatingPoint (FP ei si (fst (bfOp (mkBFOpts ei si brm) v1 v2)))
+  | True
+  = mkDef (Just rm) a b
+  where ei = intOfProxy (Proxy @eb)
+        si = intOfProxy (Proxy @sb)
+
+-- | Lift a 3 arg Big-float op
+lift3FP :: forall eb sb. ValidFloat eb sb =>
+           (BFOpts -> BigFloat -> BigFloat -> BigFloat -> (BigFloat, Status))
+        -> (Maybe SRoundingMode -> SFloatingPoint eb sb -> SFloatingPoint eb sb -> SFloatingPoint eb sb -> SFloatingPoint eb sb)
+        -> SRoundingMode
+        -> SFloatingPoint eb sb
+        -> SFloatingPoint eb sb
+        -> SFloatingPoint eb sb
+        -> SFloatingPoint eb sb
+lift3FP bfOp mkDef rm a b c
+  | Just (FloatingPoint (FP _ _ v1)) <- unliteral a
+  , Just (FloatingPoint (FP _ _ v2)) <- unliteral b
+  , Just (FloatingPoint (FP _ _ v3)) <- unliteral c
+  , Just brm <- rmToRM rm
+  = literal $ FloatingPoint (FP ei si (fst (bfOp (mkBFOpts ei si brm) v1 v2 v3)))
+  | True
+  = mkDef (Just rm) a b c
+  where ei = intOfProxy (Proxy @eb)
+        si = intOfProxy (Proxy @sb)
+
+-- Sized-floats have a special instance, since it can handle arbitrary rounding modes when it matters.
+instance ValidFloat eb sb => IEEEFloating (FloatingPoint eb sb) where
+  fpAdd  = lift2FP bfAdd      (lift2 FP_Add  (Just (+)))
+  fpSub  = lift2FP bfSub      (lift2 FP_Sub  (Just (-)))
+  fpMul  = lift2FP bfMul      (lift2 FP_Mul  (Just (*)))
+  fpDiv  = lift2FP bfDiv      (lift2 FP_Div  (Just (/)))
+  fpFMA  = lift3FP bfFMA      (lift3 FP_FMA  Nothing)
+  fpSqrt = lift1FP bfSqrt     (lift1 FP_Sqrt (Just sqrt))
+
+  fpRoundToIntegral rm a
+    | Just (FloatingPoint (FP ei si v)) <- unliteral a
+    , Just brm <- rmToRM rm
+    = literal $ FloatingPoint (FP ei si (fst (bfRoundInt brm v)))
+    | True
+    = lift1 FP_RoundToIntegral (Just fpRoundToIntegralH) (Just rm) a
+
+  -- All other operations are agnostic to the rounding mode, hence the defaults are sufficient:
+  --
+  --       fpAbs            :: SBV a -> SBV a
+  --       fpNeg            :: SBV a -> SBV a
+  --       fpRem            :: SBV a -> SBV a -> SBV a
+  --       fpMin            :: SBV a -> SBV a -> SBV a
+  --       fpMax            :: SBV a -> SBV a -> SBV a
+  --       fpIsEqualObject  :: SBV a -> SBV a -> SBool
+  --       fpIsNormal       :: SBV a -> SBool
+  --       fpIsSubnormal    :: SBV a -> SBool
+  --       fpIsZero         :: SBV a -> SBool
+  --       fpIsInfinite     :: SBV a -> SBool
+  --       fpIsNaN          :: SBV a -> SBool
+  --       fpIsNegative     :: SBV a -> SBool
+  --       fpIsPositive     :: SBV a -> SBool
+  --       fpIsNegativeZero :: SBV a -> SBool
+  --       fpIsPositiveZero :: SBV a -> SBool
+  --       fpIsPoint        :: SBV a -> SBool
+
 {-# ANN module ("HLint: ignore Reduce duplication" :: String) #-}
