@@ -25,12 +25,10 @@ module Data.SBV.Core.Sized (
           SWord, WordN, sWord, sWord_, sWords
         -- * Type-sized signed bit-vectors
         , SInt, IntN, sInt, sInt_, sInts
-        -- Bit-vector operations
+        -- * Bit-vector operations
         , bvExtract, (#), zeroExtend, signExtend, bvDrop, bvTake
-        -- Splitting and reconstructing from bytes
+        -- * Splitting and reconstructing from bytes
         , ByteConverter(..)
-        -- Non-zero constraint
-        , IsNonZero
        ) where
 
 import Data.Bits
@@ -38,9 +36,9 @@ import Data.Maybe (fromJust)
 import Data.Proxy (Proxy(..))
 
 import GHC.TypeLits
-import Data.Kind
 
 import Data.SBV.Core.Data
+import Data.SBV.Core.Kind
 import Data.SBV.Core.Model
 import Data.SBV.Core.Operations
 import Data.SBV.Core.Symbolic
@@ -63,24 +61,12 @@ type SWord (n :: Nat) = SBV (WordN n)
 instance Show (WordN n) where
   show (WordN v) = show v
 
--- | Grab the bit-size from the proxy
-intOfProxy :: KnownNat n => Proxy n -> Int
-intOfProxy = fromEnum . natVal
-
--- | Catch 0-width cases
-type ZeroWidth = 'Text "Zero-width BV's are not allowed."
-
--- | Type family to create the appropriate non-zero constraint
-type family IsNonZero (arg :: Nat) :: Constraint where
-   IsNonZero 0 = TypeError ZeroWidth
-   IsNonZero n = ()
-
 -- | 'WordN' has a kind
-instance (KnownNat n, IsNonZero n) => HasKind (WordN n) where
+instance (KnownNat n, BVIsNonZero n) => HasKind (WordN n) where
   kindOf _ = KBounded False (intOfProxy (Proxy @n))
 
 -- | 'SymVal' instance for 'WordN'
-instance (KnownNat n, IsNonZero n) => SymVal (WordN n) where
+instance (KnownNat n, BVIsNonZero n) => SymVal (WordN n) where
    literal  x = genLiteral  (kindOf x) x
    mkSymVal   = genMkSymVar (kindOf (undefined :: WordN n))
    fromCV     = genFromCV
@@ -96,17 +82,17 @@ instance Show (IntN n) where
   show (IntN v) = show v
 
 -- | 'IntN' has a kind
-instance (KnownNat n, IsNonZero n) => HasKind (IntN n) where
+instance (KnownNat n, BVIsNonZero n) => HasKind (IntN n) where
   kindOf _ = KBounded True (intOfProxy (Proxy @n))
 
 -- | 'SymVal' instance for 'IntN'
-instance (KnownNat n, IsNonZero n) => SymVal (IntN n) where
+instance (KnownNat n, BVIsNonZero n) => SymVal (IntN n) where
    literal  x = genLiteral  (kindOf x) x
    mkSymVal   = genMkSymVar (kindOf (undefined :: IntN n))
    fromCV     = genFromCV
 
 -- Lift a unary operation via SVal
-lift1 :: (KnownNat n, IsNonZero n, HasKind (bv n), Integral (bv n), Show (bv n)) => String -> (SVal -> SVal) -> bv n -> bv n
+lift1 :: (KnownNat n, BVIsNonZero n, HasKind (bv n), Integral (bv n), Show (bv n)) => String -> (SVal -> SVal) -> bv n -> bv n
 lift1 nm op x = uc $ op (c x)
   where k = kindOf x
         c = SVal k . Left . CV k . CInteger . toInteger
@@ -114,7 +100,7 @@ lift1 nm op x = uc $ op (c x)
         uc r                                   = error $ "Impossible happened while lifting " ++ show nm ++ " over " ++ show (k, x, r)
 
 -- Lift a binary operation via SVal
-lift2 :: (KnownNat n, IsNonZero n, HasKind (bv n), Integral (bv n), Show (bv n)) => String -> (SVal -> SVal -> SVal) -> bv n -> bv n -> bv n
+lift2 :: (KnownNat n, BVIsNonZero n, HasKind (bv n), Integral (bv n), Show (bv n)) => String -> (SVal -> SVal -> SVal) -> bv n -> bv n -> bv n
 lift2 nm op x y = uc $ c x `op` c y
   where k = kindOf x
         c = SVal k . Left . CV k . CInteger . toInteger
@@ -122,7 +108,7 @@ lift2 nm op x y = uc $ c x `op` c y
         uc r                                   = error $ "Impossible happened while lifting " ++ show nm ++ " over " ++ show (k, x, y, r)
 
 -- Lift a binary operation via SVal where second argument is an Int
-lift2I :: (KnownNat n, IsNonZero n, HasKind (bv n), Integral (bv n), Show (bv n)) => String -> (SVal -> Int -> SVal) -> bv n -> Int -> bv n
+lift2I :: (KnownNat n, BVIsNonZero n, HasKind (bv n), Integral (bv n), Show (bv n)) => String -> (SVal -> Int -> SVal) -> bv n -> Int -> bv n
 lift2I nm op x i = uc $ c x `op` i
   where k = kindOf x
         c = SVal k . Left . CV k . CInteger . toInteger
@@ -130,7 +116,7 @@ lift2I nm op x i = uc $ c x `op` i
         uc r                                   = error $ "Impossible happened while lifting " ++ show nm ++ " over " ++ show (k, x, i, r)
 
 -- Lift a binary operation via SVal where second argument is an Int and returning a Bool
-lift2IB :: (KnownNat n, IsNonZero n, HasKind (bv n), Integral (bv n), Show (bv n)) => String -> (SVal -> Int -> SVal) -> bv n -> Int -> Bool
+lift2IB :: (KnownNat n, BVIsNonZero n, HasKind (bv n), Integral (bv n), Show (bv n)) => String -> (SVal -> Int -> SVal) -> bv n -> Int -> Bool
 lift2IB nm op x i = uc $ c x `op` i
   where k = kindOf x
         c = SVal k . Left . CV k . CInteger . toInteger
@@ -138,17 +124,17 @@ lift2IB nm op x i = uc $ c x `op` i
         uc r                 = error $ "Impossible happened while lifting " ++ show nm ++ " over " ++ show (k, x, i, r)
 
 -- | 'Bounded' instance for 'WordN'
-instance (KnownNat n, IsNonZero n) => Bounded (WordN n) where
+instance (KnownNat n, BVIsNonZero n) => Bounded (WordN n) where
    minBound = WordN 0
    maxBound = let sz = intOfProxy (Proxy @n) in WordN $ 2 ^ sz - 1
 
 -- | 'Bounded' instance for 'IntN'
-instance (KnownNat n, IsNonZero n) => Bounded (IntN n) where
+instance (KnownNat n, BVIsNonZero n) => Bounded (IntN n) where
    minBound = let sz1 = intOfProxy (Proxy @n) - 1 in IntN $ - (2 ^ sz1)
    maxBound = let sz1 = intOfProxy (Proxy @n) - 1 in IntN $ 2 ^ sz1 - 1
 
 -- | 'Num' instance for 'WordN'
-instance (KnownNat n, IsNonZero n) => Num (WordN n) where
+instance (KnownNat n, BVIsNonZero n) => Num (WordN n) where
    (+)         = lift2 "(+)"    svPlus
    (-)         = lift2 "(*)"    svMinus
    (*)         = lift2 "(*)"    svTimes
@@ -158,7 +144,7 @@ instance (KnownNat n, IsNonZero n) => Num (WordN n) where
    fromInteger = WordN . fromJust . svAsInteger . svInteger (kindOf (undefined :: WordN n))
 
 -- | 'Num' instance for 'IntN'
-instance (KnownNat n, IsNonZero n) => Num (IntN n) where
+instance (KnownNat n, BVIsNonZero n) => Num (IntN n) where
    (+)         = lift2 "(+)"    svPlus
    (-)         = lift2 "(*)"    svMinus
    (*)         = lift2 "(*)"    svTimes
@@ -168,35 +154,35 @@ instance (KnownNat n, IsNonZero n) => Num (IntN n) where
    fromInteger = IntN . fromJust . svAsInteger . svInteger (kindOf (undefined :: IntN n))
 
 -- | 'Enum' instance for 'WordN'
-instance (KnownNat n, IsNonZero n) => Enum (WordN n) where
+instance (KnownNat n, BVIsNonZero n) => Enum (WordN n) where
    toEnum   = fromInteger  . toInteger
    fromEnum = fromIntegral . toInteger
 
 -- | 'Enum' instance for 'IntN'
-instance (KnownNat n, IsNonZero n) => Enum (IntN n) where
+instance (KnownNat n, BVIsNonZero n) => Enum (IntN n) where
    toEnum   = fromInteger  . toInteger
    fromEnum = fromIntegral . toInteger
 
 -- | 'Real' instance for 'WordN'
-instance (KnownNat n, IsNonZero n) => Real (WordN n) where
+instance (KnownNat n, BVIsNonZero n) => Real (WordN n) where
    toRational (WordN x) = toRational x
 
 -- | 'Real' instance for 'IntN'
-instance (KnownNat n, IsNonZero n) => Real (IntN n) where
+instance (KnownNat n, BVIsNonZero n) => Real (IntN n) where
    toRational (IntN x) = toRational x
 
 -- | 'Integral' instance for 'WordN'
-instance (KnownNat n, IsNonZero n) => Integral (WordN n) where
+instance (KnownNat n, BVIsNonZero n) => Integral (WordN n) where
    toInteger (WordN x)           = x
    quotRem   (WordN x) (WordN y) = let (q, r) = quotRem x y in (WordN q, WordN r)
 
 -- | 'Integral' instance for 'IntN'
-instance (KnownNat n, IsNonZero n) => Integral (IntN n) where
+instance (KnownNat n, BVIsNonZero n) => Integral (IntN n) where
    toInteger (IntN x)          = x
    quotRem   (IntN x) (IntN y) = let (q, r) = quotRem x y in (IntN q, IntN r)
 
 --  'Bits' instance for 'WordN'
-instance (KnownNat n, IsNonZero n) => Bits (WordN n) where
+instance (KnownNat n, BVIsNonZero n) => Bits (WordN n) where
    (.&.)        = lift2   "(.&.)"      svAnd
    (.|.)        = lift2   "(.|.)"      svOr
    xor          = lift2   "xor"        svXOr
@@ -213,7 +199,7 @@ instance (KnownNat n, IsNonZero n) => Bits (WordN n) where
    popCount     = fromIntegral . popCount . toInteger
 
 --  'Bits' instance for 'IntN'
-instance (KnownNat n, IsNonZero n) => Bits (IntN n) where
+instance (KnownNat n, BVIsNonZero n) => Bits (IntN n) where
    (.&.)        = lift2   "(.&.)"      svAnd
    (.|.)        = lift2   "(.|.)"      svOr
    xor          = lift2   "xor"        svXOr
@@ -230,94 +216,94 @@ instance (KnownNat n, IsNonZero n) => Bits (IntN n) where
    popCount     = fromIntegral . popCount . toInteger
 
 -- | 'SIntegral' instance for 'WordN'
-instance (KnownNat n, IsNonZero n) => SIntegral (WordN n)
+instance (KnownNat n, BVIsNonZero n) => SIntegral (WordN n)
 
 -- | 'SIntegral' instance for 'IntN'
-instance (KnownNat n, IsNonZero n) => SIntegral (IntN n)
+instance (KnownNat n, BVIsNonZero n) => SIntegral (IntN n)
 
 -- | 'SDivisible' instance for 'WordN'
-instance (KnownNat n, IsNonZero n) => SDivisible (WordN n) where
+instance (KnownNat n, BVIsNonZero n) => SDivisible (WordN n) where
   sQuotRem x 0 = (0, x)
   sQuotRem x y = x `quotRem` y
   sDivMod  x 0 = (0, x)
   sDivMod  x y = x `divMod` y
 
 -- | 'SDivisible' instance for 'IntN'
-instance (KnownNat n, IsNonZero n) => SDivisible (IntN n) where
+instance (KnownNat n, BVIsNonZero n) => SDivisible (IntN n) where
   sQuotRem x 0 = (0, x)
   sQuotRem x y = x `quotRem` y
   sDivMod  x 0 = (0, x)
   sDivMod  x y = x `divMod` y
 
 -- | 'SDivisible' instance for 'SWord'
-instance (KnownNat n, IsNonZero n) => SDivisible (SWord n) where
+instance (KnownNat n, BVIsNonZero n) => SDivisible (SWord n) where
   sQuotRem = liftQRem
   sDivMod  = liftDMod
 
 -- | 'SDivisible' instance for 'SInt'
-instance (KnownNat n, IsNonZero n) => SDivisible (SInt n) where
+instance (KnownNat n, BVIsNonZero n) => SDivisible (SInt n) where
   sQuotRem = liftQRem
   sDivMod  = liftDMod
 
 -- | 'SFiniteBits' instance for 'WordN'
-instance (KnownNat n, IsNonZero n) => SFiniteBits (WordN n) where
+instance (KnownNat n, BVIsNonZero n) => SFiniteBits (WordN n) where
    sFiniteBitSize _ = intOfProxy (Proxy @n)
 
 -- | 'SFiniteBits' instance for 'IntN'
-instance (KnownNat n, IsNonZero n) => SFiniteBits (IntN n) where
+instance (KnownNat n, BVIsNonZero n) => SFiniteBits (IntN n) where
    sFiniteBitSize _ = intOfProxy (Proxy @n)
 
 -- | Constructing models for 'WordN'
-instance (KnownNat n, IsNonZero n) => SatModel (WordN n) where
+instance (KnownNat n, BVIsNonZero n) => SatModel (WordN n) where
   parseCVs = genParse (kindOf (undefined :: WordN n))
 
 -- | Constructing models for 'IntN'
-instance (KnownNat n, IsNonZero n) => SatModel (IntN n) where
+instance (KnownNat n, BVIsNonZero n) => SatModel (IntN n) where
   parseCVs = genParse (kindOf (undefined :: IntN n))
 
 -- | Optimizing 'WordN'
-instance (KnownNat n, IsNonZero n) => Metric (WordN n)
+instance (KnownNat n, BVIsNonZero n) => Metric (WordN n)
 
 -- | Optimizing 'IntN'
-instance (KnownNat n, IsNonZero n) => Metric (IntN n) where
+instance (KnownNat n, BVIsNonZero n) => Metric (IntN n) where
   type MetricSpace (IntN n) = WordN n
   toMetricSpace    x        = sFromIntegral x + 2 ^ (intOfProxy (Proxy @n) - 1)
   fromMetricSpace  x        = sFromIntegral x - 2 ^ (intOfProxy (Proxy @n) - 1)
 
 -- | Generalization of 'Data.SBV.sWord'
-sWord :: (KnownNat n, IsNonZero n) => MonadSymbolic m => String -> m (SWord n)
+sWord :: (KnownNat n, BVIsNonZero n) => MonadSymbolic m => String -> m (SWord n)
 sWord = symbolic
 
 -- | Generalization of 'Data.SBV.sWord_'
-sWord_ :: (KnownNat n, IsNonZero n) => MonadSymbolic m => m (SWord n)
+sWord_ :: (KnownNat n, BVIsNonZero n) => MonadSymbolic m => m (SWord n)
 sWord_ = free_
 
 -- | Generalization of 'Data.SBV.sWord64s'
-sWords :: (KnownNat n, IsNonZero n) => MonadSymbolic m => [String] -> m [SWord n]
+sWords :: (KnownNat n, BVIsNonZero n) => MonadSymbolic m => [String] -> m [SWord n]
 sWords = symbolics
 
 -- | Generalization of 'Data.SBV.sInt'
-sInt :: (KnownNat n, IsNonZero n) => MonadSymbolic m => String -> m (SInt n)
+sInt :: (KnownNat n, BVIsNonZero n) => MonadSymbolic m => String -> m (SInt n)
 sInt = symbolic
 
 -- | Generalization of 'Data.SBV.sInt_'
-sInt_ :: (KnownNat n, IsNonZero n) => MonadSymbolic m => m (SInt n)
+sInt_ :: (KnownNat n, BVIsNonZero n) => MonadSymbolic m => m (SInt n)
 sInt_ = free_
 
 -- | Generalization of 'Data.SBV.sInts'
-sInts :: (KnownNat n, IsNonZero n) => MonadSymbolic m => [String] -> m [SInt n]
+sInts :: (KnownNat n, BVIsNonZero n) => MonadSymbolic m => [String] -> m [SInt n]
 sInts = symbolics
 
 -- | Extract a portion of bits to form a smaller bit-vector.
 --
 -- >>> prove $ \x -> bvExtract (Proxy @7) (Proxy @3) (x :: SWord 12) .== bvDrop (Proxy @4) (bvTake (Proxy @9) x)
 -- Q.E.D.
-bvExtract :: forall i j n bv proxy. ( KnownNat n, IsNonZero n, SymVal (bv n)
+bvExtract :: forall i j n bv proxy. ( KnownNat n, BVIsNonZero n, SymVal (bv n)
                                     , KnownNat i
                                     , KnownNat j
                                     , i + 1 <= n
                                     , j <= i
-                                    , IsNonZero (i - j + 1)
+                                    , BVIsNonZero (i - j + 1)
                                     ) => proxy i                -- ^ @i@: Start position, numbered from @n-1@ to @0@
                                       -> proxy j                -- ^ @j@: End position, numbered from @n-1@ to @0@, @j <= i@ must hold
                                       -> SBV (bv n)             -- ^ Input bit vector of size @n@
@@ -330,8 +316,8 @@ bvExtract start end = SBV . svExtract i j . unSBV
 --
 -- >>> prove $ \x y -> x .== bvExtract (Proxy @79) (Proxy @71) ((x :: SWord 9) # (y :: SWord 71))
 -- Q.E.D.
-(#) :: ( KnownNat n, IsNonZero n, SymVal (bv n)
-       , KnownNat m, IsNonZero m, SymVal (bv m)
+(#) :: ( KnownNat n, BVIsNonZero n, SymVal (bv n)
+       , KnownNat m, BVIsNonZero m, SymVal (bv m)
        ) => SBV (bv n)                     -- ^ First input, of size @n@, becomes the left side
          -> SBV (bv m)                     -- ^ Second input, of size @m@, becomes the right side
          -> SBV (bv (n + m))               -- ^ Concatenation, of size @n+m@
@@ -342,11 +328,11 @@ infixr 5 #
 --
 -- >>> prove $ \x -> bvExtract (Proxy @20) (Proxy @12) (zeroExtend (x :: SInt 12) :: SInt 21) .== 0
 -- Q.E.D.
-zeroExtend :: forall n m bv. ( KnownNat n, IsNonZero n, SymVal (bv n)
-                             , KnownNat m, IsNonZero m, SymVal (bv m)
+zeroExtend :: forall n m bv. ( KnownNat n, BVIsNonZero n, SymVal (bv n)
+                             , KnownNat m, BVIsNonZero m, SymVal (bv m)
                              , n + 1 <= m
                              , SIntegral (bv (m - n))
-                             , IsNonZero (m - n)
+                             , BVIsNonZero (m - n)
                              ) => SBV (bv n)    -- ^ Input, of size @n@
                                -> SBV (bv m)    -- ^ Output, of size @m@. @n < m@ must hold
 zeroExtend n = SBV $ svJoin (unSBV zero) (unSBV n)
@@ -359,12 +345,12 @@ zeroExtend n = SBV $ svJoin (unSBV zero) (unSBV n)
 -- Q.E.D.
 -- >>> prove $ \x ->       msb x  .=> bvExtract (Proxy @20) (Proxy @12) (signExtend (x :: SInt 12) :: SInt 21) .== complement 0
 -- Q.E.D.
-signExtend :: forall n m bv. ( KnownNat n, IsNonZero n, SymVal (bv n)
-                             , KnownNat m, IsNonZero m, SymVal (bv m)
+signExtend :: forall n m bv. ( KnownNat n, BVIsNonZero n, SymVal (bv n)
+                             , KnownNat m, BVIsNonZero m, SymVal (bv m)
                              , n + 1 <= m
                              , SFiniteBits (bv n)
                              , SIntegral   (bv (m - n))
-                             , IsNonZero   (m - n)
+                             , BVIsNonZero   (m - n)
                              ) => SBV (bv n)  -- ^ Input, of size @n@
                                -> SBV (bv m)  -- ^ Output, of size @m@. @n < m@ must hold
 signExtend n = SBV $ svJoin (unSBV ext) (unSBV n)
@@ -379,11 +365,11 @@ signExtend n = SBV $ svJoin (unSBV ext) (unSBV n)
 -- Q.E.D.
 -- >>> prove $ \x -> bvDrop (Proxy @20) (x :: SWord 21) .== ite (lsb x) 1 0
 -- Q.E.D.
-bvDrop :: forall i n m bv proxy. ( KnownNat n, IsNonZero n
+bvDrop :: forall i n m bv proxy. ( KnownNat n, BVIsNonZero n
                                  , KnownNat i
                                  , i + 1 <= n
                                  , i + m - n <= 0
-                                 , IsNonZero (n - i)
+                                 , BVIsNonZero (n - i)
                                  ) => proxy i                    -- ^ @i@: Number of bits to drop. @i < n@ must hold.
                                    -> SBV (bv n)                 -- ^ Input, of size @n@
                                    -> SBV (bv m)                 -- ^ Output, of size @m@. @m = n - i@ holds.
@@ -399,8 +385,8 @@ bvDrop i = SBV . svExtract start 0 . unSBV
 -- Q.E.D.
 -- >>> prove $ \x -> bvTake (Proxy @4) x # bvDrop (Proxy @4) x .== (x :: SWord 23)
 -- Q.E.D.
-bvTake :: forall i n bv proxy. ( KnownNat n, IsNonZero n
-                               , KnownNat i, IsNonZero i
+bvTake :: forall i n bv proxy. ( KnownNat n, BVIsNonZero n
+                               , KnownNat i, BVIsNonZero i
                                , i <= n
                                ) => proxy i                  -- ^ @i@: Number of bits to take. @0 < i <= n@ must hold.
                                  -> SBV (bv n)               -- ^ Input, of size @n@
