@@ -23,13 +23,10 @@ module Data.SBV.Dynamic
   , HasKind(..), Kind(..), CV(..), CVal(..), cvToBool
   -- *** SMT Arrays of symbolic values
   , SArr, readSArr, writeSArr, mergeSArr, newSArr, eqSArr
-  -- *** Functional arrays of symbolic values
-  , SFunArr, readSFunArr, writeSFunArr, mergeSFunArr, newSFunArr
-
   -- ** Creating a symbolic variable
   , Symbolic
   , Quantifier(..)
-  , svMkSymVar
+  , svMkSymVar, svNewVar_, svNewVar
   , sWordN, sWordN_, sIntN, sIntN_
   -- ** Operations on symbolic values
   -- *** Boolean literals
@@ -37,11 +34,11 @@ module Data.SBV.Dynamic
   -- *** Integer literals
   , svInteger, svAsInteger
   -- *** Float literals
-  , svFloat, svDouble
+  , svFloat, svDouble, svFloatingPoint
   -- *** Algebraic reals (only from rationals)
   , svReal, svNumerator, svDenominator
   -- *** Symbolic equality
-  , svEqual, svNotEqual
+  , svEqual, svNotEqual, svStrongEqual
   -- *** Constructing concrete lists
   , svEnumFromThenTo
   -- *** Symbolic ordering
@@ -95,7 +92,7 @@ module Data.SBV.Dynamic
   -- ** Programmable model extraction
   , genParse, getModelAssignment, getModelDictionary
   -- * SMT Interface: Configurations and solvers
-  , SMTConfig(..), SMTLibVersion(..), Solver(..), SMTSolver(..), boolector, cvc4, yices, z3, mathSAT, abc, defaultSolverConfig, defaultSMTCfg, sbvCheckSolverInstallation, sbvAvailableSolvers
+  , SMTConfig(..), SMTLibVersion(..), Solver(..), SMTSolver(..), boolector, cvc4, yices, z3, mathSAT, abc, defaultSolverConfig, defaultSMTCfg, sbvCheckSolverInstallation, getAvailableSolvers
 
   -- * Symbolic computations
   , outputSVal
@@ -128,6 +125,8 @@ module Data.SBV.Dynamic
   , generateSMTBenchmark
   ) where
 
+import Control.Monad.Trans (liftIO)
+
 import Data.Map.Strict (Map)
 
 import Data.SBV.Core.Kind
@@ -147,7 +146,7 @@ import Data.SBV.Compilers.C       (compileToC, compileToCLib)
 
 import Data.SBV.Provers.Prover (boolector, cvc4, yices, z3, mathSAT, abc, defaultSMTCfg)
 import Data.SBV.SMT.SMT        (ThmResult(..), SatResult(..), SafeResult(..), OptimizeResult(..), AllSatResult(..), genParse)
-import Data.SBV                (sbvCheckSolverInstallation, defaultSolverConfig, sbvAvailableSolvers)
+import Data.SBV                (sbvCheckSolverInstallation, defaultSolverConfig, getAvailableSolvers)
 
 import qualified Data.SBV                as SBV (SBool, proveWithAll, proveWithAny, satWithAll, satWithAny
                                                 , proveConcurrentWithAll, proveConcurrentWithAny
@@ -249,3 +248,11 @@ getModelAssignment = SBV.getModelAssignment
 -- their respective values as returned by the SMT solver. Also see `Data.SBV.SMT.getModelDictionaries`.
 getModelDictionary :: SMTResult -> Map String CV
 getModelDictionary = SBV.getModelDictionary
+
+-- | Create a named fresh existential variable in the current context
+svNewVar :: MonadSymbolic m => Kind -> String -> m SVal
+svNewVar k n = symbolicEnv >>= liftIO . svMkSymVar (NonQueryVar (Just EX)) k (Just n)
+
+-- | Create an unnamed fresh existential variable in the current context
+svNewVar_ :: MonadSymbolic m => Kind -> m SVal
+svNewVar_ k = symbolicEnv >>= liftIO . svMkSymVar (NonQueryVar (Just EX)) k Nothing
