@@ -30,10 +30,15 @@ abc = SMTSolver {
          , engine       = standardEngine "SBV_ABC" "SBV_ABC_OPTIONS"
          , capabilities = SolverCapabilities {
                                 supportsQuantifiers        = False
+                              , supportsDefineFun          = True
+                              , supportsDistinct           = True
+                              , supportsBitVectors         = True
                               , supportsUninterpretedSorts = False
                               , supportsUnboundedInts      = False
+                              , supportsInt2bv             = False
                               , supportsReals              = False
                               , supportsApproxReals        = False
+                              , supportsDeltaSat           = Nothing
                               , supportsIEEE754            = False
                               , supportsSets               = False
                               , supportsOptimization       = False
@@ -41,6 +46,7 @@ abc = SMTSolver {
                               , supportsCustomQueries      = False
                               , supportsGlobalDecls        = False
                               , supportsDataTypes          = False
+                              , supportsDirectAccessors    = False
                               , supportsFlattenedModels    = Nothing
                               }
          }
