@@ -22,7 +22,7 @@ import Data.SBV.Control.Types
 
 -- | The description of the MathSAT SMT solver
 -- The default executable is @\"mathsat\"@, which must be in your path. You can use the @SBV_MATHSAT@ environment variable to point to the executable on your system.
--- The default options are @\"-input=smt2\"@. You can use the @SBV_MATHSAT_OPTIONS@ environment variable to override the options.
+-- You can use the @SBV_MATHSAT_OPTIONS@ environment variable to override the options.
 mathSAT :: SMTSolver
 mathSAT = SMTSolver {
            name         = MathSAT
@@ -32,10 +32,15 @@ mathSAT = SMTSolver {
          , engine       = standardEngine "SBV_MATHSAT" "SBV_MATHSAT_OPTIONS"
          , capabilities = SolverCapabilities {
                                 supportsQuantifiers        = True
+                              , supportsDefineFun          = True
+                              , supportsDistinct           = True
+                              , supportsBitVectors         = True
                               , supportsUninterpretedSorts = True
                               , supportsUnboundedInts      = True
+                              , supportsInt2bv             = False
                               , supportsReals              = True
                               , supportsApproxReals        = False
+                              , supportsDeltaSat           = Nothing
                               , supportsIEEE754            = True
                               , supportsSets               = False
                               , supportsOptimization       = False
@@ -43,6 +48,7 @@ mathSAT = SMTSolver {
                               , supportsCustomQueries      = True
                               , supportsGlobalDecls        = True
                               , supportsDataTypes          = True
+                              , supportsDirectAccessors    = True
                               , supportsFlattenedModels    = Nothing
                               }
          }
