@@ -20,7 +20,7 @@ import Data.SBV.SMT.SMT
 
 -- | The description of the Yices SMT solver
 -- The default executable is @\"yices-smt2\"@, which must be in your path. You can use the @SBV_YICES@ environment variable to point to the executable on your system.
--- SBV does not pass any arguments to yices. You can use the @SBV_YICES_OPTIONS@ environment variable to override the options.
+-- You can use the @SBV_YICES_OPTIONS@ environment variable to override the options.
 yices :: SMTSolver
 yices = SMTSolver {
            name         = Yices
@@ -30,17 +30,23 @@ yices = SMTSolver {
          , engine       = standardEngine "SBV_YICES" "SBV_YICES_OPTIONS"
          , capabilities = SolverCapabilities {
                                 supportsQuantifiers        = False
+                              , supportsDefineFun          = True
+                              , supportsDistinct           = True
+                              , supportsBitVectors         = True
                               , supportsUninterpretedSorts = True
                               , supportsUnboundedInts      = True
+                              , supportsInt2bv             = False
                               , supportsReals              = True
                               , supportsApproxReals        = False
+                              , supportsDeltaSat           = Nothing
                               , supportsIEEE754            = False
                               , supportsSets               = False
                               , supportsOptimization       = False
                               , supportsPseudoBooleans     = False
                               , supportsCustomQueries      = True
-                              , supportsGlobalDecls        = False
+                              , supportsGlobalDecls        = True
                               , supportsDataTypes          = False
+                              , supportsDirectAccessors    = False
                               , supportsFlattenedModels    = Nothing
                               }
          }
