@@ -20,7 +20,7 @@ import Data.SBV.SMT.SMT
 
 -- | The description of the Z3 SMT solver.
 -- The default executable is @\"z3\"@, which must be in your path. You can use the @SBV_Z3@ environment variable to point to the executable on your system.
--- The default options are @\"-nw -in -smt2\"@. You can use the @SBV_Z3_OPTIONS@ environment variable to override the options.
+-- You can use the @SBV_Z3_OPTIONS@ environment variable to override the options.
 z3 :: SMTSolver
 z3 = SMTSolver {
            name         = Z3
@@ -30,10 +30,15 @@ z3 = SMTSolver {
          , engine       = standardEngine "SBV_Z3" "SBV_Z3_OPTIONS"
          , capabilities = SolverCapabilities {
                                 supportsQuantifiers        = True
+                              , supportsDefineFun          = True
+                              , supportsDistinct           = True
+                              , supportsBitVectors         = True
                               , supportsUninterpretedSorts = True
                               , supportsUnboundedInts      = True
+                              , supportsInt2bv             = True
                               , supportsReals              = True
                               , supportsApproxReals        = True
+                              , supportsDeltaSat           = Nothing
                               , supportsIEEE754            = True
                               , supportsSets               = True
                               , supportsOptimization       = True
@@ -41,6 +46,7 @@ z3 = SMTSolver {
                               , supportsCustomQueries      = True
                               , supportsGlobalDecls        = True
                               , supportsDataTypes          = True
+                              , supportsDirectAccessors    = False -- Needs ascriptions. (See the CVC4 version of this)
                               , supportsFlattenedModels    = Just [ "(set-option :pp.max_depth      4294967295)"
                                                                   , "(set-option :pp.min_alias_size 4294967295)"
                                                                   , "(set-option :model.inline_def  true      )"
