@@ -10,10 +10,14 @@
 -----------------------------------------------------------------------------
 
 {-# LANGUAGE DefaultSignatures          #-}
+{-# LANGUAGE FlexibleInstances          #-}
 {-# LANGUAGE GeneralizedNewtypeDeriving #-}
 {-# LANGUAGE NamedFieldPuns             #-}
+{-# LANGUAGE OverloadedStrings          #-}
 {-# LANGUAGE Rank2Types                 #-}
 {-# LANGUAGE ScopedTypeVariables        #-}
+{-# LANGUAGE TypeApplications           #-}
+{-# LANGUAGE ViewPatterns               #-}
 
 {-# OPTIONS_GHC -Wall -Werror #-}
 
@@ -48,6 +52,9 @@ import Data.Int           (Int8, Int16, Int32, Int64)
 import Data.List          (intercalate, isPrefixOf, transpose, isInfixOf)
 import Data.Word          (Word8, Word16, Word32, Word64)
 
+import GHC.TypeLits
+import Data.Proxy
+
 import Data.IORef (readIORef, writeIORef)
 
 import Data.Time          (getZonedTime, defaultTimeLocale, formatTime, diffUTCTime, getCurrentTime)
@@ -59,12 +66,15 @@ import System.IO          (hClose, hFlush, hPutStrLn, hGetContents, hGetLine)
 import System.Process     (runInteractiveProcess, waitForProcess, terminateProcess)
 
 import qualified Data.Map.Strict as M
+import qualified Data.Text       as T
 
 import Data.SBV.Core.AlgReals
 import Data.SBV.Core.Data
 import Data.SBV.Core.Symbolic (SMTEngine, State(..))
 import Data.SBV.Core.Concrete (showCV)
-import Data.SBV.Core.Kind     (showBaseKind)
+import Data.SBV.Core.Kind     (showBaseKind, intOfProxy)
+
+import Data.SBV.Core.SizedFloats(FloatingPoint(..))
 
 import Data.SBV.SMT.Utils     (showTimeoutValue, alignPlain, debug, mergeSExpr, SBVException(..))
 
@@ -75,10 +85,15 @@ import Data.SBV.Utils.TDiff     (Timing(..), showTDiff)
 
 import qualified System.Timeout as Timeout (timeout)
 
+import Numeric
+
+import qualified Data.SBV.Utils.CrackNum as CN
+
 -- | Extract the final configuration from a result
 resultConfig :: SMTResult -> SMTConfig
 resultConfig (Unsatisfiable c _  ) = c
 resultConfig (Satisfiable   c _  ) = c
+resultConfig (DeltaSat      c _ _) = c
 resultConfig (SatExtField   c _  ) = c
 resultConfig (Unknown       c _  ) = c
 resultConfig (ProofError    c _ _) = c
@@ -92,11 +107,13 @@ newtype ThmResult = ThmResult SMTResult
 newtype SatResult = SatResult SMTResult
                   deriving NFData
 
--- | An 'Data.SBV.allSat' call results in a 'AllSatResult'. The first boolean says whether we
--- hit the max-model limit as we searched. The second boolean says whether
--- there were prefix-existentials. The third boolean says whether we stopped because
--- the solver returned 'Unknown'.
-newtype AllSatResult = AllSatResult (Bool, Bool, Bool, [SMTResult])
+-- | An 'Data.SBV.allSat' call results in a 'AllSatResult'
+data AllSatResult = AllSatResult { allSatMaxModelCountReached  :: Bool          -- ^ Did we reach the user given model count limit?
+                                 , allSatHasPrefixExistentials :: Bool          -- ^ Were there quantifiers in the problem (unique upto prefix existentials)
+                                 , allSatSolverReturnedUnknown :: Bool          -- ^ Did the solver report unknown at the end?
+                                 , allSatSolverReturnedDSat    :: Bool          -- ^ Did the solver report delta-satisfiable at the end?
+                                 , allSatResults               :: [SMTResult]   -- ^ All satisfying models
+                                 }
 
 -- | A 'Data.SBV.safe' call results in a 'SafeResult'
 newtype SafeResult   = SafeResult   (Maybe String, String, SMTResult)
@@ -108,48 +125,73 @@ data OptimizeResult = LexicographicResult SMTResult
                     | ParetoResult        (Bool, [SMTResult])
                     | IndependentResult   [(String, SMTResult)]
 
+-- | What's the precision of a delta-sat query?
+getPrecision :: SMTResult -> Maybe String -> String
+getPrecision r queriedPrecision = case (queriedPrecision, dsatPrecision (resultConfig r)) of
+                                   (Just s, _     ) -> s
+                                   (_,      Just d) -> showFFloat Nothing d ""
+                                   _                -> "tool default"
+
 -- User friendly way of printing theorem results
 instance Show ThmResult where
   show (ThmResult r) = showSMTResult "Q.E.D."
                                      "Unknown"
-                                     "Falsifiable" "Falsifiable. Counter-example:\n" "Falsifiable in an extension field:\n" r
+                                     "Falsifiable"
+                                     "Falsifiable. Counter-example:\n"
+                                     (\mbP -> "Delta falsifiable, precision: " ++ getPrecision r mbP ++ ". Counter-example:\n")
+                                     "Falsifiable in an extension field:\n"
+                                     r
 
 -- User friendly way of printing satisfiablity results
 instance Show SatResult where
   show (SatResult r) = showSMTResult "Unsatisfiable"
                                      "Unknown"
-                                     "Satisfiable" "Satisfiable. Model:\n" "Satisfiable in an extension field. Model:\n" r
+                                     "Satisfiable"
+                                     "Satisfiable. Model:\n"
+                                     (\mbP -> "Delta satisfiable, precision: " ++ getPrecision r mbP ++ ". Model:\n")
+                                     "Satisfiable in an extension field. Model:\n"
+                                     r
 
 -- User friendly way of printing safety results
 instance Show SafeResult where
    show (SafeResult (mbLoc, msg, r)) = showSMTResult (tag "No violations detected")
                                                      (tag "Unknown")
-                                                     (tag "Violated") (tag "Violated. Model:\n") (tag "Violated in an extension field:\n") r
+                                                     (tag "Violated")
+                                                     (tag "Violated. Model:\n")
+                                                     (\mbP -> tag "Violated in a delta-satisfiable context, precision: " ++ getPrecision r mbP ++ ". Model:\n")
+                                                     (tag "Violated in an extension field:\n")
+                                                     r
         where loc   = maybe "" (++ ": ") mbLoc
               tag s = loc ++ msg ++ ": " ++ s
 
--- The Show instance of AllSatResults. Note that we have to be careful in being lazy enough
--- as the typical use case is to pull results out as they become available.
+-- The Show instance of AllSatResults.
 instance Show AllSatResult where
-  show (AllSatResult (l, e, u, xs)) = go (0::Int) xs
+  show AllSatResult { allSatMaxModelCountReached  = l
+                    , allSatHasPrefixExistentials = e
+                    , allSatSolverReturnedUnknown = u
+                    , allSatSolverReturnedDSat    = d
+                    , allSatResults               = xs
+                    } = go (0::Int) xs
     where warnings = case (e, u) of
                        (False, False) -> ""
                        (False, True)  -> " (Search stopped since solver has returned unknown.)"
                        (True,  False) -> " (Unique up to prefix existentials.)"
-                       (True,  True)  -> " (Search stopped becase solver has returned unknown, only prefix existentials were considered.)"
+                       (True,  True)  -> " (Search stopped because solver has returned unknown, only prefix existentials were considered.)"
 
           go c (s:ss) = let c'      = c+1
                             (ok, o) = sh c' s
                         in c' `seq` if ok then o ++ "\n" ++ go c' ss else o
-          go c []     = case (l, c) of
-                          (True,  _) -> "Search stopped since model count request was reached." ++ warnings
-                          (False, 0) -> "No solutions found."
-                          (False, 1) -> "This is the only solution." ++ warnings
-                          (False, _) -> "Found " ++ show c ++ " different solutions." ++ warnings
+          go c []     = case (l, d, c) of
+                          (True,  _   , _) -> "Search stopped since model count request was reached."  ++ warnings
+                          (_   ,  True, _) -> "Search stopped since the result was delta-satisfiable." ++ warnings
+                          (False, _   , 0) -> "No solutions found."
+                          (False, _   , 1) -> "This is the only solution." ++ warnings
+                          (False, _   , _) -> "Found " ++ show c ++ " different solutions." ++ warnings
 
           sh i c = (ok, showSMTResult "Unsatisfiable"
                                       "Unknown"
                                       ("Solution #" ++ show i ++ ":\nSatisfiable") ("Solution #" ++ show i ++ ":\n")
+                                      (\mbP -> "Solution $" ++ show i ++ " with delta-satisfiability, precision: " ++ getPrecision c mbP ++ ":\n")
                                       ("Solution $" ++ show i ++ " in an extension field:\n")
                                       c)
               where ok = case c of
@@ -175,11 +217,13 @@ instance Show OptimizeResult where
              shI n = sh (\s -> "Objective "     ++ show n ++ ": " ++ s)
              shP i = sh (\s -> "Pareto front #" ++ show i ++ ": " ++ s)
 
-             sh tag = showSMTResult (tag "Unsatisfiable.")
-                                    (tag "Unknown.")
-                                    (tag "Optimal with no assignments.")
-                                    (tag "Optimal model:" ++ "\n")
-                                    (tag "Optimal in an extension field:" ++ "\n")
+             sh tag r = showSMTResult (tag "Unsatisfiable.")
+                                      (tag "Unknown.")
+                                      (tag "Optimal with no assignments.")
+                                      (tag "Optimal model:" ++ "\n")
+                                      (\mbP -> tag "Optimal model with delta-satisfiability, precision: " ++ getPrecision r mbP ++ ":" ++ "\n")
+                                      (tag "Optimal in an extension field:" ++ "\n")
+                                      r
 
 -- | Instances of 'SatModel' can be automatically extracted from models returned by the
 -- solvers. The idea is that the sbv infrastructure provides a stream of CV's (constant values)
@@ -266,6 +310,12 @@ instance SatModel Double where
   parseCVs (CV KDouble (CDouble i) : r) = Just (i, r)
   parseCVs _                            = Nothing
 
+-- | A general floating-point extracted from a model
+instance (KnownNat eb, KnownNat sb) => SatModel (FloatingPoint eb sb) where
+  parseCVs (CV (KFP ei si) (CFP fp) : r)
+    | intOfProxy (Proxy @eb) == ei , intOfProxy (Proxy @sb) == si = Just (FloatingPoint fp, r)
+  parseCVs _                                                      = Nothing
+
 -- | @CV@ as extracted from a model; trivial definition
 instance SatModel CV where
   parseCVs (cv : r) = Just (cv, r)
@@ -274,10 +324,20 @@ instance SatModel CV where
 -- | A rounding mode, extracted from a model. (Default definition suffices)
 instance SatModel RoundingMode
 
+-- | 'String' as extracted from a model
+instance {-# OVERLAPS #-} SatModel [Char] where
+  parseCVs (CV _ (CString c):r) = Just (c, r)
+  parseCVs _                    = Nothing
+
+-- | 'Char' as extracted from a model
+instance SatModel Char where
+  parseCVs (CV _ (CChar c):r) = Just (c, r)
+  parseCVs _                  = Nothing
+
 -- | A list of values as extracted from a model. When reading a list, we
 -- go as long as we can (maximal-munch). Note that this never fails, as
 -- we can always return the empty list!
-instance SatModel a => SatModel [a] where
+instance {-# OVERLAPPABLE #-} SatModel a => SatModel [a] where
   parseCVs [] = Just ([], [])
   parseCVs xs = case parseCVs xs of
                   Just (a, ys) -> case parseCVs ys of
@@ -369,19 +429,19 @@ class Modelable a where
 -- | Return all the models from an 'Data.SBV.allSat' call, similar to 'extractModel' but
 -- is suitable for the case of multiple results.
 extractModels :: SatModel a => AllSatResult -> [a]
-extractModels (AllSatResult (_, _, _, xs)) = [ms | Right (_, ms) <- map getModelAssignment xs]
+extractModels AllSatResult{allSatResults = xs} = [ms | Right (_, ms) <- map getModelAssignment xs]
 
 -- | Get dictionaries from an all-sat call. Similar to `getModelDictionary`.
 getModelDictionaries :: AllSatResult -> [M.Map String CV]
-getModelDictionaries (AllSatResult (_, _, _, xs)) = map getModelDictionary xs
+getModelDictionaries AllSatResult{allSatResults = xs} = map getModelDictionary xs
 
 -- | Extract value of a variable from an all-sat call. Similar to `getModelValue`.
 getModelValues :: SymVal b => String -> AllSatResult -> [Maybe b]
-getModelValues s (AllSatResult (_, _, _, xs)) =  map (s `getModelValue`) xs
+getModelValues s AllSatResult{allSatResults = xs} =  map (s `getModelValue`) xs
 
 -- | Extract value of an uninterpreted variable from an all-sat call. Similar to `getModelUninterpretedValue`.
 getModelUninterpretedValues :: String -> AllSatResult -> [Maybe String]
-getModelUninterpretedValues s (AllSatResult (_, _, _, xs)) =  map (s `getModelUninterpretedValue`) xs
+getModelUninterpretedValues s AllSatResult{allSatResults = xs} =  map (s `getModelUninterpretedValue`) xs
 
 -- | 'ThmResult' as a generic model provider
 instance Modelable ThmResult where
@@ -402,7 +462,8 @@ instance Modelable SatResult where
 -- | 'SMTResult' as a generic model provider
 instance Modelable SMTResult where
   getModelAssignment (Unsatisfiable _ _  ) = Left "SBV.getModelAssignment: Unsatisfiable result"
-  getModelAssignment (Satisfiable   _ m  ) = Right (False, parseModelOut m)
+  getModelAssignment (Satisfiable   _   m) = Right (False, parseModelOut m)
+  getModelAssignment (DeltaSat      _ _ m) = Right (False, parseModelOut m)
   getModelAssignment (SatExtField   _ _  ) = Left "SBV.getModelAssignment: The model is in an extension field"
   getModelAssignment (Unknown       _ m  ) = Left $ "SBV.getModelAssignment: Solver state is unknown: " ++ show m
   getModelAssignment (ProofError    _ s _) = error $ unlines $ "SBV.getModelAssignment: Failed to produce a model: " : s
@@ -411,23 +472,26 @@ instance Modelable SMTResult where
   modelExists Unknown{}       = False -- don't risk it
   modelExists _               = False
 
-  getModelDictionary Unsatisfiable{}   = M.empty
-  getModelDictionary (Satisfiable _ m) = M.fromList (modelAssocs m)
-  getModelDictionary SatExtField{}     = M.empty
-  getModelDictionary Unknown{}         = M.empty
-  getModelDictionary ProofError{}      = M.empty
+  getModelDictionary Unsatisfiable{}     = M.empty
+  getModelDictionary (Satisfiable _   m) = M.fromList (modelAssocs m)
+  getModelDictionary (DeltaSat    _ _ m) = M.fromList (modelAssocs m)
+  getModelDictionary SatExtField{}       = M.empty
+  getModelDictionary Unknown{}           = M.empty
+  getModelDictionary ProofError{}        = M.empty
 
-  getModelObjectives Unsatisfiable{}   = M.empty
-  getModelObjectives (Satisfiable _ m) = M.fromList (modelObjectives m)
-  getModelObjectives (SatExtField _ m) = M.fromList (modelObjectives m)
-  getModelObjectives Unknown{}         = M.empty
-  getModelObjectives ProofError{}      = M.empty
+  getModelObjectives Unsatisfiable{}     = M.empty
+  getModelObjectives (Satisfiable _ m  ) = M.fromList (modelObjectives m)
+  getModelObjectives (DeltaSat    _ _ m) = M.fromList (modelObjectives m)
+  getModelObjectives (SatExtField _ m  ) = M.fromList (modelObjectives m)
+  getModelObjectives Unknown{}           = M.empty
+  getModelObjectives ProofError{}        = M.empty
 
-  getModelUIFuns Unsatisfiable{}   = M.empty
-  getModelUIFuns (Satisfiable _ m) = M.fromList (modelUIFuns m)
-  getModelUIFuns (SatExtField _ m) = M.fromList (modelUIFuns m)
-  getModelUIFuns Unknown{}         = M.empty
-  getModelUIFuns ProofError{}      = M.empty
+  getModelUIFuns Unsatisfiable{}     = M.empty
+  getModelUIFuns (Satisfiable _ m  ) = M.fromList (modelUIFuns m)
+  getModelUIFuns (DeltaSat    _ _ m) = M.fromList (modelUIFuns m)
+  getModelUIFuns (SatExtField _ m  ) = M.fromList (modelUIFuns m)
+  getModelUIFuns Unknown{}           = M.empty
+  getModelUIFuns ProofError{}        = M.empty
 
 -- | Extract a model out, will throw error if parsing is unsuccessful
 parseModelOut :: SatModel a => SMTModel -> a
@@ -439,21 +503,22 @@ parseModelOut m = case parseCVs [c | (_, c) <- modelAssocs m] of
 -- | Given an 'Data.SBV.allSat' call, we typically want to iterate over it and print the results in sequence. The
 -- 'displayModels' function automates this task by calling @disp@ on each result, consecutively. The first
 -- 'Int' argument to @disp@ 'is the current model number. The second argument is a tuple, where the first
--- element indicates whether the model is alleged (i.e., if the solver is not sure, returing Unknown).
+-- element indicates whether the model is alleged (i.e., if the solver is not sure, returning Unknown).
 -- The arrange argument can sort the results in any way you like, if necessary.
 displayModels :: SatModel a => ([(Bool, a)] -> [(Bool, a)]) -> (Int -> (Bool, a) -> IO ()) -> AllSatResult -> IO Int
-displayModels arrange disp (AllSatResult (_, _, _, ms)) = do
+displayModels arrange disp AllSatResult{allSatResults = ms} = do
     let models = [a | Right a <- map (getModelAssignment . SatResult) ms]
     inds <- zipWithM display (arrange models) [(1::Int)..]
     return $ last (0:inds)
   where display r i = disp i r >> return i
 
 -- | Show an SMTResult; generic version
-showSMTResult :: String -> String -> String -> String -> String -> SMTResult -> String
-showSMTResult unsatMsg unkMsg satMsg satMsgModel satExtMsg result = case result of
+showSMTResult :: String -> String -> String -> String -> (Maybe String -> String) -> String -> SMTResult -> String
+showSMTResult unsatMsg unkMsg satMsg satMsgModel dSatMsgModel satExtMsg result = case result of
   Unsatisfiable _ uc                 -> unsatMsg ++ showUnsatCore uc
   Satisfiable _ (SMTModel _ _ [] []) -> satMsg
-  Satisfiable _ m                    -> satMsgModel ++ showModel cfg m
+  Satisfiable _   m                  -> satMsgModel    ++ showModel cfg m
+  DeltaSat    _ p m                  -> dSatMsgModel p ++ showModel cfg m
   SatExtField _ (SMTModel b _ _ _)   -> satExtMsg   ++ showModelDictionary True False cfg b
   Unknown     _ r                    -> unkMsg ++ ".\n" ++ "  Reason: " `alignPlain` show r
   ProofError  _ [] Nothing           -> "*** An error occurred. No additional information available. Try running in verbose mode."
@@ -463,7 +528,7 @@ showSMTResult unsatMsg unkMsg satMsg satMsgModel satExtMsg result = case result
                                                             , "*** Alleged model:"
                                                             , "***"
                                                             ]
-                                                         ++ ["*** "  ++ l | l <- lines (showSMTResult unsatMsg unkMsg satMsg satMsgModel satExtMsg r)]
+                                                         ++ ["*** "  ++ l | l <- lines (showSMTResult unsatMsg unkMsg satMsg satMsgModel dSatMsgModel satExtMsg r)]
 
  where cfg = resultConfig result
        showUnsatCore Nothing   = ""
@@ -494,9 +559,9 @@ showModelDictionary warnEmpty includeEverything cfg allVars
   where warn s = if warnEmpty then s else ""
 
         relevantVars  = filter (not . ignore) allVars
-        ignore (s, _)
+        ignore (T.pack -> s, _)
           | includeEverything = False
-          | True              = "__internal_sbv_" `isPrefixOf` s || isNonModelVar cfg s
+          | True              = "__internal_sbv_" `T.isPrefixOf` s || isNonModelVar cfg (T.unpack s)
 
         shM (s, RegularCV v) = let vs = shCV cfg v in ((length s, s), (vlength vs, vs))
         shM (s, other)       = let vs = show other in ((length s, s), (vlength vs, vs))
@@ -537,6 +602,8 @@ showModelUI cfg (nm, (SBVType ts, (defs, dflt))) = intercalate "\n" ["  " ++ l |
         align (xs, r) = unwords $ zipWith left colWidths xs ++ ["=", left resWidth r]
            where left i x = take i (x ++ repeat ' ')
 
+        -- NB. We'll ignore crackNum here. Seems to be overkill while displaying an
+        -- uninterpreted function.
         scv = sh (printBase cfg)
           where sh 2  = binP
                 sh 10 = showCV False
@@ -559,12 +626,18 @@ showModelUI cfg (nm, (SBVType ts, (defs, dflt))) = intercalate "\n" ["  " ++ l |
 
 -- | Show a constant value, in the user-specified base
 shCV :: SMTConfig -> CV -> String
-shCV = sh . printBase
+shCV SMTConfig{printBase, crackNum} cv = cracked (sh printBase cv)
   where sh 2  = binS
         sh 10 = show
         sh 16 = hexS
         sh n  = \w -> show w ++ " -- Ignoring unsupported printBase " ++ show n ++ ", use 2, 10, or 16."
 
+        cracked def
+          | not crackNum = def
+          | True         = case CN.crackNum cv of
+                             Nothing -> def
+                             Just cs -> def ++ "\n" ++ cs
+
 -- | Helper function to spin off to an SMT solver.
 pipeProcess :: SMTConfig -> State -> String -> [String] -> String -> (State -> IO a) -> IO a
 pipeProcess cfg ctx execName opts pgm continuation = do
@@ -599,7 +672,7 @@ standardEngine envName envOptName cfg ctx pgm continuation = do
 
 -- | A standard solver interface. If the solver is SMT-Lib compliant, then this function should suffice in
 -- communicating with it.
-standardSolver :: SMTConfig       -- ^ The currrent configuration
+standardSolver :: SMTConfig       -- ^ The current configuration
                -> State           -- ^ Context in which we are running
                -> String          -- ^ The program
                -> (State -> IO a) -- ^ The continuation
@@ -608,7 +681,7 @@ standardSolver config ctx pgm continuation = do
     let msg s    = debug config ["** " ++ s]
         smtSolver= solver config
         exec     = executable smtSolver
-        opts     = options smtSolver config
+        opts     = options smtSolver config ++ extraArgs config
     msg $ "Calling: "  ++ (exec ++ (if null opts then "" else " ") ++ joinArgs opts)
     rnf pgm `seq` pipeProcess config ctx exec opts pgm continuation
 
@@ -785,7 +858,7 @@ runSolver cfg ctx execPath opts pgm continuation
       let executeSolver = do let sendAndGetSuccess :: Maybe Int -> String -> IO ()
                                  sendAndGetSuccess mbTimeOut l
                                    -- The pathetic case when the solver doesn't support queries, so we pretend it responded "success"
-                                   -- Currently ABC is the only such solver. Filed a request for ABC at: http://bitbucket.org/alanmi/abc/issues/70/
+                                   -- Currently ABC is the only such solver.
                                    | not (supportsCustomQueries (capabilities (solver cfg)))
                                    = do send mbTimeOut l
                                         debug cfg ["[ISSUE] " `alignPlain` l]
@@ -927,7 +1000,7 @@ startTranscript (Just f) cfg = do ts <- show <$> getZonedTime
                                   , ";;;"
                                   , ";;;           Solver    : " ++ show name
                                   , ";;;           Executable: " ++ fromMaybe "Unable to locate the executable" mbPath
-                                  , ";;;           Options   : " ++ unwords (options cfg)
+                                  , ";;;           Options   : " ++ unwords (options cfg ++ extraArgs cfg)
                                   , ";;;"
                                   , ";;; This file is an auto-generated loadable SMT-Lib file."
                                   , ";;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;"
