@@ -58,6 +58,8 @@ import Data.SBV.Core.Data
 import Data.SBV.Core.Model    ((.==), (./=))
 import Data.SBV.Core.Symbolic (SetOp(..))
 
+import qualified Data.Generics.Uniplate.Data as G
+
 -- For doctest use only
 --
 -- $setup
@@ -116,6 +118,16 @@ fromList = literal . RegularSet . Set.fromList
 -- Q.E.D.
 complement :: forall a. (Ord a, SymVal a) => SSet a -> SSet a
 complement ss
+  | KChar `elem` G.universe k
+  = error $ unlines [ "*** Data.SBV: Set.complement is not available for the type " ++ show k
+                    , "***"
+                    , "*** See: https://github.com/LeventErkok/sbv/issues/601 for a discussion"
+                    , "*** on why SBV does not support this operation at this type."
+                    , "***"
+                    , "*** Alternative: Use sets of strings instead, though the match isn't perfect."
+                    , "*** If you run into this issue, please comment on the above ticket for"
+                    , "*** possible improvements."
+                    ]
   | Just (RegularSet rs) <- unliteral ss
   = literal $ ComplementSet rs
   | Just (ComplementSet cs) <- unliteral ss
@@ -138,8 +150,8 @@ complement ss
 --
 -- >>> prove $ \x (s :: SSet Integer) -> x `delete` (x `insert` s) .== s
 -- Falsifiable. Counter-example:
---   s0 =   0 :: Integer
---   s1 = {0} :: {Integer}
+--   s0 =   2 :: Integer
+--   s1 = {2} :: {Integer}
 --
 -- But the above is true if the element isn't in the set to start with:
 --
@@ -181,7 +193,7 @@ insert se ss
 --
 -- >>> prove $ \x (s :: SSet Integer) -> x `insert` (x `delete` s) .== s
 -- Falsifiable. Counter-example:
---   s0 =  0 :: Integer
+--   s0 =  2 :: Integer
 --   s1 = {} :: {Integer}
 --
 -- But the above is true if the element is in the set to start with:
@@ -283,8 +295,8 @@ isEmpty = null
 --
 -- >>> prove $ \x -> isFull (observe "set" (x `delete` (full :: SSet Integer)))
 -- Falsifiable. Counter-example:
---   set = U - {0} :: {Integer}
---   s0  =       0 :: Integer
+--   set = U - {2} :: {Integer}
+--   s0  =       2 :: Integer
 --
 -- >>> isFull (full :: SSet Integer)
 -- True
@@ -371,7 +383,7 @@ isSubsetOf sa sb
 --
 -- >>> prove $ \x (s :: SSet Integer) -> s `isProperSubsetOf` (x `insert` s)
 -- Falsifiable. Counter-example:
---   s0 = 0 :: Integer
+--   s0 = 2 :: Integer
 --   s1 = U :: {Integer}
 --
 -- >>> prove $ \x (s :: SSet Integer) -> x `notMember` s .=> s `isProperSubsetOf` (x `insert` s)
@@ -379,8 +391,8 @@ isSubsetOf sa sb
 --
 -- >>> prove $ \x (s :: SSet Integer) -> (x `delete` s) `isProperSubsetOf` s
 -- Falsifiable. Counter-example:
---   s0 =         0 :: Integer
---   s1 = U - {0,1} :: {Integer}
+--   s0 =         2 :: Integer
+--   s1 = U - {2,3} :: {Integer}
 --
 -- >>> prove $ \x (s :: SSet Integer) -> x `member` s .=> (x `delete` s) `isProperSubsetOf` s
 -- Q.E.D.
@@ -534,7 +546,7 @@ True
 False
 >>> sat $ \(x::SSet (Maybe Integer)) y z -> distinct [x, y, z]
 Satisfiable. Model:
-  s0 = U - {Just 0} :: {Maybe Integer}
+  s0 = U - {Just 2} :: {Maybe Integer}
   s1 =           {} :: {Maybe Integer}
   s2 =            U :: {Maybe Integer}
 
