@@ -48,7 +48,7 @@ genTest n m = gen 0 []
          | i == n = return $ TV $ reverse sofar
          | True   = do t <- tc
                        gen (i+1) (t:sofar)
-        tc = do (_, Result {resTraces=tvals, resConsts=cs, resConstraints=cstrs, resOutputs=os}) <- runSymbolic (Concrete Nothing) (m >>= output)
+        tc = do (_, Result {resTraces=tvals, resConsts=(_, cs), resConstraints=cstrs, resOutputs=os}) <- runSymbolic (Concrete Nothing) (m >>= output)
                 let cval = fromMaybe (error "Cannot generate tests in the presence of uninterpeted constants!") . (`lookup` cs)
                     cond = and [cvToBool (cval v) | (False, _, v) <- F.toList cstrs] -- Only pick-up "hard" constraints, as indicated by False in the fist component
                 if cond
@@ -118,41 +118,43 @@ haskell vname vs = intercalate "\n" $ [ "-- Automatically generated by SBV. Do n
         valOf  xs    = "[" ++ intercalate ", " (map s xs) ++ "]"
 
         t cv = case kindOf cv of
-                 KBool               -> "Bool"
-                 KBounded False 8    -> "Word8"
-                 KBounded False 16   -> "Word16"
-                 KBounded False 32   -> "Word32"
-                 KBounded False 64   -> "Word64"
-                 KBounded True  8    -> "Int8"
-                 KBounded True  16   -> "Int16"
-                 KBounded True  32   -> "Int32"
-                 KBounded True  64   -> "Int64"
-                 KUnbounded          -> "Integer"
-                 KFloat              -> "Float"
-                 KDouble             -> "Double"
-                 KChar               -> error "SBV.renderTest: Unsupported char"
-                 KString             -> error "SBV.renderTest: Unsupported string"
-                 KReal               -> error $ "SBV.renderTest: Unsupported real valued test value: " ++ show cv
-                 KList es            -> error $ "SBV.renderTest: Unsupported list valued test: [" ++ show es ++ "]"
-                 KSet  es            -> error $ "SBV.renderTest: Unsupported set valued test: {" ++ show es ++ "}"
-                 KUninterpreted us _ -> error $ "SBV.renderTest: Unsupported uninterpreted sort: " ++ us
-                 _                   -> error $ "SBV.renderTest: Unexpected CV: " ++ show cv
+                 KBool             -> "Bool"
+                 KBounded False 8  -> "Word8"
+                 KBounded False 16 -> "Word16"
+                 KBounded False 32 -> "Word32"
+                 KBounded False 64 -> "Word64"
+                 KBounded True  8  -> "Int8"
+                 KBounded True  16 -> "Int16"
+                 KBounded True  32 -> "Int32"
+                 KBounded True  64 -> "Int64"
+                 KUnbounded        -> "Integer"
+                 KFloat            -> "Float"
+                 KDouble           -> "Double"
+                 KChar             -> error "SBV.renderTest: Unsupported char"
+                 KString           -> error "SBV.renderTest: Unsupported string"
+                 KReal             -> error $ "SBV.renderTest: Unsupported real valued test value: " ++ show cv
+                 KList es          -> error $ "SBV.renderTest: Unsupported list valued test: [" ++ show es ++ "]"
+                 KSet  es          -> error $ "SBV.renderTest: Unsupported set valued test: {" ++ show es ++ "}"
+                 KUserSort us _    -> error $ "SBV.renderTest: Unsupported uninterpreted sort: " ++ us
+                 _                 -> error $ "SBV.renderTest: Unexpected CV: " ++ show cv
 
         s cv = case kindOf cv of
-                  KBool               -> take 5 (show (cvToBool cv) ++ repeat ' ')
-                  KBounded sgn   sz   -> let CInteger w = cvVal cv in shex  False True (sgn, sz) w
-                  KUnbounded          -> let CInteger w = cvVal cv in shexI False True           w
-                  KFloat              -> let CFloat   w = cvVal cv in showHFloat w
-                  KDouble             -> let CDouble  w = cvVal cv in showHDouble w
-                  KChar               -> error "SBV.renderTest: Unsupported char"
-                  KString             -> error "SBV.renderTest: Unsupported string"
-                  KReal               -> let CAlgReal w = cvVal cv in algRealToHaskell w
-                  KList es            -> error $ "SBV.renderTest: Unsupported list valued sort: [" ++ show es ++ "]"
-                  KSet  es            -> error $ "SBV.renderTest: Unsupported set valued sort: {" ++ show es ++ "}"
-                  KUninterpreted us _ -> error $ "SBV.renderTest: Unsupported uninterpreted sort: " ++ us
-                  k@KTuple{}          -> error $ "SBV.renderTest: Unsupported tuple: " ++ show k
-                  k@KMaybe{}          -> error $ "SBV.renderTest: Unsupported maybe: " ++ show k
-                  k@KEither{}         -> error $ "SBV.renderTest: Unsupported sum: " ++ show k
+                  KBool             -> take 5 (show (cvToBool cv) ++ repeat ' ')
+                  KBounded sgn   sz -> let CInteger w = cvVal cv in shex  False True (sgn, sz) w
+                  KUnbounded        -> let CInteger w = cvVal cv in shexI False True           w
+                  KFloat            -> let CFloat   w = cvVal cv in showHFloat w
+                  KDouble           -> let CDouble  w = cvVal cv in showHDouble w
+                  KRational         -> error "SBV.renderTest: Unsupported rational number"
+                  KFP{}             -> error "SBV.renderTest: Unsupported arbitrary float"
+                  KChar             -> error "SBV.renderTest: Unsupported char"
+                  KString           -> error "SBV.renderTest: Unsupported string"
+                  KReal             -> let CAlgReal w = cvVal cv in algRealToHaskell w
+                  KList es          -> error $ "SBV.renderTest: Unsupported list valued sort: [" ++ show es ++ "]"
+                  KSet  es          -> error $ "SBV.renderTest: Unsupported set valued sort: {" ++ show es ++ "}"
+                  KUserSort us _    -> error $ "SBV.renderTest: Unsupported uninterpreted sort: " ++ us
+                  k@KTuple{}        -> error $ "SBV.renderTest: Unsupported tuple: " ++ show k
+                  k@KMaybe{}        -> error $ "SBV.renderTest: Unsupported maybe: " ++ show k
+                  k@KEither{}       -> error $ "SBV.renderTest: Unsupported sum: " ++ show k
 
 c :: String -> [([CV], [CV])] -> String
 c n vs = intercalate "\n" $
@@ -218,47 +220,51 @@ c n vs = intercalate "\n" $
               ]
   where mkField p cv i = "    " ++ t ++ " " ++ p ++ show i ++ ";"
             where t = case kindOf cv of
-                        KBool               -> "SBool"
-                        KBounded False 8    -> "SWord8"
-                        KBounded False 16   -> "SWord16"
-                        KBounded False 32   -> "SWord32"
-                        KBounded False 64   -> "SWord64"
-                        KBounded True  8    -> "SInt8"
-                        KBounded True  16   -> "SInt16"
-                        KBounded True  32   -> "SInt32"
-                        KBounded True  64   -> "SInt64"
-                        k@KBounded{}        -> error $ "SBV.renderTest: Unsupported kind: " ++ show k
-                        KFloat              -> "SFloat"
-                        KDouble             -> "SDouble"
-                        KChar               -> error "SBV.renderTest: Unsupported char"
-                        KString             -> error "SBV.renderTest: Unsupported string"
-                        KUnbounded          -> error "SBV.renderTest: Unbounded integers are not supported when generating C test-cases."
-                        KReal               -> error "SBV.renderTest: Real values are not supported when generating C test-cases."
-                        KUninterpreted us _ -> error $ "SBV.renderTest: Unsupported uninterpreted sort: " ++ us
-                        k@KList{}           -> error $ "SBV.renderTest: Unsupported list sort: "   ++ show k
-                        k@KSet{}            -> error $ "SBV.renderTest: Unsupported set sort: "   ++ show k
-                        k@KTuple{}          -> error $ "SBV.renderTest: Unsupported tuple sort: "  ++ show k
-                        k@KMaybe{}          -> error $ "SBV.renderTest: Unsupported maybe sort: "  ++ show k
-                        k@KEither{}         -> error $ "SBV.renderTest: Unsupported either sort: " ++ show k
+                        KBool             -> "SBool"
+                        KBounded False 8  -> "SWord8"
+                        KBounded False 16 -> "SWord16"
+                        KBounded False 32 -> "SWord32"
+                        KBounded False 64 -> "SWord64"
+                        KBounded True  8  -> "SInt8"
+                        KBounded True  16 -> "SInt16"
+                        KBounded True  32 -> "SInt32"
+                        KBounded True  64 -> "SInt64"
+                        k@KBounded{}      -> error $ "SBV.renderTest: Unsupported kind: " ++ show k
+                        KFloat            -> "SFloat"
+                        KDouble           -> "SDouble"
+                        KRational         -> error "SBV.renderTest: Unsupported rational number"
+                        KFP{}             -> error "SBV.renderTest: Unsupported arbitrary float"
+                        KChar             -> error "SBV.renderTest: Unsupported char"
+                        KString           -> error "SBV.renderTest: Unsupported string"
+                        KUnbounded        -> error "SBV.renderTest: Unbounded integers are not supported when generating C test-cases."
+                        KReal             -> error "SBV.renderTest: Real values are not supported when generating C test-cases."
+                        KUserSort us _    -> error $ "SBV.renderTest: Unsupported uninterpreted sort: " ++ us
+                        k@KList{}         -> error $ "SBV.renderTest: Unsupported list sort: "   ++ show k
+                        k@KSet{}          -> error $ "SBV.renderTest: Unsupported set sort: "   ++ show k
+                        k@KTuple{}        -> error $ "SBV.renderTest: Unsupported tuple sort: "  ++ show k
+                        k@KMaybe{}        -> error $ "SBV.renderTest: Unsupported maybe sort: "  ++ show k
+                        k@KEither{}       -> error $ "SBV.renderTest: Unsupported either sort: " ++ show k
 
 
         mkLine (is, os) = "{{" ++ intercalate ", " (map v is) ++ "}, {" ++ intercalate ", " (map v os) ++ "}}"
 
         v cv = case kindOf cv of
-                  KBool               -> if cvToBool cv then "true " else "false"
-                  KBounded sgn sz     -> let CInteger w = cvVal cv in chex  False True (sgn, sz) w
-                  KUnbounded          -> let CInteger w = cvVal cv in shexI False True           w
-                  KFloat              -> let CFloat w   = cvVal cv in showCFloat w
-                  KDouble             -> let CDouble w  = cvVal cv in showCDouble w
-                  KChar               -> error "SBV.renderTest: Unsupported char"
-                  KString             -> error "SBV.renderTest: Unsupported string"
-                  k@KList{}           -> error $ "SBV.renderTest: Unsupported list sort!" ++ show k
-                  k@KSet{}            -> error $ "SBV.renderTest: Unsupported set sort!" ++ show k
-                  KUninterpreted us _ -> error $ "SBV.renderTest: Unsupported uninterpreted sort: " ++ us
-                  KReal               -> error "SBV.renderTest: Real values are not supported when generating C test-cases."
-                  k@KTuple{}          -> error $ "SBV.renderTest: Unsupported tuple sort!" ++ show k
-                  k@KMaybe{}          -> error $ "SBV.renderTest: Unsupported maybe sort!" ++ show k
-                  k@KEither{}         -> error $ "SBV.renderTest: Unsupported sum sort!" ++ show k
+                  KBool            -> if cvToBool cv then "true " else "false"
+                  KBounded sgn sz  -> let CInteger w = cvVal cv in chex  False True (sgn, sz) w
+                  KUnbounded       -> let CInteger w = cvVal cv in shexI False True           w
+                  KFloat           -> let CFloat w   = cvVal cv in showCFloat w
+                  KDouble          -> let CDouble w  = cvVal cv in showCDouble w
+                  KRational        -> error "SBV.renderTest: Unsupported rational number"
+                  KFP{}            -> error "SBV.renderTest: Unsupported arbitrary float"
+                  KChar            -> error "SBV.renderTest: Unsupported char"
+                  KString          -> error "SBV.renderTest: Unsupported string"
+                  k@KList{}        -> error $ "SBV.renderTest: Unsupported list sort!" ++ show k
+                  k@KSet{}         -> error $ "SBV.renderTest: Unsupported set sort!" ++ show k
+                  KUserSort us _   -> error $ "SBV.renderTest: Unsupported uninterpreted sort: " ++ us
+                  KReal            -> error "SBV.renderTest: Real values are not supported when generating C test-cases."
+                  k@KTuple{}       -> error $ "SBV.renderTest: Unsupported tuple sort!" ++ show k
+                  k@KMaybe{}       -> error $ "SBV.renderTest: Unsupported maybe sort!" ++ show k
+                  k@KEither{}      -> error $ "SBV.renderTest: Unsupported sum sort!" ++ show k
 
         outLine
           | null vs = "printf(\"\");"
@@ -309,32 +315,34 @@ forte vname bigEndian ss vs = intercalate "\n" $ [ "// Automatically generated b
         toF False = '0'
         blast cv = let noForte w = error "SBV.renderTest: " ++ w ++ " values are not supported when generating Forte test-cases."
                    in case kindOf cv of
-                        KBool              -> [toF (cvToBool cv)]
-                        KBounded False 8   -> xlt  8 (cvVal cv)
-                        KBounded False 16  -> xlt 16 (cvVal cv)
-                        KBounded False 32  -> xlt 32 (cvVal cv)
-                        KBounded False 64  -> xlt 64 (cvVal cv)
-                        KBounded True 8    -> xlt  8 (cvVal cv)
-                        KBounded True 16   -> xlt 16 (cvVal cv)
-                        KBounded True 32   -> xlt 32 (cvVal cv)
-                        KBounded True 64   -> xlt 64 (cvVal cv)
-                        KFloat             -> noForte "Float"
-                        KDouble            -> noForte "Double"
-                        KChar              -> noForte "Char"
-                        KString            -> noForte "String"
-                        KReal              -> noForte "Real"
-                        KList ek           -> noForte $ "List of " ++ show ek
-                        KSet  ek           -> noForte $ "Set of " ++ show ek
-                        KUnbounded         -> noForte "Unbounded integers"
-                        KUninterpreted s _ -> noForte $ "Uninterpreted kind " ++ show s
-                        _                  -> error $ "SBV.renderTest: Unexpected CV: " ++ show cv
+                        KBool             -> [toF (cvToBool cv)]
+                        KBounded False 8  -> xlt  8 (cvVal cv)
+                        KBounded False 16 -> xlt 16 (cvVal cv)
+                        KBounded False 32 -> xlt 32 (cvVal cv)
+                        KBounded False 64 -> xlt 64 (cvVal cv)
+                        KBounded True 8   -> xlt  8 (cvVal cv)
+                        KBounded True 16  -> xlt 16 (cvVal cv)
+                        KBounded True 32  -> xlt 32 (cvVal cv)
+                        KBounded True 64  -> xlt 64 (cvVal cv)
+                        KFloat            -> noForte "Float"
+                        KDouble           -> noForte "Double"
+                        KChar             -> noForte "Char"
+                        KString           -> noForte "String"
+                        KReal             -> noForte "Real"
+                        KList ek          -> noForte $ "List of " ++ show ek
+                        KSet  ek          -> noForte $ "Set of " ++ show ek
+                        KUnbounded        -> noForte "Unbounded integers"
+                        KUserSort s _     -> noForte $ "Uninterpreted kind " ++ show s
+                        _                 -> error $ "SBV.renderTest: Unexpected CV: " ++ show cv
 
         xlt s (CInteger  v)  = [toF (testBit v i) | i <- [s-1, s-2 .. 0]]
-        xlt _ (CFloat    r)  = error $ "SBV.renderTest.Forte: Unexpected float value: "         ++ show r
-        xlt _ (CDouble   r)  = error $ "SBV.renderTest.Forte: Unexpected double value: "        ++ show r
-        xlt _ (CChar     r)  = error $ "SBV.renderTest.Forte: Unexpected char value: "          ++ show r
-        xlt _ (CString   r)  = error $ "SBV.renderTest.Forte: Unexpected string value: "        ++ show r
-        xlt _ (CAlgReal  r)  = error $ "SBV.renderTest.Forte: Unexpected real value: "          ++ show r
+        xlt _ (CFloat    r)  = error $ "SBV.renderTest.Forte: Unexpected float value: "            ++ show r
+        xlt _ (CDouble   r)  = error $ "SBV.renderTest.Forte: Unexpected double value: "           ++ show r
+        xlt _ (CFP       r)  = error $ "SBV.renderTest.Forte: Unexpected arbitrary float value: "  ++ show r
+        xlt _ (CRational r)  = error $ "SBV.renderTest.Forte: Unexpected rational  value: "        ++ show r
+        xlt _ (CChar     r)  = error $ "SBV.renderTest.Forte: Unexpected char value: "             ++ show r
+        xlt _ (CString   r)  = error $ "SBV.renderTest.Forte: Unexpected string value: "           ++ show r
+        xlt _ (CAlgReal  r)  = error $ "SBV.renderTest.Forte: Unexpected real value: "             ++ show r
         xlt _ CList{}        = error   "SBV.renderTest.Forte: Unexpected list value!"
         xlt _ CSet{}         = error   "SBV.renderTest.Forte: Unexpected set value!"
         xlt _ CTuple{}       = error   "SBV.renderTest.Forte: Unexpected list value!"
