@@ -173,7 +173,7 @@ instance (Show st, Show m) => Show (VC st m) where
                                                          ]
   show (InvariantPre      nm s)                 = dispVC ("Invariant for loop " ++ show nm ++ " fails upon entry")
                                                          [("", show s)]
-  show (InvariantMaintain nm s1 s2)             = dispVC ("Invariant for loop " ++ show nm ++ " is not maintaned by the body")
+  show (InvariantMaintain nm s1 s2)             = dispVC ("Invariant for loop " ++ show nm ++ " is not maintained by the body")
                                                          [ ("Before", show s1)
                                                          , ("After ", show s2)
                                                          ]
@@ -220,52 +220,54 @@ wpProveWith cfg@WPConfig{wpVerbose} Program{setup, precondition, program, postco
 
                cs <- checkSat
                case cs of
-                 Unk   -> Indeterminate . show <$> getUnknownReason
+                 Unk    -> Indeterminate . show <$> getUnknownReason
 
-                 Unsat -> do let t = isTotal program
+                 Unsat  -> do let t = isTotal program
 
-                             if t then msg "Total correctness is established."
-                                  else msg "Partial correctness is established."
+                              if t then msg "Total correctness is established."
+                                   else msg "Partial correctness is established."
 
-                             return $ Proven t
+                              pure $ Proven t
 
-                 Sat   -> do let checkVC :: (SBool, VC st SInteger) -> Query [VC res Integer]
-                                 checkVC (cond, vc) = do c <- getValue cond
-                                                         if c
-                                                            then return []   -- The VC was OK
-                                                            else do vc' <- case vc of
-                                                                             BadPrecondition     s                 -> BadPrecondition     <$> project s
-                                                                             BadPostcondition    s1 s2             -> BadPostcondition    <$> project s1 <*> project s2
-                                                                             Unstable          l s1 s2             -> Unstable          l <$> project s1 <*> project s2
-                                                                             AbortReachable    l s1 s2             -> AbortReachable    l <$> project s1 <*> project s2
-                                                                             InvariantPre      l s                 -> InvariantPre      l <$> project s
-                                                                             InvariantMaintain l s1 s2             -> InvariantMaintain l <$> project s1 <*> project s2
-                                                                             MeasureBound      l (s, m)            -> do r <- project s
-                                                                                                                         v <- mapM getValue m
-                                                                                                                         return $ MeasureBound l (r, v)
-                                                                             MeasureDecrease   l (s1, i1) (s2, i2) -> do r1 <- project s1
-                                                                                                                         v1 <- mapM getValue i1
-                                                                                                                         r2 <- project s2
-                                                                                                                         v2 <- mapM getValue i2
-                                                                                                                         return $ MeasureDecrease l (r1, v1) (r2, v2)
-                                                                    return [vc']
+                 DSat{} -> pure $ Indeterminate "Unsupported: Solver returned a delta-satisfiable answer."
 
-                             badVCs <- concat <$> mapM checkVC vcs
+                 Sat    -> do let checkVC :: (SBool, VC st SInteger) -> Query [VC res Integer]
+                                  checkVC (cond, vc) = do c <- getValue cond
+                                                          if c
+                                                             then return []   -- The VC was OK
+                                                             else do vc' <- case vc of
+                                                                              BadPrecondition     s                 -> BadPrecondition     <$> project s
+                                                                              BadPostcondition    s1 s2             -> BadPostcondition    <$> project s1 <*> project s2
+                                                                              Unstable          l s1 s2             -> Unstable          l <$> project s1 <*> project s2
+                                                                              AbortReachable    l s1 s2             -> AbortReachable    l <$> project s1 <*> project s2
+                                                                              InvariantPre      l s                 -> InvariantPre      l <$> project s
+                                                                              InvariantMaintain l s1 s2             -> InvariantMaintain l <$> project s1 <*> project s2
+                                                                              MeasureBound      l (s, m)            -> do r <- project s
+                                                                                                                          v <- mapM getValue m
+                                                                                                                          return $ MeasureBound l (r, v)
+                                                                              MeasureDecrease   l (s1, i1) (s2, i2) -> do r1 <- project s1
+                                                                                                                          v1 <- mapM getValue i1
+                                                                                                                          r2 <- project s2
+                                                                                                                          v2 <- mapM getValue i2
+                                                                                                                          return $ MeasureDecrease l (r1, v1) (r2, v2)
+                                                                     return [vc']
 
-                             when (null badVCs) $ error "Data.SBV.proveWP: Impossible happened. Proof failed, but no failing VC found!"
+                              badVCs <- concat <$> mapM checkVC vcs
 
-                             let plu w (_:_:_) = w ++ "s"
-                                 plu w _       = w
+                              when (null badVCs) $ error "Data.SBV.proveWP: Impossible happened. Proof failed, but no failing VC found!"
 
-                                 m = "Following proof " ++ plu "obligation" badVCs ++ " failed:"
+                              let plu w (_:_:_) = w ++ "s"
+                                  plu w _       = w
 
-                             msg m
-                             msg $ replicate (length m) '='
+                                  m = "Following proof " ++ plu "obligation" badVCs ++ " failed:"
 
-                             let disp c = mapM_ msg ["  " ++ l | l <- lines (show c)]
-                             mapM_ disp badVCs
+                              msg m
+                              msg $ replicate (length m) '='
 
-                             return $ Failed badVCs
+                              let disp c = mapM_ msg ["  " ++ l | l <- lines (show c)]
+                              mapM_ disp badVCs
+
+                              return $ Failed badVCs
 
         msg = io . when wpVerbose . putStrLn
 
@@ -484,4 +486,4 @@ traceExecution Program{precondition, program, postcondition, stability} start =
                            where mCur = currentMeasure is
                                  zero = map (const 0) mCur
 
-{-# ANN traceExecution ("HLint: ignore Replace case with fromMaybe" :: String) #-}
+{-# ANN traceExecution ("HLint: ignore Use fromMaybe" :: String) #-}
