@@ -27,11 +27,11 @@ module Data.SBV.Trans (
   -- *** Signed bit-vectors
   , SInt8, SInt16, SInt32, SInt64, SInt, IntN
   -- *** Converting between fixed-size and arbitrary bitvectors
-  , IsNonZero, FromSized, ToSized, fromSized, toSized
+  , BVIsNonZero, FromSized, ToSized, fromSized, toSized
   -- ** Unbounded integers
   , SInteger
   -- ** Floating point numbers
-  , SFloat, SDouble
+  , SFloat, SDouble, SFloatingPoint
   -- ** Algebraic reals
   , SReal, AlgReal, sRealToSInteger
   -- ** Characters, Strings and Regular Expressions
@@ -39,7 +39,7 @@ module Data.SBV.Trans (
   -- ** Symbolic lists
   , SList
   -- * Arrays of symbolic values
-  , SymArray(newArray_, newArray, readArray, writeArray, mergeArrays), SArray, SFunArray
+  , SymArray(newArray_, newArray, readArray, writeArray, mergeArrays), SArray
 
   -- * Creating symbolic values
   -- ** Single value
@@ -74,14 +74,22 @@ module Data.SBV.Trans (
   , sRoundNearestTiesToEven, sRoundNearestTiesToAway, sRoundTowardPositive, sRoundTowardNegative, sRoundTowardZero, sRNE, sRNA, sRTP, sRTN, sRTZ
   -- ** Conversion to/from floats
   , IEEEFloatConvertible(..)
+
   -- ** Bit-pattern conversions
-  , sFloatAsSWord32, sWord32AsSFloat, sDoubleAsSWord64, sWord64AsSDouble, blastSFloat, blastSDouble
+  , sFloatAsSWord32,       sWord32AsSFloat
+  , sDoubleAsSWord64,      sWord64AsSDouble
+  , sFloatingPointAsSWord, sWordAsSFloatingPoint
+
+  -- ** Extracting bit patterns from floats
+  , blastSFloat
+  , blastSDouble
+  , blastSFloatingPoint
 
   -- * Enumerations
   , mkSymbolicEnumeration
 
   -- * Uninterpreted sorts, axioms, constants, and functions
-  , Uninterpreted(..), addAxiom
+  , mkUninterpretedSort, Uninterpreted(..), addAxiom
 
   -- * Properties, proofs, and satisfiability
   , Predicate, Goal, MProvable(..), Provable, proveWithAll, proveWithAny , satWithAll
@@ -139,7 +147,7 @@ module Data.SBV.Trans (
   -- ** Solvers
   , boolector, cvc4, yices, z3, mathSAT, abc
   -- ** Configurations
-  , defaultSolverConfig, defaultSMTCfg, sbvCheckSolverInstallation, sbvAvailableSolvers
+  , defaultSolverConfig, defaultSMTCfg, sbvCheckSolverInstallation, getAvailableSolvers
   , setLogic, Logic(..), setOption, setInfo, setTimeOut
   -- ** SBV exceptions
   , SBVException(..)
@@ -158,6 +166,7 @@ module Data.SBV.Trans (
 
 import Data.SBV.Core.AlgReals
 import Data.SBV.Core.Data
+import Data.SBV.Core.Kind
 import Data.SBV.Core.Model
 import Data.SBV.Core.Floating
 import Data.SBV.Core.Sized
