@@ -16,11 +16,13 @@
 
 module Data.SBV.Utils.PrettyNum (
         PrettyNum(..), readBin, shex, chex, shexI, sbin, sbinI
-      , showCFloat, showCDouble, showHFloat, showHDouble
+      , showCFloat, showCDouble, showHFloat, showHDouble, showBFloat, showFloatAtBase
       , showSMTFloat, showSMTDouble, smtRoundingMode, cvToSMTLib, mkSkolemZero
+      , showNegativeNumber
       ) where
 
-import Data.Char  (intToDigit, ord)
+import Data.Bits  ((.&.), countTrailingZeros)
+import Data.Char  (intToDigit, ord, chr, toUpper)
 import Data.Int   (Int8, Int16, Int32, Int64)
 import Data.List  (isPrefixOf)
 import Data.Maybe (fromJust, fromMaybe, listToMaybe)
@@ -29,14 +31,14 @@ import Data.Word  (Word8, Word16, Word32, Word64)
 
 import qualified Data.Set as Set
 
-import Numeric (showIntAtBase, showHex, readInt)
-import qualified Numeric (showHFloat)
-
-import Data.Numbers.CrackNum (floatToFP, doubleToFP)
+import Numeric (showIntAtBase, showHex, readInt, floatToDigits)
+import qualified Numeric as N (showHFloat)
 
 import Data.SBV.Core.Data
-import Data.SBV.Core.Kind (smtType)
-import Data.SBV.Core.AlgReals (algRealToSMTLib2)
+import Data.SBV.Core.Kind (smtType, smtRoundingMode, showBaseKind)
+
+import Data.SBV.Core.AlgReals    (algRealToSMTLib2)
+import Data.SBV.Core.SizedFloats (fprToSMTLib2, bfToString)
 
 import Data.SBV.Utils.Lib (stringToQFS)
 
@@ -162,60 +164,69 @@ instance PrettyNum Integer where
   hex  = shexI False False
   bin  = sbinI False False
 
+shBKind :: HasKind a => a -> String
+shBKind a = " :: " ++ showBaseKind (kindOf a)
+
 instance PrettyNum CV where
-  hexS cv | isUninterpreted cv = show cv ++ " :: " ++ show (kindOf cv)
-          | isBoolean       cv = hexS (cvToBool cv) ++ " :: Bool"
-          | isFloat         cv = let CFloat   f = cvVal cv in show f ++ " :: Float\n"  ++ show (floatToFP f)
-          | isDouble        cv = let CDouble  d = cvVal cv in show d ++ " :: Double\n" ++ show (doubleToFP d)
-          | isReal          cv = let CAlgReal r = cvVal cv in show r ++ " :: Real"
-          | isString        cv = let CString  s = cvVal cv in show s ++ " :: String"
+  hexS cv | isUserSort      cv = shows cv                                               $  shBKind cv
+          | isBoolean       cv = hexS (cvToBool cv)                                     ++ shBKind cv
+          | isFloat         cv = let CFloat   f = cvVal cv in N.showHFloat f            $  shBKind cv
+          | isDouble        cv = let CDouble  d = cvVal cv in N.showHFloat d            $  shBKind cv
+          | isFP            cv = let CFP      f = cvVal cv in bfToString 16 True True f ++ shBKind cv
+          | isReal          cv = let CAlgReal r = cvVal cv in show r                    ++ shBKind cv
+          | isString        cv = let CString  s = cvVal cv in show s                    ++ shBKind cv
           | not (isBounded cv) = let CInteger i = cvVal cv in shexI True True i
           | True               = let CInteger i = cvVal cv in shex  True True (hasSign cv, intSizeOf cv) i
 
-  binS cv | isUninterpreted cv = show cv  ++ " :: " ++ show (kindOf cv)
-          | isBoolean       cv = binS (cvToBool cv)  ++ " :: Bool"
-          | isFloat         cv = let CFloat   f = cvVal cv in show f ++ " :: Float\n"  ++ show (floatToFP f)
-          | isDouble        cv = let CDouble  d = cvVal cv in show d ++ " :: Double\n" ++ show (doubleToFP d)
-          | isReal          cv = let CAlgReal r = cvVal cv in show r ++ " :: Real"
-          | isString        cv = let CString  s = cvVal cv in show s ++ " :: String"
+  binS cv | isUserSort      cv = shows cv                                              $  shBKind cv
+          | isBoolean       cv = binS (cvToBool cv)                                    ++ shBKind cv
+          | isFloat         cv = let CFloat   f = cvVal cv in showBFloat f             $  shBKind cv
+          | isDouble        cv = let CDouble  d = cvVal cv in showBFloat d             $  shBKind cv
+          | isFP            cv = let CFP      f = cvVal cv in bfToString 2 True True f ++ shBKind cv
+          | isReal          cv = let CAlgReal r = cvVal cv in shows r                  $  shBKind cv
+          | isString        cv = let CString  s = cvVal cv in shows s                  $  shBKind cv
           | not (isBounded cv) = let CInteger i = cvVal cv in sbinI True True i
           | True               = let CInteger i = cvVal cv in sbin  True True (hasSign cv, intSizeOf cv) i
 
-  hexP cv | isUninterpreted cv = show cv
+  hexP cv | isUserSort      cv = show cv
           | isBoolean       cv = hexS (cvToBool cv)
           | isFloat         cv = let CFloat   f = cvVal cv in show f
           | isDouble        cv = let CDouble  d = cvVal cv in show d
+          | isFP            cv = let CFP      f = cvVal cv in bfToString 16 True True f
           | isReal          cv = let CAlgReal r = cvVal cv in show r
           | isString        cv = let CString  s = cvVal cv in show s
           | not (isBounded cv) = let CInteger i = cvVal cv in shexI False True i
           | True               = let CInteger i = cvVal cv in shex  False True (hasSign cv, intSizeOf cv) i
 
-  binP cv | isUninterpreted cv = show cv
+  binP cv | isUserSort      cv = show cv
           | isBoolean       cv = binS (cvToBool cv)
           | isFloat         cv = let CFloat   f = cvVal cv in show f
           | isDouble        cv = let CDouble  d = cvVal cv in show d
+          | isFP            cv = let CFP      f = cvVal cv in bfToString 2 True True f
           | isReal          cv = let CAlgReal r = cvVal cv in show r
           | isString        cv = let CString  s = cvVal cv in show s
           | not (isBounded cv) = let CInteger i = cvVal cv in sbinI False True i
           | True               = let CInteger i = cvVal cv in sbin  False True (hasSign cv, intSizeOf cv) i
 
-  hex cv | isUninterpreted cv = show cv
-         | isBoolean       cv = hexS (cvToBool cv)
-         | isFloat         cv = let CFloat   f = cvVal cv in show f
-         | isDouble        cv = let CDouble  d = cvVal cv in show d
-         | isReal          cv = let CAlgReal r = cvVal cv in show r
-         | isString        cv = let CString  s = cvVal cv in show s
-         | not (isBounded cv) = let CInteger i = cvVal cv in shexI False False i
-         | True               = let CInteger i = cvVal cv in shex  False False (hasSign cv, intSizeOf cv) i
+  hex cv  | isUserSort      cv = show cv
+          | isBoolean       cv = hexS (cvToBool cv)
+          | isFloat         cv = let CFloat   f = cvVal cv in show f
+          | isDouble        cv = let CDouble  d = cvVal cv in show d
+          | isFP            cv = let CFP      f = cvVal cv in bfToString 16 False True f
+          | isReal          cv = let CAlgReal r = cvVal cv in show r
+          | isString        cv = let CString  s = cvVal cv in show s
+          | not (isBounded cv) = let CInteger i = cvVal cv in shexI False False i
+          | True               = let CInteger i = cvVal cv in shex  False False (hasSign cv, intSizeOf cv) i
 
-  bin cv | isUninterpreted cv = show cv
-         | isBoolean       cv = binS (cvToBool cv)
-         | isFloat         cv = let CFloat   f = cvVal cv in show f
-         | isDouble        cv = let CDouble  d = cvVal cv in show d
-         | isReal          cv = let CAlgReal r = cvVal cv in show r
-         | isString        cv = let CString  s = cvVal cv in show s
-         | not (isBounded cv) = let CInteger i = cvVal cv in sbinI False False i
-         | True               = let CInteger i = cvVal cv in sbin  False False (hasSign cv, intSizeOf cv) i
+  bin cv  | isUserSort      cv = show cv
+          | isBoolean       cv = binS (cvToBool cv)
+          | isFloat         cv = let CFloat   f = cvVal cv in show f
+          | isDouble        cv = let CDouble  d = cvVal cv in show d
+          | isFP            cv = let CFP      f = cvVal cv in bfToString 2 False True f
+          | isReal          cv = let CAlgReal r = cvVal cv in show r
+          | isString        cv = let CString  s = cvVal cv in show s
+          | not (isBounded cv) = let CInteger i = cvVal cv in sbinI False False i
+          | True               = let CInteger i = cvVal cv in sbin  False False (hasSign cv, intSizeOf cv) i
 
 instance (SymVal a, PrettyNum a) => PrettyNum (SBV a) where
   hexS s = maybe (show s) (hexS :: a -> String) $ unliteral s
@@ -228,7 +239,7 @@ instance (SymVal a, PrettyNum a) => PrettyNum (SBV a) where
   bin  s = maybe (show s) (bin  :: a -> String) $ unliteral s
 
 -- | Show as a hexadecimal value. First bool controls whether type info is printed
--- while the second boolean controls wether 0x prefix is printed. The tuple is
+-- while the second boolean controls whether 0x prefix is printed. The tuple is
 -- the signedness and the bit-length of the input. The length of the string
 -- will /not/ depend on the value, but rather the bit-length.
 shex :: (Show a, Integral a) => Bool -> Bool -> (Bool, Int) -> a -> String
@@ -337,7 +348,7 @@ showCFloat f
    | isNaN f             = "((float) NAN)"
    | isInfinite f, f < 0 = "((float) (-INFINITY))"
    | isInfinite f        = "((float) INFINITY)"
-   | True                = Numeric.showHFloat f $ "F /* " ++ show f ++ "F */"
+   | True                = N.showHFloat f $ "F /* " ++ show f ++ "F */"
 
 -- | A version of show for doubles that generates correct C literals for nan/infinite. NB. Requires "math.h" to be included.
 showCDouble :: Double -> String
@@ -345,7 +356,7 @@ showCDouble d
    | isNaN d             = "((double) NAN)"
    | isInfinite d, d < 0 = "((double) (-INFINITY))"
    | isInfinite d        = "((double) INFINITY)"
-   | True                = Numeric.showHFloat d " /* " ++ show d ++ " */"
+   | True                = N.showHFloat d " /* " ++ show d ++ " */"
 
 -- | A version of show for floats that generates correct Haskell literals for nan/infinite
 showHFloat :: Float -> String
@@ -374,6 +385,7 @@ showSMTFloat rm f
    | True                = "((_ to_fp 8 24) " ++ smtRoundingMode rm ++ " " ++ toSMTLibRational (toRational f) ++ ")"
    where as s = "(_ " ++ s ++ " 8 24)"
 
+
 -- | A version of show for doubles that generates correct SMTLib literals using the rounding mode
 showSMTDouble :: RoundingMode -> Double -> String
 showSMTDouble rm d
@@ -385,7 +397,11 @@ showSMTDouble rm d
    | True                = "((_ to_fp 11 53) " ++ smtRoundingMode rm ++ " " ++ toSMTLibRational (toRational d) ++ ")"
    where as s = "(_ " ++ s ++ " 11 53)"
 
--- | Show a rational in SMTLib format
+-- | Show an SBV rational as an SMTLib value. This is used for faithful rationals.
+showSMTRational :: Rational -> String
+showSMTRational r = "(SBV.Rational " ++ showNegativeNumber (numerator r) ++ " " ++ showNegativeNumber (denominator r) ++ ")"
+
+-- | Show a rational in SMTLib format. This is used for conversions from regular rationals.
 toSMTLibRational :: Rational -> String
 toSMTLibRational r
    | n < 0
@@ -395,22 +411,16 @@ toSMTLibRational r
   where n = numerator r
         d = denominator r
 
--- | Convert a rounding mode to the format SMT-Lib2 understands.
-smtRoundingMode :: RoundingMode -> String
-smtRoundingMode RoundNearestTiesToEven = "roundNearestTiesToEven"
-smtRoundingMode RoundNearestTiesToAway = "roundNearestTiesToAway"
-smtRoundingMode RoundTowardPositive    = "roundTowardPositive"
-smtRoundingMode RoundTowardNegative    = "roundTowardNegative"
-smtRoundingMode RoundTowardZero        = "roundTowardZero"
-
 -- | Convert a CV to an SMTLib2 compliant value
 cvToSMTLib :: RoundingMode -> CV -> String
 cvToSMTLib rm x
   | isBoolean       x, CInteger  w      <- cvVal x = if w == 0 then "false" else "true"
-  | isUninterpreted x, CUserSort (_, s) <- cvVal x = roundModeConvert s
+  | isUserSort      x, CUserSort (_, s) <- cvVal x = roundModeConvert s
   | isReal          x, CAlgReal  r      <- cvVal x = algRealToSMTLib2 r
   | isFloat         x, CFloat    f      <- cvVal x = showSMTFloat  rm f
   | isDouble        x, CDouble   d      <- cvVal x = showSMTDouble rm d
+  | isRational      x, CRational r      <- cvVal x = showSMTRational r
+  | isFP            x, CFP       f      <- cvVal x = fprToSMTLib2 f
   | not (isBounded x), CInteger  w      <- cvVal x = if w >= 0 then show w else "(- " ++ show (abs w) ++ ")"
   | not (hasSign x)  , CInteger  w      <- cvVal x = smtLibHex (intSizeOf x) w
   -- signed numbers (with 2's complement representation) is problematic
@@ -419,7 +429,7 @@ cvToSMTLib rm x
   | hasSign x        , CInteger  w      <- cvVal x = if w == negate (2 ^ intSizeOf x)
                                                      then mkMinBound (intSizeOf x)
                                                      else negIf (w < 0) $ smtLibHex (intSizeOf x) (abs w)
-  | isChar x         , CChar c          <- cvVal x = smtLibHex 8 (fromIntegral (ord c))
+  | isChar x         , CChar c          <- cvVal x = "(_ char " ++ smtLibHex 8 (fromIntegral (ord c)) ++ ")"
   | isString x       , CString s        <- cvVal x = '\"' : stringToQFS s ++ "\""
   | isList x         , CList xs         <- cvVal x = smtLibSeq (kindOf x) xs
   | isSet x          , CSet s           <- cvVal x = smtLibSet (kindOf x) s
@@ -471,7 +481,7 @@ cvToSMTLib rm x
         dtConstructor fld args res = "((as " ++ fld ++ " " ++ smtType res ++ ") " ++ unwords args ++ ")"
 
         smtLibMaybe :: Kind -> Maybe CVal -> String
-        smtLibMaybe km@ KMaybe{}  Nothing   = dtConstructor "nothing_SBVMaybe" []                       km
+        smtLibMaybe km@KMaybe{}   Nothing   = dtConstructor "nothing_SBVMaybe" []                       km
         smtLibMaybe km@(KMaybe k) (Just  c) = dtConstructor "just_SBVMaybe"    [cvToSMTLib rm (CV k c)] km
         smtLibMaybe k             _         = error $ "SBV.cvToSMTLib: Impossible case (smtLibMaybe), received kind: " ++ show k
 
@@ -487,6 +497,74 @@ cvToSMTLib rm x
 
 -- | Create a skolem 0 for the kind
 mkSkolemZero :: RoundingMode -> Kind -> String
-mkSkolemZero _ (KUninterpreted _ (Right (f:_))) = f
-mkSkolemZero _ (KUninterpreted s _)             = error $ "SBV.mkSkolemZero: Unexpected uninterpreted sort: " ++ s
-mkSkolemZero rm k                               = cvToSMTLib rm (mkConstCV k (0::Integer))
+mkSkolemZero _ (KUserSort _ (Just (f:_))) = f
+mkSkolemZero _ (KUserSort s _)            = error $ "SBV.mkSkolemZero: Unexpected user sort: " ++ s
+mkSkolemZero rm k                         = cvToSMTLib rm (mkConstCV k (0::Integer))
+
+-- | Show a float as a binary
+showBFloat :: (Show a, RealFloat a) => a -> ShowS
+showBFloat = showFloatAtBase 2
+
+-- | Like Haskell's showHFloat, but uses arbitrary base instead.
+-- Note that the exponent is always written in decimal. Let the exponent value be d:
+--    If base=10, then we use @e@ to denote the exponent; meaning 10^d
+--    If base is a power of 2, then we use @p@ to denote the exponent; meaning 2^d
+--    Otherwise, we use @ to denote the exponent, and it means base^d
+showFloatAtBase :: (Show a, RealFloat a) => Int -> a -> ShowS
+showFloatAtBase base input
+  | base < 2 = error $ "showFloatAtBase: Received invalid base (must be >= 2): " ++ show base
+  | True     = showString $ fmt input
+  where fmt x
+         | isNaN x                   = "NaN"
+         | isInfinite x              = (if x < 0 then "-" else "") ++ "Infinity"
+         | x < 0 || isNegativeZero x = '-' : cvt (-x)
+         | True                      = cvt x
+
+        basePow2 = base .&. (base-1) == 0
+        lg2Base  = countTrailingZeros base  -- only used when basePow2 is true
+
+        prefix = case base of
+                   2  -> "0b"
+                   8  -> "0o"
+                   10 -> ""
+                   16 -> "0x"
+                   x  -> "0<" ++ show x ++ ">"
+
+        powChar
+          | base == 10 = 'e'
+          | basePow2   = 'p'
+          | True       = '@'
+
+        -- why r-1? Because we're shifting the fraction by 1 digit; does reducing the exponent by 1
+        f2d x = case floatToDigits (fromIntegral base) x of
+                  ([],   e) -> (0, [], e - 1)
+                  (d:ds, e) -> (d, ds, e - 1)
+
+        cvt x
+         | x == 0 = prefix ++ '0' : powChar : "+0"
+         | True   = prefix ++ toDigit d ++ frac ds ++ pow
+         where (d, ds, e)  = f2d x
+               pow
+                | base == 10 = powChar : shSigned e
+                | basePow2   = powChar : shSigned (e * lg2Base)
+                | True       = powChar : shSigned e
+
+               shSigned v
+                | v < 0      =       show v
+                | True       = '+' : show v
+
+        -- Given digits, show them except if they're all 0 then drop
+        frac digits
+         | all (== 0) digits = ""
+         | True              = "." ++ concatMap toDigit digits
+
+        toDigit v = map toUpper d
+           where d | v <= 15 = [intToDigit v]
+                   | v <  36 = [chr (ord 'a' + v - 10)]
+                   | True    = '<' : show v ++ ">"
+
+-- | When we show a negative number in SMTLib, we must properly parenthesize.
+showNegativeNumber :: (Show a, Num a, Ord a) => a -> String
+showNegativeNumber i
+  | i < 0 = "(- " ++ show (-i) ++ ")"
+  | True  = show i
