@@ -18,26 +18,28 @@ module Data.SBV.Utils.SExpr (SExpr(..), parenDeficit, parseSExpr, parseSExprFunc
 import Data.Bits   (setBit, testBit)
 import Data.Char   (isDigit, ord, isSpace)
 import Data.Either (partitionEithers)
-import Data.List   (isPrefixOf)
+import Data.List   (isPrefixOf, nubBy)
 import Data.Maybe  (fromMaybe, listToMaybe)
 import Data.Word   (Word32, Word64)
 
+import Control.Monad (foldM)
+
 import Numeric    (readInt, readDec, readHex, fromRat)
 
 import Data.SBV.Core.AlgReals
+import Data.SBV.Core.SizedFloats
 import Data.SBV.Core.Data (nan, infinity, RoundingMode(..))
 
-import Data.SBV.Utils.Numeric (fpIsEqualObjectH)
-
-import Data.Numbers.CrackNum (wordToFloat, wordToDouble)
+import Data.SBV.Utils.Numeric (fpIsEqualObjectH, wordToFloat, wordToDouble)
 
 -- | ADT S-Expression format, suitable for representing get-model output of SMT-Lib
-data SExpr = ECon    String
-           | ENum    (Integer, Maybe Int)  -- Second argument is how wide the field was in bits, if known. Useful in FP parsing.
-           | EReal   AlgReal
-           | EFloat  Float
-           | EDouble Double
-           | EApp    [SExpr]
+data SExpr = ECon           String
+           | ENum           (Integer, Maybe Int)  -- Second argument is how wide the field was in bits, if known. Useful in FP parsing.
+           | EReal          AlgReal
+           | EFloat         Float
+           | EFloatingPoint FP
+           | EDouble        Double
+           | EApp           [SExpr]
            deriving Show
 
 -- | Extremely simple minded tokenizer, good for our use model.
@@ -124,12 +126,17 @@ parseSExpr inp = do (sexp, extras) <- parse inpToks
         pTok ('#':'b':r)                                 = mkNum (Just (length r))     $ readInt 2 (`elem` "01") (\c -> ord c - ord '0') r
         pTok ('#':'x':r)                                 = mkNum (Just (4 * length r)) $ readHex r
 
-        pTok n
-          | not (null n) && isDigit (head n)
-          = if '.' `elem` n then getReal n
-            else mkNum Nothing $ readDec n
+        pTok n | possiblyNum n = if all intChar n then mkNum Nothing $ readDec n else getReal n
         pTok n                 = return $ ECon (constantMap n)
 
+        -- crude, but effective!
+        possiblyNum s = case s of
+                          ""        -> False
+                          ('-':c:_) -> isDigit c
+                          (c:_)     -> isDigit c
+
+        intChar c = c == '-' || isDigit c
+
         mkNum l [(n, "")] = return $ ENum (n, l)
         mkNum _ _         = die "cannot read number"
 
@@ -169,18 +176,30 @@ parseSExpr inp = do (sexp, extras) <- parse inpToks
                                                   _           -> die $ "Cannot parse a CVC4 approximate value from: " ++ show x
 
         -- NB. Note the lengths on the mantissa for the following two are 23/52; not 24/53!
-        cvt (EApp [ECon "fp",    ENum (s, Just 1), ENum ( e, Just 8),  ENum (m, Just 23)])           = return $ EFloat  $ getTripleFloat  s e m
-        cvt (EApp [ECon "fp",    ENum (s, Just 1), ENum ( e, Just 11), ENum (m, Just 52)])           = return $ EDouble $ getTripleDouble s e m
-        cvt (EApp [ECon "_",     ECon "NaN",       ENum ( 8, _),       ENum (24,      _)])           = return $ EFloat  nan
-        cvt (EApp [ECon "_",     ECon "NaN",       ENum (11, _),       ENum (53,      _)])           = return $ EDouble nan
-        cvt (EApp [ECon "_",     ECon "+oo",       ENum ( 8, _),       ENum (24,      _)])           = return $ EFloat  infinity
-        cvt (EApp [ECon "_",     ECon "+oo",       ENum (11, _),       ENum (53,      _)])           = return $ EDouble infinity
-        cvt (EApp [ECon "_",     ECon "-oo",       ENum ( 8, _),       ENum (24,      _)])           = return $ EFloat  (-infinity)
-        cvt (EApp [ECon "_",     ECon "-oo",       ENum (11, _),       ENum (53,      _)])           = return $ EDouble (-infinity)
+        cvt (EApp [ECon "fp",    ENum (s, Just 1), ENum ( e, Just 8),  ENum (m, Just 23)])           = return $ EFloat         $ getTripleFloat  s e m
+        cvt (EApp [ECon "fp",    ENum (s, Just 1), ENum ( e, Just 11), ENum (m, Just 52)])           = return $ EDouble        $ getTripleDouble s e m
+        cvt (EApp [ECon "fp",    ENum (s, Just 1), ENum ( e, Just eb), ENum (m, Just sb)])           = return $ EFloatingPoint $ fpFromRawRep (s == 1) (e, eb) (m, sb+1)
+
+        cvt (EApp [ECon "_",     ECon "NaN",       ENum ( 8, _),       ENum (24,      _)])           = return $ EFloat           nan
+        cvt (EApp [ECon "_",     ECon "NaN",       ENum (11, _),       ENum (53,      _)])           = return $ EDouble          nan
+        cvt (EApp [ECon "_",     ECon "NaN",       ENum (eb, _),       ENum (sb,      _)])           = return $ EFloatingPoint $ fpNaN (fromIntegral eb) (fromIntegral sb)
+
+        cvt (EApp [ECon "_",     ECon "+oo",       ENum ( 8, _),       ENum (24,      _)])           = return $ EFloat           infinity
+        cvt (EApp [ECon "_",     ECon "+oo",       ENum (11, _),       ENum (53,      _)])           = return $ EDouble          infinity
+        cvt (EApp [ECon "_",     ECon "+oo",       ENum (eb, _),       ENum (sb,      _)])           = return $ EFloatingPoint $ fpInf False (fromIntegral eb) (fromIntegral sb)
+
+        cvt (EApp [ECon "_",     ECon "-oo",       ENum ( 8, _),       ENum (24,      _)])           = return $ EFloat         $ -infinity
+        cvt (EApp [ECon "_",     ECon "-oo",       ENum (11, _),       ENum (53,      _)])           = return $ EDouble        $ -infinity
+        cvt (EApp [ECon "_",     ECon "-oo",       ENum (eb, _),       ENum (sb,      _)])           = return $ EFloatingPoint $ fpInf True (fromIntegral eb) (fromIntegral sb)
+
         cvt (EApp [ECon "_",     ECon "+zero",     ENum ( 8, _),       ENum (24,      _)])           = return $ EFloat  0
         cvt (EApp [ECon "_",     ECon "+zero",     ENum (11, _),       ENum (53,      _)])           = return $ EDouble 0
-        cvt (EApp [ECon "_",     ECon "-zero",     ENum ( 8, _),       ENum (24,      _)])           = return $ EFloat  (-0)
-        cvt (EApp [ECon "_",     ECon "-zero",     ENum (11, _),       ENum (53,      _)])           = return $ EDouble (-0)
+        cvt (EApp [ECon "_",     ECon "+zero",     ENum (eb, _),       ENum (sb,      _)])           = return $ EFloatingPoint $ fpZero False (fromIntegral eb) (fromIntegral sb)
+
+        cvt (EApp [ECon "_",     ECon "-zero",     ENum ( 8, _),       ENum (24,      _)])           = return $ EFloat         $ -0
+        cvt (EApp [ECon "_",     ECon "-zero",     ENum (11, _),       ENum (53,      _)])           = return $ EDouble        $ -0
+        cvt (EApp [ECon "_",     ECon "-zero",     ENum (eb, _),       ENum (sb,      _)])           = return $ EFloatingPoint $ fpZero True (fromIntegral eb) (fromIntegral sb)
+
         cvt x                                                                                        = return x
 
         getCoeff (EApp [ECon "*", ENum k, EApp [ECon "^", ECon "x", ENum p]]) = return (fst k, fst p)  -- kx^p
@@ -261,9 +280,84 @@ constantMap n = fromMaybe n (listToMaybe [to | (from, to) <- special, n `elem` f
 parseSExprFunction :: SExpr -> Maybe (Either String ([([SExpr], SExpr)], SExpr))
 parseSExprFunction e
   | Just r <- parseLambdaExpression  e = Just (Right r)
+  | Just r <- parseSetLambda         e = Just (Right r)
   | Just r <- parseStoreAssociations e = Just r
   | True                               = Nothing         -- out-of luck. NB. This is where we would add support for other solvers!
 
+-- | Parse a set-lambda expression, which is literally a lambda function, that might look like this:
+--        (lambda ((x!1 String))
+--          (or (not (or (= x!1 "o") (= x!1 "l") (= x!1 "e") (= x!1 "h")))
+--              (= x!1 "o")
+--              (= x!1 "l")
+--              (= x!1 "e")
+--              (= x!1 "h")))
+--   For this, we do a little bit of an interpretative dance to see if we can "construct" the necesary expression.
+--
+--   In parsed form:
+--      EApp [ECon "lambda",EApp [EApp [ECon "x!1",ECon "String"]],EApp [ECon "not",EApp [ECon "or",EApp [ECon "=",ECon "x!1",ECon "\"e\""],EApp [ECon "=",ECon "x!1",ECon "\"l\""]]]]
+--
+--   This is by no means comprehensive, and is quite crude, but hopefully covers the cases we see in practice.
+parseSetLambda :: SExpr -> Maybe ([([SExpr], SExpr)], SExpr)
+parseSetLambda funExpr = case funExpr of
+                               EApp [l@(ECon "lambda"), bv@(EApp [EApp [ECon _, _]]), body] -> go (\bd -> EApp [l, bv, bd]) body
+                               _                                                            -> Nothing
+  where go mkLambda = build
+         where build (EApp [ECon "not",  rest      ]) =         neg =<<      build rest
+               build (EApp (ECon "or"  : rest@(_:_))) = foldM1 disj =<< mapM build rest
+               build (EApp (ECon "and" : rest@(_:_))) = foldM1 conj =<< mapM build rest
+               build other                            = parseLambdaExpression (mkLambda other)
+
+        -- We're guaranteed by above construction that foldM1 will never take an empty list (due to rest@(_:_) pattern match.)
+        foldM1 _ []     = error "Data.SBV.parseSetLambda: Impossible happened; empty arg to foldM1"
+        foldM1 f (x:xs) = foldM f x xs
+
+        checkBool (ENum (1, Nothing)) = True
+        checkBool (ENum (0, Nothing)) = True
+        checkBool _                   = False
+
+        negBool (ENum (1, Nothing)) = ENum (0, Nothing)
+        negBool _                   = ENum (1, Nothing)
+
+        orBool t@(ENum (1, Nothing)) _                      = t
+        orBool _                     t@(ENum (1, Nothing))  = t
+        orBool _ _                                          = ENum (0, Nothing)
+
+        andBool f@(ENum (0, Nothing)) _                     = f
+        andBool _                     f@(ENum (0, Nothing)) = f
+        andBool _ _                                         = ENum (1, Nothing)
+
+        neg :: ([([SExpr], SExpr)], SExpr) -> Maybe ([([SExpr], SExpr)], SExpr)
+        neg (rows, dflt)
+         | all checkBool (dflt : map snd rows) = Just ([(e, negBool r) | (e, r) <- rows], negBool dflt)
+         | True                                = Nothing
+
+        disj, conj :: ([([SExpr], SExpr)], SExpr) -> ([([SExpr], SExpr)], SExpr) -> Maybe ([([SExpr], SExpr)], SExpr)
+        disj = bin orBool
+        conj = bin andBool
+
+        bin f rd1@(rows1, dflt1) rd2@(rows2, dflt2)
+          | all checkBool (dflt1 : dflt2 : map snd rows1 ++ map snd rows2) = Just (combine f rd1 rd2)
+          | True                                                           = Nothing
+
+        -- Since we don't have equality over SExprs (can of worms!), we use "show" equality here. The ice is thin, but it works!
+        combine f (rows1, dflt1) (rows2, dflt2) = (rows, f dflt1 dflt2)
+          where rows = map calc $ nubBy (\x y -> show x == show y) (map fst rows1 ++ map fst rows2)
+
+                calc :: [SExpr] -> ([SExpr], SExpr)
+                calc args = (args, f (find rows1 dflt1 args) (find rows2 dflt2 args))
+
+                find rs d a = case [r | (v, r) <- rs, show v == show a] of
+                               []  -> d
+                               [x] -> x
+                               x   -> error $ unlines [ "Data.SBV.parseSetLambda: Impossible happened while combining rows."
+                                                      , "   First row  :"   ++ show rows1
+                                                      , "   First dflt :"  ++ show dflt1
+                                                      , "   Second row :"  ++ show rows2
+                                                      , "   Second dflt:" ++ show dflt2
+                                                      , "   Looking for: " ++ show a
+                                                      , "Multiple matches found: " ++ show x
+                                                      ]
+
 -- | Parse a lambda expression, most likely z3 specific. There's some guess work
 -- involved here regarding how z3 produces lambda-expressions; while we try to
 -- be flexible, this is certainly not a full fledged parser. But hopefully it'll
@@ -412,11 +506,12 @@ chainAssigns chain = regroup $ partitionEithers chain
         -- you ever get the error line above fire, because you must've disabled the pattern-match
         -- completion check warning! Shame on you.
         eRank :: SExpr -> Int
-        eRank ECon{}    = 0
-        eRank ENum{}    = 1
-        eRank EReal{}   = 2
-        eRank EFloat{}  = 3
-        eRank EDouble{} = 4
-        eRank EApp{}    = 5
+        eRank ECon{}           = 0
+        eRank ENum{}           = 1
+        eRank EReal{}          = 2
+        eRank EFloat{}         = 3
+        eRank EFloatingPoint{} = 4
+        eRank EDouble{}        = 5
+        eRank EApp{}           = 6
 
 {-# ANN chainAssigns ("HLint: ignore Redundant if" :: String) #-}
