@@ -40,17 +40,14 @@ import Data.SBV
 --   mask = 0x8080808080808080 :: Word64
 --   mult = 0x0002040810204081 :: Word64
 --
--- That is, any 64 bit value masked by the first and multipled by the second
+-- That is, any 64 bit value masked by the first and multiplied by the second
 -- value above will have its bits at positions @[7,15,23,31,39,47,55,63]@ moved
 -- to positions @[56,57,58,59,60,61,62,63]@ respectively.
 --
 -- NB. Depending on your z3 version, you might also get the following
 -- multiplier as the result: 0x8202040810204081. That value works just fine as well!
---
--- Note that we have to use a custom SAT command for this problem, as otherwise it takes too
--- long. See <https://github.com/Z3Prover/z3/issues/2587> for details.
 maskAndMult :: IO ()
-maskAndMult = print =<< satWith z3{printBase=16, satCmd = "(check-sat-using (and-then simplify smtfd))"} find
+maskAndMult = print =<< satWith z3{printBase=16} find
   where find = do mask <- exists "mask"
                   mult <- exists "mult"
                   inp  <- forall "inp"
