@@ -36,25 +36,10 @@ data State = Idle     -- ^ Regular work
 -- | Make 'State' a symbolic enumeration
 mkSymbolicEnumeration ''State
 
--- | The type synonym 'SState' is mnemonic for symbolic state.
-type SState = SBV State
-
--- | Symbolic version of 'Idle'
-idle :: SState
-idle = literal Idle
-
--- | Symbolic version of 'Ready'
-ready :: SState
-ready = literal Ready
-
--- | Symbolic version of 'Critical'
-critical :: SState
-critical = literal Critical
-
 -- | A bounded mutex property holds for two sequences of state transitions, if they are not in
 -- their critical section at the same time up to that given bound.
 mutex :: Int -> SList State -> SList State -> SBool
-mutex i p1s p2s = L.band i $ L.bzipWith i (\p1 p2 -> p1 ./= critical .|| p2 ./= critical) p1s p2s
+mutex i p1s p2s = L.band i $ L.bzipWith i (\p1 p2 -> p1 ./= sCritical .|| p2 ./= sCritical) p1s p2s
 
 -- | A sequence is valid upto a bound if it starts at 'Idle', and follows the mutex rules. That is:
 --
@@ -65,19 +50,19 @@ mutex i p1s p2s = L.band i $ L.bzipWith i (\p1 p2 -> p1 ./= critical .|| p2 ./=
 -- The variable @me@ identifies the agent id.
 validSequence :: Int -> Integer -> SList Integer -> SList State -> SBool
 validSequence b me pturns proc = sAnd [ L.length proc .== fromIntegral b
-                                      , idle .== L.head proc
-                                      , check b pturns proc idle
+                                      , sIdle .== L.head proc
+                                      , check b pturns proc sIdle
                                       ]
    where check 0 _  _  _    = sTrue
          check i ts ps prev = let (cur,  rest)  = L.uncons ps
                                   (turn, turns) = L.uncons ts
-                                  ok   = ite (prev .== idle)                          (cur `sElem` [idle, ready])
-                                       $ ite (prev .== ready .&& turn .== literal me) (cur `sElem` [critical])
-                                       $ ite (prev .== critical)                      (cur `sElem` [critical, idle])
-                                                                                      (cur `sElem` [prev])
+                                  ok   = ite (prev .== sIdle)                          (cur `sElem` [sIdle, sReady])
+                                       $ ite (prev .== sReady .&& turn .== literal me) (cur `sElem` [sCritical])
+                                       $ ite (prev .== sCritical)                      (cur `sElem` [sCritical, sIdle])
+                                                                                       (cur `sElem` [prev])
                               in ok .&& check (i-1) turns rest cur
 
--- | The mutex algorithm, coded implicity as an assignment to turns. Turns start at @1@, and at each stage is either
+-- | The mutex algorithm, coded implicitly as an assignment to turns. Turns start at @1@, and at each stage is either
 -- @1@ or @2@; giving preference to that process. The only condition is that if either process is in its critical
 -- section, then the turn value stays the same. Note that this is sufficient to satisfy safety (i.e., mutual
 -- exclusion), though it does not guarantee liveness.
@@ -88,7 +73,7 @@ validTurns b turns process1 process2 = sAnd [ L.length turns .== fromIntegral b
                                             ]
    where check 0 _  _     _     _    = sTrue
          check i ts proc1 proc2 prev =   cur `sElem` [1, 2]
-                                     .&& (p1 .== critical .|| p2 .== critical .=> cur .== prev)
+                                     .&& (p1 .== sCritical .|| p2 .== sCritical .=> cur .== prev)
                                      .&& check (i-1) rest p1s p2s cur
             where (cur, rest) = L.uncons ts
                   (p1,  p1s)  = L.uncons proc1
@@ -117,16 +102,17 @@ checkMutex b = runSMT $ do
 
                   query $ do cs <- checkSat
                              case cs of
-                               Unk   -> error "Solver said Unknown!"
-                               Unsat -> io . putStrLn $ "All is good!"
-                               Sat   -> do io . putStrLn $ "Violation detected!"
-                                           do p1V <- getValue p1
-                                              p2V <- getValue p2
-                                              ts  <- getValue turns
+                               Unk    -> error "Solver said Unknown!"
+                               DSat{} -> error "Solver said delta-satisfiable!"
+                               Unsat  -> io . putStrLn $ "All is good!"
+                               Sat    -> do io . putStrLn $ "Violation detected!"
+                                            do p1V <- getValue p1
+                                               p2V <- getValue p2
+                                               ts  <- getValue turns
 
-                                              io . putStrLn $ "P1: " ++ show p1V
-                                              io . putStrLn $ "P2: " ++ show p2V
-                                              io . putStrLn $ "Ts: " ++ show ts
+                                               io . putStrLn $ "P1: " ++ show p1V
+                                               io . putStrLn $ "P2: " ++ show p2V
+                                               io . putStrLn $ "Ts: " ++ show ts
 
 -- | Our algorithm is correct, but it is not fair. It does not guarantee that a process that
 -- wants to enter its critical-section will always do so eventually. Demonstrate this by
@@ -156,23 +142,24 @@ notFair b = runSMT $ do p1    :: SList State   <- sList "p1"
                         constrain $ validTurns    b turns p1 p2
 
                         -- Ensure that the second process becomes ready in the second cycle:
-                        constrain $ p2 .!! 1 .== ready
+                        constrain $ p2 .!! 1 .== sReady
 
                         -- Find a trace where p2 never goes critical
                         -- counter example, we would've found a violation!
-                        constrain $ sNot $ L.belem b critical p2
+                        constrain $ sNot $ L.belem b sCritical p2
 
                         query $ do cs <- checkSat
                                    case cs of
-                                     Unk   -> error "Solver said Unknown!"
-                                     Unsat -> error "Solver couldn't find a violating trace!"
-                                     Sat   -> do io . putStrLn $ "Fairness is violated at bound: " ++ show b
-                                                 do p1V <- getValue p1
-                                                    p2V <- getValue p2
-                                                    ts  <- getValue turns
+                                     Unk    -> error "Solver said Unknown!"
+                                     DSat{} -> error "Solver said delta-satisfiable!"
+                                     Unsat  -> error "Solver couldn't find a violating trace!"
+                                     Sat    -> do io . putStrLn $ "Fairness is violated at bound: " ++ show b
+                                                  do p1V <- getValue p1
+                                                     p2V <- getValue p2
+                                                     ts  <- getValue turns
 
-                                                    io . putStrLn $ "P1: " ++ show p1V
-                                                    io . putStrLn $ "P2: " ++ show p2V
-                                                    io . putStrLn $ "Ts: " ++ show ts
+                                                     io . putStrLn $ "P1: " ++ show p1V
+                                                     io . putStrLn $ "P2: " ++ show p2V
+                                                     io . putStrLn $ "Ts: " ++ show ts
 
 {-# ANN module ("HLint: ignore Reduce duplication" :: String) #-}
