@@ -36,18 +36,15 @@ data E = A | B | C
 -- | Make 'E' a symbolic value.
 mkSymbolicEnumeration ''E
 
--- | Give a name to the symbolic variants of 'E', for convenience
-type SE = SBV E
-
 -- | Have the SMT solver enumerate the elements of the domain. We have:
 --
 -- >>> elts
 -- Solution #1:
---   s0 = A :: E
+--   s0 = C :: E
 -- Solution #2:
 --   s0 = B :: E
 -- Solution #3:
---   s0 = C :: E
+--   s0 = A :: E
 -- Found 3 different solutions.
 elts :: IO AllSatResult
 elts = allSat $ \(x::SE) -> x .== x
@@ -71,7 +68,7 @@ maxE = sat $ do mx <- exists "maxE"
                 e  <- forall "e"
                 return $ mx .>= (e::SE)
 
--- | Similarly, we get the minumum element. We have:
+-- | Similarly, we get the minimum element. We have:
 --
 -- >>> minE
 -- Satisfiable. Model:
