@@ -7,13 +7,17 @@
 -- Stability : experimental
 --
 -- Several examples involving IEEE-754 floating point numbers, i.e., single
--- precision 'Float' ('SFloat') and double precision 'Double' ('SDouble') types.
+-- precision 'Float' ('SFloat'), double precision 'Double' ('SDouble'), and
+-- the generic 'SFloatingPoint' @eb@ @sb@ type where the user can specify the
+-- exponent and significand bit-widths. (Note that there is always an extra
+-- sign-bit, and the value of @sb@ includes the hidden bit.)
 --
--- Note that arithmetic with floating point is full of surprises; due to precision
+-- Arithmetic with floating point is full of surprises; due to precision
 -- issues associativity of arithmetic operations typically do not hold. Also,
 -- the presence of @NaN@ is always something to look out for.
 -----------------------------------------------------------------------------
 
+{-# LANGUAGE DataKinds           #-}
 {-# LANGUAGE ScopedTypeVariables #-}
 
 {-# OPTIONS_GHC -Wall -Werror #-}
@@ -58,19 +62,19 @@ assocPlus x y z = x + (y + z) .== (x + y) + z
 --
 -- >>> assocPlusRegular
 -- Falsifiable. Counter-example:
---   x =    128.00029 :: Float
---   y =  -7.27236e-4 :: Float
---   z = -6.875994e-3 :: Float
+--   x = -1.860921e38 :: Float
+--   y = 2.1240016e32 :: Float
+--   z = 2.2772656e33 :: Float
 --
 -- Indeed, we have:
 --
--- >>> let x =    128.00029 :: Float
--- >>> let y =  -7.27236e-4 :: Float
--- >>> let z = -6.875994e-3 :: Float
+-- >>> let x = -1.860921e38 :: Float
+-- >>> let y = 2.1240016e32 :: Float
+-- >>> let z = 2.2772656e33 :: Float
 -- >>> x + (y + z)
--- 127.99268
+-- -1.860896e38
 -- >>> (x + y) + z
--- 127.99269
+-- -1.8608963e38
 --
 -- Note the difference in the results!
 assocPlusRegular :: IO ThmResult
@@ -92,13 +96,13 @@ assocPlusRegular = prove $ do [x, y, z] <- sFloats ["x", "y", "z"]
 --
 -- >>> nonZeroAddition
 -- Falsifiable. Counter-example:
---   a =  5.060287e28 :: Float
---   b = 3.6780381e19 :: Float
+--   a =  3.2446953e28 :: Float
+--   b = -4.486113e-39 :: Float
 --
 -- Indeed, we have:
 --
--- >>> let a =  5.060287e28 :: Float
--- >>> let b = 3.6780381e19 :: Float
+-- >>> let a =  3.2446953e28 :: Float
+-- >>> let b = -4.486113e-39 :: Float
 -- >>> a + b == a
 -- True
 -- >>> b == 0
@@ -121,13 +125,13 @@ nonZeroAddition = prove $ do [a, b] <- sFloats ["a", "b"]
 --
 -- >>> multInverse
 -- Falsifiable. Counter-example:
---   a = 2.4907063e38 :: Float
+--   a = 2.944504e-39 :: Float
 --
 -- Indeed, we have:
 --
--- >>> let a = 2.4907063e38 :: Float
+-- >>> let a = 2.944504e-39 :: Float
 -- >>> a * (1/a)
--- 1.0000001
+-- 0.99999994
 multInverse :: IO ThmResult
 multInverse = prove $ do a <- sFloat "a"
                          constrain $ fpIsPoint a
@@ -148,29 +152,24 @@ multInverse = prove $ do a <- sFloat "a"
 -- >>> roundingAdd
 -- Satisfiable. Model:
 --   rm = RoundTowardPositive :: RoundingMode
---   x  =      -2.3509886e-38 :: Float
---   y  =            -6.0e-45 :: Float
+--   x  =       3.6969435e-35 :: Float
+--   y  =      -1.1628614e-38 :: Float
 --
 -- (Note that depending on your version of Z3, you might get a different result.)
--- Unfortunately we can't directly validate this result at the Haskell level, as Haskell only supports
--- 'RoundNearestTiesToEven'. We have:
+-- Unfortunately Haskell floats do not allow computation with arbitrary rounding modes, but SBV's
+-- 'SFloatingPoint' type does. We have:
 --
--- >>> -2.3509886e-38 + (-6.0e-45) :: Float
--- -2.3509893e-38
+-- >>> fpAdd sRoundNearestTiesToEven 3.6969435e-35 (-1.1628614e-38) :: SFPSingle
+-- 3.69578064e-35 :: SFloatingPoint 8 24
+-- >>> fpAdd sRoundTowardPositive    3.6969435e-35 (-1.1628614e-38) :: SFPSingle
+-- 3.69578093e-35 :: SFloatingPoint 8 24
 --
--- While we cannot directly see the result when the mode is 'RoundTowardPositive' in Haskell, we can use
--- SBV to provide us with that result thusly:
---
--- >>> sat $ \z -> z .== fpAdd sRoundTowardPositive (-2.3509886e-38) (-6.0e-45 :: SFloat)
--- Satisfiable. Model:
---   s0 = -2.350989e-38 :: Float
---
--- We can see why these two resuls are indeed different: The 'RoundTowardPositive'
+-- We can see why these two results are indeed different: The 'RoundTowardPositive'
 -- (which rounds towards positive infinity from zero) produces a larger result. Indeed, if we treat these numbers
 -- as 'Double' values, we get:
 --
--- >> -2.3509886e-38 + (-6.0e-45) :: Double
--- -2.3509892e-38
+-- >>> 3.6969435e-35 + (-1.1628614e-38) :: Double
+-- 3.6957806386e-35
 --
 -- we see that the "more precise" result is larger than what the 'Float' value is, justifying the
 -- larger value with 'RoundTowardPositive'. A more detailed study is beyond our current scope, so we'll
@@ -187,3 +186,40 @@ roundingAdd = sat $ do m :: SRoundingMode <- free "rm"
                        constrain $ fpIsPoint lhs
                        constrain $ fpIsPoint rhs
                        return $ lhs ./= rhs
+
+-- | Arbitrary precision floating-point numbers. SBV can talk about floating point numbers with arbitrary
+-- exponent and significand sizes as well. Here is a simple example demonstrating the minimum non-zero positive
+-- and maximum floating point values with exponent width 5 and significand width 4, which is actually 3
+-- bits for the significand explicitly stored, includes the hidden bit. We have:
+--
+-- >>> fp54Bounds
+-- Objective "max": Optimal model:
+--   x   = 61400 :: FloatingPoint 5 4
+--   max =   503 :: WordN 9
+--   min =   503 :: WordN 9
+-- Objective "min": Optimal model:
+--   x   = 0.00000763 :: FloatingPoint 5 4
+--   max =        257 :: WordN 9
+--   min =        257 :: WordN 9
+--
+-- The careful reader will notice that the numbers @61400@ and @0.00000763@ are quite suspicious, but the metric
+-- space equivalents are correct. The reason for this is due to the sparcity of floats. The "computed" value of
+-- the maximum in this bound is actually @61440@, however in @FloatingPoint 5 4@ representation all numbers
+-- between @57344@ and @61440@ collapse to the same bit-pattern, and the pretty-printer picks a string
+-- representation in decimal that falls within range that it considers is the "simplest." (Printing
+-- floats precisely is a thorny subject!) Likewise, the minimum value we're looking for is actually
+-- 2^-17, but any number between 2^-16 and 2^-17 will map to this number. It turns out that 0.00000763
+-- in decimal is one such value. Moral of the story is that when reading floating-point numbers in
+-- decimal notation one should be very careful about the printed representation and the numeric value; while
+-- they will match in vsalue (if there are no bugs!), they can print quite differently! (Also keep in
+-- mind the rounding modes that impact how the conversion is done.)
+fp54Bounds :: IO OptimizeResult
+fp54Bounds = optimize Independent $ do x :: SFloatingPoint 5 4 <- sFloatingPoint "x"
+
+                                       constrain $ fpIsPoint x
+                                       constrain $ x .> 0
+
+                                       maximize "max" x
+                                       minimize "min" x
+
+                                       pure sTrue
