@@ -30,6 +30,7 @@
 
 {-# LANGUAGE DeriveAnyClass      #-}
 {-# LANGUAGE DeriveDataTypeable  #-}
+{-# LANGUAGE OverloadedStrings   #-}
 {-# LANGUAGE StandaloneDeriving  #-}
 {-# LANGUAGE TemplateHaskell     #-}
 
@@ -70,7 +71,7 @@ puzzle = do n <- sInteger "N"
 
             let valid = validPick n
 
-            -- Declare three existential flowers. We declar these with
+            -- Declare three existential flowers. We declare these with
             -- _modelIgnore suffix, because we don't care different assignments
             -- to them to be a different model. See 'isNonModelVar' below.
             ef1 <- exists "ef1_modelIgnore"
@@ -85,7 +86,7 @@ puzzle = do n <- sInteger "N"
 
             -- Each color is represented:
             constrain $ valid ef1 ef2 ef3
-            constrain $ map col [ef1, ef2, ef3] .== map literal [Red, Yellow, Blue]
+            constrain $ map col [ef1, ef2, ef3] .== [sRed, sYellow, sBlue]
 
             -- Pick any three, at least one is Red
             constrain $ valid af1 af2 af3 .=> count Red    [af1, af2, af3] .>= 1
