@@ -58,9 +58,6 @@ data Color = Black | Blue | Green | Red
 -- | Make 'Color' a symbolic value.
 mkSymbolicEnumeration ''Color
 
--- | Give symbolic colors a name for convenience.
-type SColor = SBV Color
-
 -- | Use 8-bit words for button numbers, even though we only have 1 to 19.
 type Button  = Word8
 
@@ -68,13 +65,13 @@ type Button  = Word8
 type SButton = SBV Button
 
 -- | The grid is an array mapping each button to its color.
-type Grid = SFunArray Button Color
+type Grid = SArray Button Color
 
 -- | Given a button press, and the current grid, compute the next grid.
 -- If the button is "unpressable", i.e., if it is not one of the center
 -- buttons or it is currently colored black, we return the grid unchanged.
 next :: SButton -> Grid -> Grid
-next b g = ite (readArray g b .== literal Black) g
+next b g = ite (readArray g b .== sBlack) g
          $ ite (b .==  5)                        (rot [ 1,  2,  6, 10,  9,  4])
          $ ite (b .==  6)                        (rot [ 2,  3,  7, 11, 10,  5])
          $ ite (b .==  9)                        (rot [ 4,  5, 10, 14, 13,  8])
@@ -89,7 +86,7 @@ next b g = ite (readArray g b .== literal Black) g
 -- | Iteratively search at increasing depths of button-presses to see if we can
 -- transform from the initial board position to a final board position.
 search :: [Color] -> [Color] -> IO ()
-search initial final = runSMT $ do emptyGrid :: Grid <- newArray "emptyGrid" (Just (literal Black))
+search initial final = runSMT $ do emptyGrid :: Grid <- newArray "emptyGrid" (Just sBlack)
                                    let initGrid = foldr (\(i, c) a -> writeArray a (literal i) (literal c)) emptyGrid (zip [1..] initial)
                                    query $ loop (0 :: Int) initGrid []
 
@@ -101,27 +98,30 @@ search initial final = runSMT $ do emptyGrid :: Grid <- newArray "emptyGrid" (Ju
                             cs <- checkSat
 
                             case cs of
-                              Unk   -> error $ "Solver said Unknown, depth: " ++ show i
+                              Unk    -> error $ "Solver said Unknown, depth: " ++ show i
 
-                              Unsat -> do -- It didn't work out. Pop and try again with one more move:
-                                          pop 1
-                                          b <- freshVar ("press_" ++ show i)
-                                          constrain $ b `sElem` map literal [5, 6, 9, 10, 11, 14, 15]
-                                          loop (i+1) (next b g) (sofar ++ [b])
+                              DSat{} -> error $ "Solver returned a delta-satisfiable result, depth: " ++ show i
 
-                              Sat   -> do vs <- mapM getValue sofar
-                                          io $ putStrLn $ "Found: " ++ show vs
-                                          findOthers sofar vs
+                              Unsat  -> do -- It didn't work out. Pop and try again with one more move:
+                                           pop 1
+                                           b <- freshVar ("press_" ++ show i)
+                                           constrain $ b `sElem` map literal [5, 6, 9, 10, 11, 14, 15]
+                                           loop (i+1) (next b g) (sofar ++ [b])
+
+                              Sat    -> do vs <- mapM getValue sofar
+                                           io $ putStrLn $ "Found: " ++ show vs
+                                           findOthers sofar vs
 
         findOthers vs = go
                 where go curVals = do constrain $ sOr $ zipWith (\v c -> v ./= literal c) vs curVals
                                       cs <- checkSat
                                       case cs of
-                                       Unk   -> error "Unknown!"
-                                       Unsat -> io $ putStrLn "There are no more solutions."
-                                       Sat   -> do newVals <- mapM getValue vs
-                                                   io $ putStrLn $ "Found: " ++ show newVals
-                                                   go newVals
+                                       Unk    -> error "Unknown!"
+                                       DSat{} -> error "Delta-sat!"
+                                       Unsat  -> io $ putStrLn "There are no more solutions."
+                                       Sat    -> do newVals <- mapM getValue vs
+                                                    io $ putStrLn $ "Found: " ++ show newVals
+                                                    go newVals
 
 -- | A particular example run. We have:
 --
@@ -133,8 +133,8 @@ search initial final = runSMT $ do emptyGrid :: Grid <- newArray "emptyGrid" (Ju
 -- Searching at depth: 4
 -- Searching at depth: 5
 -- Searching at depth: 6
--- Found: [10,10,11,9,14,6]
 -- Found: [10,10,9,11,14,6]
+-- Found: [10,10,11,9,14,6]
 -- There are no more solutions.
 example :: IO ()
 example = search initBoard finalBoard
