@@ -40,13 +40,6 @@ data U2Member = Bono | Edge | Adam | Larry
 -- | Make 'U2Member' a symbolic value.
 mkSymbolicEnumeration ''U2Member
 
--- | Symbolic shorthand for a 'U2Member'
-type SU2Member = SBV U2Member
-
--- | Shorthands for symbolic versions of the members
-bono, edge, adam, larry :: SU2Member
-[bono, edge, adam, larry] = map literal [Bono, Edge, Adam, Larry]
-
 -- | Model time using 32 bits
 type Time  = Word32
 
@@ -62,10 +55,10 @@ crossTime Larry = 10
 
 -- | The symbolic variant.. The duplication is unfortunate.
 sCrossTime :: SU2Member -> STime
-sCrossTime m =   ite (m .== bono) (literal (crossTime Bono))
-               $ ite (m .== edge) (literal (crossTime Edge))
-               $ ite (m .== adam) (literal (crossTime Adam))
-                                  (literal (crossTime Larry)) -- Must be Larry
+sCrossTime m =   ite (m .== sBono) (literal (crossTime Bono))
+               $ ite (m .== sEdge) (literal (crossTime Edge))
+               $ ite (m .== sAdam) (literal (crossTime Adam))
+                                   (literal (crossTime Larry)) -- Must be Larry
 
 -- | Location of the flash
 data Location = Here | There
@@ -73,13 +66,6 @@ data Location = Here | There
 -- | Make 'Location' a symbolic value.
 mkSymbolicEnumeration ''Location
 
--- | Symbolic variant of 'Location'
-type SLocation = SBV Location
-
--- | Shorthands for symbolic versions of locations
-here, there :: SLocation
-[here, there]  = map literal [Here, There]
-
 -- | The status of the puzzle after each move
 --
 -- This type is equipped with an automatically derived 'Mergeable' instance
@@ -107,14 +93,14 @@ data Status = Status { time   :: STime       -- ^ elapsed time
 --                                    , lLarry = symbolicMerge f t (lLarry s1) (lLarry s2)
 --                                    }
 
--- | Start configuration, time elapsed is 0 and everybody is 'here'
+-- | Start configuration, time elapsed is 0 and everybody is here
 start :: Status
 start = Status { time   = 0
-               , flash  = here
-               , lBono  = here
-               , lEdge  = here
-               , lAdam  = here
-               , lLarry = here
+               , flash  = sHere
+               , lBono  = sHere
+               , lEdge  = sHere
+               , lAdam  = sHere
+               , lLarry = sHere
                }
 
 -- | A puzzle move is modeled as a state-transformer
@@ -136,23 +122,23 @@ peek = gets
 
 -- | Given an arbitrary member, return his location
 whereIs :: SU2Member -> Move SLocation
-whereIs p =  ite (p .== bono) (peek lBono)
-           $ ite (p .== edge) (peek lEdge)
-           $ ite (p .== adam) (peek lAdam)
-                              (peek lLarry)
+whereIs p =  ite (p .== sBono) (peek lBono)
+           $ ite (p .== sEdge) (peek lEdge)
+           $ ite (p .== sAdam) (peek lAdam)
+                               (peek lLarry)
 
 -- | Transferring the flash to the other side
 xferFlash :: Move ()
-xferFlash = modify $ \s -> s{flash = ite (flash s .== here) there here}
+xferFlash = modify $ \s -> s{flash = ite (flash s .== sHere) sThere sHere}
 
 -- | Transferring a person to the other side
 xferPerson :: SU2Member -> Move ()
 xferPerson p =  do ~[lb, le, la, ll] <- mapM peek [lBono, lEdge, lAdam, lLarry]
-                   let move l = ite (l .== here) there here
-                       lb' = ite (p .== bono)  (move lb) lb
-                       le' = ite (p .== edge)  (move le) le
-                       la' = ite (p .== adam)  (move la) la
-                       ll' = ite (p .== larry) (move ll) ll
+                   let move l = ite (l .== sHere) sThere sHere
+                       lb' = ite (p .== sBono)  (move lb) lb
+                       le' = ite (p .== sEdge)  (move le) le
+                       la' = ite (p .== sAdam)  (move la) la
+                       ll' = ite (p .== sLarry) (move ll) ll
                    modify $ \s -> s{lBono = lb', lEdge = le', lAdam = la', lLarry = ll'}
 
 -- | Increment the time, when only one person crosses
@@ -209,9 +195,9 @@ run = mapM step
 -- | Check if a given sequence of actions is valid, i.e., they must all
 -- cross the bridge according to the rules and in less than 17 seconds
 isValid :: Actions -> SBool
-isValid as = time end .<= 17 .&& sAll check as .&& zigZag (cycle [there, here]) (map flash states) .&& sAll (.== there) [lBono end, lEdge end, lAdam end, lLarry end]
-  where check (s, p1, p2) =   (sNot s .=> p1 .> p2)      -- for two person moves, ensure first person is "larger"
-                          .&& (s      .=> p2 .== bono)   -- for one person moves, ensure second person is always "bono"
+isValid as = time end .<= 17 .&& sAll check as .&& zigZag (cycle [sThere, sHere]) (map flash states) .&& sAll (.== sThere) [lBono end, lEdge end, lAdam end, lLarry end]
+  where check (s, p1, p2) =   (sNot s .=> p1 .> p2)       -- for two person moves, ensure first person is "larger"
+                          .&& (s      .=> p2 .== sBono)   -- for one person moves, ensure second person is always "bono"
         states = evalState (run as) start
         end = last states
         zigZag reqs locs = sAnd $ zipWith (.==) locs reqs
