@@ -78,9 +78,10 @@ exampleMathSAT = do
        -- To obtain the interpolant, we run a query
        query $ do cs <- checkSat
                   case cs of
-                    Unsat -> getInterpolantMathSAT ["A"]
-                    Sat   -> error "Unexpected sat result!"
-                    Unk   -> error "Unexpected unknown result!"
+                    Unsat  -> getInterpolantMathSAT ["A"]
+                    DSat{} -> error "Unexpected delta-sat result!"
+                    Sat    -> error "Unexpected sat result!"
+                    Unk    -> error "Unexpected unknown result!"
 
 -- | Z3 example. Compute the interpolant for formulas @y = 2x@ and @y = 2z+1@.
 --
@@ -100,7 +101,7 @@ exampleMathSAT = do
 --
 -- Notice that the only symbol is @y@, as required. To establish that this is
 -- indeed an interpolant, we should establish that when @y@ is even, this formula
--- is @True@; and if @y@ is odd, then then it should be @False@. You can argue
+-- is @True@; and if @y@ is odd, then it should be @False@. You can argue
 -- mathematically that this indeed the case, but let's just use SBV to prove these:
 --
 -- >>> prove $ \y -> (y `sMod` 2 .== 0) .=> ((y .== 0) .|| (y .== 2 * (y `sDiv` (2::SInteger))))
