@@ -38,7 +38,7 @@ solveCrossword rowRegExps colRegExps = runSMT $ do
 
         rows <- mapM mkRow rowRegExps
 
-        -- constrain colums
+        -- constrain columns
         let mkCol colRegExp = do col <- free_
                                  constrain $ col `R.match` colRegExp
                                  constrain $ S.length col .== literal numRows
@@ -55,9 +55,10 @@ solveCrossword rowRegExps colRegExps = runSMT $ do
         -- Now query to extract the solution
         query $ do cs <- checkSat
                    case cs of
-                     Unk   -> error "Solver returned unknown!"
-                     Unsat -> error "There are no solutions to this puzzle!"
-                     Sat   -> mapM getValue rows
+                     Unk    -> error "Solver returned unknown!"
+                     DSat{} -> error "Solver returned delta-sat!"
+                     Unsat  -> error "There are no solutions to this puzzle!"
+                     Sat    -> mapM getValue rows
 
 -- | Solve <http://regexcrossword.com/challenges/intermediate/puzzles/1>
 --
