@@ -22,6 +22,6 @@ import BenchSuite.Bench.Bench as S
 
 -- benchmark suite
 benchmarks :: Runner
-benchmarks = S.run "Counts" countPgm `using` runner allSatWith
- where countPgm = forAll_ puzzle' >>= return -- avoiding 'output' here again
-       puzzle' d0 d1 d2 d3 d4 d5 d6 d7 d8 d9 = puzzle [d0, d1, d2, d3, d4, d5, d6, d7, d8, d9]
+benchmarks = rGroup [ S.run "Counts" countPgm ]
+ where countPgm = puzzle `fmap` mkExistVars 10
+       -- puzzle' d0 d1 d2 d3 d4 d5 d6 d7 d8 d9 = puzzle [d0, d1, d2, d3, d4, d5, d6, d7, d8, d9]
