@@ -10,11 +10,12 @@
 -- Bench suite for Documentation.SBV.Examples.Puzzles.Garden
 -----------------------------------------------------------------------------
 
-{-# OPTIONS_GHC -Wall -Werror #-}
+{-# OPTIONS_GHC -Wall -Werror  #-}
+{-# LANGUAGE OverloadedStrings #-}
 
 module BenchSuite.Puzzles.Garden(benchmarks) where
 
-import Data.List (isSuffixOf)
+import Data.Text (isSuffixOf)
 
 import Documentation.SBV.Examples.Puzzles.Garden
 
@@ -24,5 +25,5 @@ import BenchSuite.Bench.Bench as S
 
 -- benchmark suite
 benchmarks :: Runner
-benchmarks = S.runWith s "Garden" puzzle `using` runner allSatWith
+benchmarks = rGroup [ S.runWith s "Garden" puzzle `using` runner allSatWith ]
   where s = z3{satTrackUFs = False, isNonModelVar = ("_modelIgnore" `isSuffixOf`)}
