@@ -23,11 +23,11 @@ import BenchSuite.Bench.Bench as S
 -- benchmark suite
 benchmarks :: Runner
 benchmarks = rGroup
-  [ S.run "U2Bridge_cnt1" (count 1) `using` runner allSatWith
-  , S.run "U2Bridge_cnt2" (count 2) `using` runner allSatWith
-  , S.run "U2Bridge_cnt3" (count 3) `using` runner allSatWith
-  , S.run "U2Bridge_cnt4" (count 4) `using` runner allSatWith
-  , S.run "U2Bridge_cnt6" (count 6) `using` runner allSatWith
+  [ S.run "U2Bridge_cnt1" (count 1) `using` runner satWith
+  , S.run "U2Bridge_cnt2" (count 2) `using` runner satWith
+  , S.run "U2Bridge_cnt3" (count 3) `using` runner satWith
+  , S.run "U2Bridge_cnt4" (count 4) `using` runner satWith
+  , S.run "U2Bridge_cnt6" (count 6) `using` runner satWith
   ]
   where
     act     = do b <- exists_; p1 <- exists_; p2 <- exists_; return (b, p1, p2)
