@@ -18,8 +18,6 @@
 
 module TestSuite.Basics.ArithNoSolver(tests) where
 
-import qualified Data.Numbers.CrackNum as CN (wordToFloat, wordToDouble, floatToWord, doubleToWord)
-
 import Data.SBV.Internals
 import Utils.SBVTestFramework
 
@@ -400,11 +398,11 @@ genFloats = bTests ++ uTests ++ fpTests1 ++ fpTests2 ++ converts
 
                  -- NB. We don't constant fold float/double conversions, so we skip these
                  --
-                 ++  [cvtTest  ("reinterp_Word32_Float",  show x, sWord32AsSFloat  (literal x), literal (CN.wordToFloat  x)) | x <- w32s]
-                 ++  [cvtTest  ("reinterp_Word64_Double", show x, sWord64AsSDouble (literal x), literal (CN.wordToDouble x)) | x <- w64s]
+                 ++  [cvtTest  ("reinterp_Word32_Float",  show x, sWord32AsSFloat  (literal x), literal (wordToFloat  x)) | x <- w32s]
+                 ++  [cvtTest  ("reinterp_Word64_Double", show x, sWord64AsSDouble (literal x), literal (wordToDouble x)) | x <- w64s]
 
-                 ++  [cvtTestI ("reinterp_Float_Word32",  show x, sFloatAsSWord32  (literal x), CN.floatToWord x)  | x <- fs, not (isNaN x)] -- Not unique for NaN
-                 ++  [cvtTestI ("reinterp_Double_Word64", show x, sDoubleAsSWord64 (literal x), CN.doubleToWord x) | x <- ds, not (isNaN x)] -- Not unique for NaN
+                 ++  [cvtTestI ("reinterp_Float_Word32",  show x, sFloatAsSWord32  (literal x), floatToWord x)  | x <- fs, not (isNaN x)] -- Not unique for NaN
+                 ++  [cvtTestI ("reinterp_Double_Word64", show x, sDoubleAsSWord64 (literal x), doubleToWord x) | x <- ds, not (isNaN x)] -- Not unique for NaN
 
         floatRun1   nm f g cmb = [(nm, cmb (x,    f x,   extract (g                     (literal x))))             | x <- fs]
         doubleRun1  nm f g cmb = [(nm, cmb (x,    f x,   extract (g                     (literal x))))             | x <- ds]
@@ -457,31 +455,31 @@ genFloats = bTests ++ uTests ++ fpTests1 ++ fpTests2 ++ converts
                      ]
 
 genChars :: [TestTree]
-genChars = map mkTest $  [("ord",           show c, check SC.ord           cord            c) | c <- cs]
-                      ++ [("toLower",       show c, check SC.toLower       C.toLower       c) | c <- cs]
-                      ++ [("toUpper",       show c, check SC.toUpper       C.toUpper       c) | c <- cs, toUpperExceptions c]
-                      ++ [("digitToInt",    show c, check SC.digitToInt    dig2Int         c) | c <- cs, digitToIntRange c]
-                      ++ [("intToDigit",    show c, check SC.intToDigit    int2Dig         c) | c <- [0 .. 15]]
-                      ++ [("isControl",     show c, check SC.isControl     C.isControl     c) | c <- cs]
-                      ++ [("isSpace",       show c, check SC.isSpace       C.isSpace       c) | c <- cs]
-                      ++ [("isLower",       show c, check SC.isLower       C.isLower       c) | c <- cs]
-                      ++ [("isUpper",       show c, check SC.isUpper       C.isUpper       c) | c <- cs]
-                      ++ [("isAlpha",       show c, check SC.isAlpha       C.isAlpha       c) | c <- cs]
-                      ++ [("isAlphaNum",    show c, check SC.isAlphaNum    C.isAlphaNum    c) | c <- cs]
-                      ++ [("isPrint",       show c, check SC.isPrint       C.isPrint       c) | c <- cs]
-                      ++ [("isDigit",       show c, check SC.isDigit       C.isDigit       c) | c <- cs]
-                      ++ [("isOctDigit",    show c, check SC.isOctDigit    C.isOctDigit    c) | c <- cs]
-                      ++ [("isHexDigit",    show c, check SC.isHexDigit    C.isHexDigit    c) | c <- cs]
-                      ++ [("isLetter",      show c, check SC.isLetter      C.isLetter      c) | c <- cs]
-                      ++ [("isMark",        show c, check SC.isMark        C.isMark        c) | c <- cs]
-                      ++ [("isNumber",      show c, check SC.isNumber      C.isNumber      c) | c <- cs]
-                      ++ [("isPunctuation", show c, check SC.isPunctuation C.isPunctuation c) | c <- cs]
-                      ++ [("isSymbol",      show c, check SC.isSymbol      C.isSymbol      c) | c <- cs]
-                      ++ [("isSeparator",   show c, check SC.isSeparator   C.isSeparator   c) | c <- cs]
-                      ++ [("isAscii",       show c, check SC.isAscii       C.isAscii       c) | c <- cs]
-                      ++ [("isLatin1",      show c, check SC.isLatin1      C.isLatin1      c) | c <- cs]
-                      ++ [("isAsciiUpper",  show c, check SC.isAsciiUpper  C.isAsciiUpper  c) | c <- cs]
-                      ++ [("isAsciiLower",  show c, check SC.isAsciiLower  C.isAsciiLower  c) | c <- cs]
+genChars = map mkTest $  [("ord",           show c, check SC.ord             cord            c) | c <- cs]
+                      ++ [("toLower",       show c, check SC.toLowerL1       C.toLower       c) | c <- cs]
+                      ++ [("toUpper",       show c, check SC.toUpperL1       C.toUpper       c) | c <- cs, toUpperExceptions c]
+                      ++ [("digitToInt",    show c, check SC.digitToInt      dig2Int         c) | c <- cs, digitToIntRange c]
+                      ++ [("intToDigit",    show c, check SC.intToDigit      int2Dig         c) | c <- [0 .. 15]]
+                      ++ [("isControl",     show c, check SC.isControlL1     C.isControl     c) | c <- cs]
+                      ++ [("isSpace",       show c, check SC.isSpaceL1       C.isSpace       c) | c <- cs]
+                      ++ [("isLower",       show c, check SC.isLowerL1       C.isLower       c) | c <- cs]
+                      ++ [("isUpper",       show c, check SC.isUpperL1       C.isUpper       c) | c <- cs]
+                      ++ [("isAlpha",       show c, check SC.isAlphaL1       C.isAlpha       c) | c <- cs]
+                      ++ [("isAlphaNum",    show c, check SC.isAlphaNumL1    C.isAlphaNum    c) | c <- cs]
+                      ++ [("isPrint",       show c, check SC.isPrintL1       C.isPrint       c) | c <- cs]
+                      ++ [("isDigit",       show c, check SC.isDigit         C.isDigit       c) | c <- cs]
+                      ++ [("isOctDigit",    show c, check SC.isOctDigit      C.isOctDigit    c) | c <- cs]
+                      ++ [("isHexDigit",    show c, check SC.isHexDigit      C.isHexDigit    c) | c <- cs]
+                      ++ [("isLetter",      show c, check SC.isLetterL1      C.isLetter      c) | c <- cs]
+                      ++ [("isMark",        show c, check SC.isMarkL1        C.isMark        c) | c <- cs]
+                      ++ [("isNumber",      show c, check SC.isNumberL1      C.isNumber      c) | c <- cs]
+                      ++ [("isPunctuation", show c, check SC.isPunctuationL1 C.isPunctuation c) | c <- cs]
+                      ++ [("isSymbol",      show c, check SC.isSymbolL1      C.isSymbol      c) | c <- cs]
+                      ++ [("isSeparator",   show c, check SC.isSeparatorL1   C.isSeparator   c) | c <- cs]
+                      ++ [("isAscii",       show c, check SC.isAscii         C.isAscii       c) | c <- cs]
+                      ++ [("isLatin1",      show c, check SC.isLatin1        C.isLatin1      c) | c <- cs]
+                      ++ [("isAsciiUpper",  show c, check SC.isAsciiUpper    C.isAsciiUpper  c) | c <- cs]
+                      ++ [("isAsciiLower",  show c, check SC.isAsciiLower    C.isAsciiLower  c) | c <- cs]
   where toUpperExceptions = (`notElem` "\181\255")
         digitToIntRange   = (`elem` "0123456789abcdefABCDEF")
         cord :: Char -> Integer
@@ -736,7 +734,7 @@ ds = xs ++ map (* (-1)) (filter (not . isNaN) xs) -- -nan is the same as nan
 sds :: [SDouble]
 sds = map literal ds
 
--- Currently we test over all latin-1 characters. But when Unicode comes along, we'll have to be more selective.
+-- Currently we test over all latin-1 characters. Maybe we should add some unicode here. Oh well.
 cs :: String
 cs = map C.chr [0..255]
 
