@@ -21,6 +21,7 @@ import Utils.SBVTestFramework
 
 import Data.SBV.String ((.!!), (.++))
 import qualified Data.SBV.String as S
+import qualified Data.SBV.Char   as SC
 import qualified Data.SBV.RegExp as R
 
 import Control.Monad (unless)
@@ -60,9 +61,10 @@ checkWith cfg props csExpected = runSMTWith cfg{verbose=True} $ do
         query $ do cs <- checkSat
                    unless (cs == csExpected) $
                      case cs of
-                       Unsat -> error "Failed! Expected Sat, got UNSAT"
-                       Sat   -> getModel         >>= \r -> error $ "Failed! Expected Unsat, got SAT:\n" ++ show (SatResult (Satisfiable cfg r))
-                       Unk   -> getUnknownReason >>= \r -> error $ "Failed! Expected Unsat, got UNK:\n" ++ show r
+                       Unsat  -> error "Failed! Expected Sat, got UNSAT"
+                       DSat{} -> error "Failed! Expected Sat, got delta-sat"
+                       Sat    -> getModel         >>= \r -> error $ "Failed! Expected Unsat, got SAT:\n" ++ show (SatResult (Satisfiable cfg r))
+                       Unk    -> getUnknownReason >>= \r -> error $ "Failed! Expected Unsat, got UNK:\n" ++ show r
 
 strConcatSat :: Symbolic ()
 strConcatSat = constrain $ "abc" .++ "def" .== "abcdef"
@@ -175,9 +177,12 @@ strExamples13 = do
    constrain $ s .== "13"
    constrain $ sNot $ S.strToNat s .== 13
 
--- Generate all length one strings, to enumerate all and making sure we can parse correctly
+-- Generate all length one strings consisting of letters A-Z, to enumerate all and making sure we can parse correctly
 strExamples14 :: IO Bool
 strExamples14 = do m <- allSat $ do s <- sString "s"
+                                    let c = SC.ord (S.head s)
+                                    constrain $ c .>= SC.ord (literal 'A')
+                                    constrain $ c .<= SC.ord (literal 'Z')
                                     return $ S.length s .== 1
                    let dicts = getModelDictionaries m
 
@@ -185,5 +190,5 @@ strExamples14 = do m <- allSat $ do s <- sString "s"
                        vals = map C.ord $ concat $ sort $ map (fromCV . snd) (concatMap M.assocs dicts)
 
                    case length dicts of
-                     256 -> return $ vals == [0 .. 255]
+                     26 -> return $ vals == map C.ord ['A' .. 'Z']
                      _   -> return False
