@@ -32,6 +32,9 @@ import Utils.SBVTestFramework
 data E = A | B | C
 mkSymbolicEnumeration ''E
 
+__unused :: SE
+__unused = error "stop GHC from complaining unused names" sA sB sC
+
 -- Test suite
 tests :: TestTree
 tests = testGroup "Basics.Tuple" [
@@ -102,7 +105,7 @@ enum = do
    constrain $ L.length vTup1 .== 3
 
    case untuple (vTup1 .!! 2)  of
-     (e, b) -> do constrain $ e .== literal C
+     (e, b) -> do constrain $ e .== sC
                   constrain $ L.length b .== 6
                   constrain $ b .!! 4 .== sTrue
 
