@@ -1,17 +1,19 @@
 -----------------------------------------------------------------------------
 -- |
--- Module      :  TestSuite.BitPrecise.Legato
--- Copyright   :  (c) Levent Erkok
--- License     :  BSD3
--- Maintainer  :  erkokl@gmail.com
--- Stability   :  experimental
+-- Module    : TestSuite.BitPrecise.Legato
+-- Copyright : (c) Levent Erkok
+-- License   : BSD3
+-- Maintainer: erkokl@gmail.com
+-- Stability : experimental
 --
 -- Test suite for Documentation.SBV.Examples.BitPrecise.Legato
 -----------------------------------------------------------------------------
 
+{-# OPTIONS_GHC -Wall -Werror #-}
+
 module TestSuite.BitPrecise.Legato(tests) where
 
-import Data.SBV.Internals
+import Data.SBV.Internals hiding (free, output)
 import Documentation.SBV.Examples.BitPrecise.Legato
 
 import Utils.SBVTestFramework
@@ -31,10 +33,13 @@ tests = testGroup "BitPrecise.Legato" [
                        flagC <- free "flagC"
                        flagZ <- free "flagZ"
                        output $ legatoIsCorrect (x, y, lo, regX, regA, flagC, flagZ)
-       legatoC = snd <$> compileToC' "legatoMult" (do
+
+       thd (_, _, r) = r
+
+       legatoC = thd <$> compileToC' "legatoMult" (do
                     cgSetDriverValues [87, 92]
                     x <- cgInput "x"
                     y <- cgInput "y"
-                    let (hi, lo) = runLegato (initMachine (x, y, 0, 0, 0, false, false))
+                    let (hi, lo) = runLegato (initMachine (x, y, 0, 0, 0, sFalse, sFalse))
                     cgOutput "hi" hi
                     cgOutput "lo" lo)
