@@ -1,14 +1,16 @@
 -----------------------------------------------------------------------------
 -- |
--- Module      :  TestSuite.CodeGeneration.CRC_USB5
--- Copyright   :  (c) Levent Erkok
--- License     :  BSD3
--- Maintainer  :  erkokl@gmail.com
--- Stability   :  experimental
+-- Module    : TestSuite.CodeGeneration.CRC_USB5
+-- Copyright : (c) Levent Erkok
+-- License   : BSD3
+-- Maintainer: erkokl@gmail.com
+-- Stability : experimental
 --
 -- Test suite for Documentation.SBV.Examples.CodeGeneration.CRC_USB5
 -----------------------------------------------------------------------------
 
+{-# OPTIONS_GHC -Wall -Werror #-}
+
 module TestSuite.CodeGeneration.CRC_USB5(tests) where
 
 import Data.SBV.Internals
@@ -22,7 +24,8 @@ tests = testGroup "CRC.CodeGen" [
    goldenVsStringShow "crcUSB5_1" $ genC crcUSB
  , goldenVsStringShow "crcUSB5_2" $ genC crcUSB'
  ]
- where genC f = snd <$> compileToC' "crcUSB5" (do
+ where thd (_, _, r) = r
+       genC f = thd <$> compileToC' "crcUSB5" (do
                    cgSetDriverValues [0xFEDC]
                    msg <- cgInput "msg"
                    cgReturn $ f msg)
